import { UnionOmit, UIStatus, UIColorAttrs } from './tsutils';
export declare type RGB = [number, number, number];
export declare type RGBA = [number, number, number, number];
export declare type HSLA = [number, number, number, number];
export declare type HSVA = [number, number, number, number];
export declare type HSL = [number, number, number];
export declare type HSV = [number, number, number];
/**
 * @param h 360
 * @param s 100
 * @param l 100
 * @returns [h, s, v] 360, 100, 100
 */
export declare function hsl2hsv(h: number, s: number, l: number): HSV;
/**
 * @param h 360
 * @param s 100
 * @param v 100
 * @returns [h, s, l] 360, 100, 100
 */
export declare function hsv2hsl(h: number, s: number, v: number): HSL;
/**
 * @param h 360
 * @param s 100
 * @param v 100
 * @returns [r, g, b] 255, 255, 255
 */
export declare function hsv2rgb(h: number, s: number, v: number): RGB;
/**
 * @param r 255
 * @param g 255
 * @param b 255
 * @returns [360, 100, 100]
 */
export declare function rgb2hsv(r: number, g: number, b: number): HSV;
/**
 * @param r 255
 * @param g 255
 * @param b 255
 * @returns [360, 100, 100]
 */
export declare function rgb2hsl(r: number, g: number, b: number): HSL;
/**
 * @param h 360
 * @param s 100
 * @param l 100
 * @returns [255, 255, 255]
 */
export declare function hsl2rgb(h: number, s: number, l: number): RGB;
export declare function setColorAlpha(color: string, alpha?: number): string;
export declare function rgba(color: string): RGBA;
export declare function roundChannel(value: number | string): number;
export declare function normalizeAlpha(alphaValue: number): number;
export declare function roundAlpha(value: number | string): number;
export declare function stringifyRgba(r: number, g: number, b: number, a: number): string;
export declare function stringifyHexa(r: string, g: string, b: string, a: string): string;
export declare function compositeChannel(v1: number, a1: number, v2: number, a2: number, a: number): number;
export declare function composite(background: string | RGB | RGBA, overlay: string | RGB | RGBA): string;
export declare function createHoverColor(rgb: string, mix?: number | RGBA): string;
export declare function createActiveColor(rgb: string, mix?: number | RGBA): string;
export declare function createDisabledColor(rgb: string, mix?: number | RGBA): string;
export declare function useColorFactory<S extends Partial<UIColorAttrs>>(inputColor: S, customMix?: Partial<Record<UnionOmit<UIStatus, 'default'>, number | RGBA>>): Record<UIStatus, S>;
