import { Fragment, Comment } from 'vue';
import { omit } from 'lodash-es';
/**
 *
 * @param vNodes flatten target
 * @param identificationKey filter key
 * @param mode When mode is true, every elements in vNodes which iKey === identificationKey will be filtered
 * @param result
 * @returns
 */
export function flatten(vNodes, identificationKey, mode = false, result = [], isInfinity = false) {
    var _a, _b, _c;
    if (!vNodes)
        return result;
    const auth = (key) => {
        return key ? (Array.isArray(identificationKey) ? (mode ? !identificationKey.includes(key) : identificationKey.includes(key)) : mode ? key !== identificationKey : key === identificationKey) : false;
    };
    const filterVNodes = identificationKey
        ? vNodes.filter(vNode => {
            const { iKey } = vNode.type;
            return auth(iKey) || vNode.type === Fragment || isInfinity;
        })
        : vNodes;
    for (const vNode of filterVNodes) {
        const children = ((_c = (isInfinity ? (_b = (_a = vNode.children) === null || _a === void 0 ? void 0 : _a.default) === null || _b === void 0 ? void 0 : _b.call(_a) : vNode.children)) !== null && _c !== void 0 ? _c : []);
        const { iKey } = vNode.type;
        if (isInfinity && auth(iKey)) {
            result.push(vNode);
        }
        else if (vNode.type === Fragment || (isInfinity && children.length > 0)) {
            flatten(children, identificationKey, mode, result, isInfinity);
        }
        else if (vNode.type !== Comment) {
            result.push(vNode);
        }
    }
    return result;
}
export function flattenWithOptions(options, mode = false, result = []) {
    var _a;
    const { slots, name, key, infinity } = options;
    if (!(slots === null || slots === void 0 ? void 0 : slots[name || 'default']))
        return result;
    return flatten((_a = slots[name || 'default']) === null || _a === void 0 ? void 0 : _a.call(slots), key, mode, result, !!infinity);
}
export function flattenTree(tree, key, result = []) {
    tree.forEach(item => {
        result.push(omit(item, key));
        if (item[key]) {
            flattenTree(item[key], key, result);
        }
    });
    return result;
}
