import { PropType } from 'vue';
import { UIStatus, UISize, UIColorAttrs } from '../_utils_';
export interface ButtonSizeSet {
    height: string;
    padding: string;
    fontSize: string;
    iconSize: string;
    iconMargin: string;
}
export declare const buttonIKey: unique symbol;
export declare type ButtonSize = UISize;
export declare type ButtonStatus = UIStatus;
export declare type ButtonType = 'default' | 'primary' | 'success' | 'warning' | 'danger' | 'custom';
export declare type ButtonRender = 'normal' | 'text' | 'link';
export declare type ButtonColorSet = Pick<UIColorAttrs, 'color' | 'borderColor' | 'backgroundColor'>;
export declare type ButtonSizeMap = Record<ButtonSize, ButtonSizeSet>;
export interface ButtonExposeInstance {
    el: HTMLButtonElement;
}
export interface ButtonProps {
    type?: ButtonType;
    size?: ButtonSize;
    disabled?: boolean;
    ghost?: boolean;
    dashed?: boolean;
    render?: ButtonRender;
    round?: boolean;
    circle?: boolean;
    block?: boolean;
    loading?: boolean;
    iconRight?: boolean;
    color?: string;
    textColor?: string;
    borderColor?: string;
    colorSet?: Partial<Record<ButtonStatus, string>>;
    textColorSet: Partial<Record<ButtonStatus, string>>;
    borderColorSet: Partial<Record<ButtonStatus, string>>;
}
export interface ButtonGroupProps {
    type?: ButtonType;
    size?: ButtonSize;
    disabled?: boolean;
    ghost?: boolean;
    dashed?: boolean;
    render?: ButtonRender;
    vertical?: boolean;
}
export declare const buttonProps: {
    type: {
        type: PropType<ButtonType | undefined>;
        default: string;
    };
    size: {
        type: PropType<UISize | undefined>;
        default: string;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    ghost: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    dashed: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    render: {
        type: PropType<ButtonRender | undefined>;
        default: string;
    };
    round: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    circle: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    loading: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    block: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    iconRight: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    color: {
        type: PropType<string | undefined>;
        default: undefined;
    };
    textColor: {
        type: PropType<string | undefined>;
        default: undefined;
    };
    borderColor: {
        type: PropType<string | undefined>;
        default: undefined;
    };
    colorSet: {
        type: PropType<Partial<Record<UIStatus, string>> | undefined>;
        default: undefined;
    };
    textColorSet: {
        type: PropType<Partial<Record<UIStatus, string>>>;
        default: undefined;
    };
    borderColorSet: {
        type: PropType<Partial<Record<UIStatus, string>>>;
        default: undefined;
    };
};
export declare const buttonGroupProps: {
    type: {
        type: PropType<ButtonType | undefined>;
        default: string;
    };
    size: {
        type: PropType<UISize | undefined>;
        default: string;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    ghost: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    dashed: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    render: {
        type: PropType<ButtonRender | undefined>;
        default: string;
    };
    vertical: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
