import { defineComponent, ref, createVNode, renderSlot, toRefs, computed, inject, mergeProps, onMounted } from 'vue';
import { or, and, not, isDefined } from '@vueuse/core';
import { useThemeRegister, createKey, createElementVNode } from '../_utils_';
import { mainCssr, lightCssr, darkCssr } from './styles';
import { IconCheckMark, IconIndeterminateMark } from './icon';
import { checkboxIKey, checkboxGroupInjectionKey, checkboxProps } from './interface';
export default defineComponent({
    name: 'Checkbox',
    iKey: checkboxIKey,
    props: checkboxProps,
    emits: ['update:value'],
    setup(props, { slots, attrs, emit }) {
        var _a;
        onMounted(() => {
            useThemeRegister({
                key: 'Checkbox',
                main: mainCssr,
                light: lightCssr,
                dark: darkCssr
            });
        });
        const key = createKey('checkbox');
        const { value: valueVM, label, size, checkedValue, uncheckedValue, disabled, indeterminate, checkedColor } = toRefs(props);
        const { groupValue, groupCheckedColor, groupDisabled, updateGroupValue, BusSelectAll, BusMaxControl } = (_a = inject(checkboxGroupInjectionKey, null)) !== null && _a !== void 0 ? _a : {};
        const checkboxElRef = ref();
        const internalValue = ref(false);
        const internalDisabled = ref(false);
        const scaleRatio = computed(() => {
            switch (size.value) {
                case 'small':
                    return 0.8;
                case 'medium':
                    return 0.9;
                case 'large':
                    return 1;
            }
        });
        const mergedValue = isDefined(valueVM) ? valueVM : internalValue;
        const mergedDisabled = or(groupDisabled, disabled, internalDisabled);
        const mergedChecked = computed(() => {
            if (groupValue === null || groupValue === void 0 ? void 0 : groupValue.value) {
                return groupValue.value.indexOf(mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) > -1;
            }
            return (mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) === checkedValue.value;
        });
        const mergedCheckedColor = and(groupCheckedColor, not(checkedColor)).value ? groupCheckedColor : checkedColor;
        const cssVars = computed(() => {
            var _a, _b;
            return {
                '--checkbox-checked-color': (_a = mergedCheckedColor === null || mergedCheckedColor === void 0 ? void 0 : mergedCheckedColor.value) !== null && _a !== void 0 ? _a : '#10b981',
                '--checkbox-hover-color': ((_b = mergedCheckedColor === null || mergedCheckedColor === void 0 ? void 0 : mergedCheckedColor.value) !== null && _b !== void 0 ? _b : '#10b981') + '0f',
                '--checkbox-scale-size': `scale(${scaleRatio.value})`
            };
        });
        const callUpdateValue = (value) => {
            if (valueVM.value === undefined) {
                mergedValue.value = value;
            }
            emit('update:value', value);
        };
        const updateInternalDisabled = (reachMax) => {
            if (reachMax) {
                !mergedChecked.value && (internalDisabled.value = true);
            }
            else {
                internalDisabled.value && (internalDisabled.value = false);
            }
        };
        const handleChange = () => {
            if (updateGroupValue) {
                updateGroupValue(mergedValue.value);
            }
            else {
                callUpdateValue(mergedChecked.value ? uncheckedValue.value : checkedValue.value);
            }
        };
        if (BusMaxControl && BusSelectAll) {
            BusMaxControl.on(updateInternalDisabled);
            BusSelectAll.on((selectDisabled) => {
                if (!mergedChecked.value) {
                    if (!selectDisabled) {
                        !mergedDisabled.value && (updateGroupValue === null || updateGroupValue === void 0 ? void 0 : updateGroupValue(mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value, false));
                    }
                    else {
                        updateGroupValue === null || updateGroupValue === void 0 ? void 0 : updateGroupValue(mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value, false);
                    }
                }
            });
        }
        const labelVNode = computed(() => {
            if (slots.default) {
                return createVNode('span', null, [renderSlot(slots, 'default')]);
            }
            else if (label === null || label === void 0 ? void 0 : label.value) {
                return createVNode('span', null, [label.value]);
            }
            else {
                return null;
            }
        });
        return () => {
            const mergedProps = mergeProps({
                ref: checkboxElRef,
                class: ['mc-checkbox', { 'mc-checkbox--checked': mergedChecked.value, 'mc-checkbox--disabled': mergedDisabled.value }],
                style: cssVars.value
            }, attrs);
            return createElementVNode('div', mergedProps, [
                createElementVNode('input', { class: 'mc-checkbox__input', value: mergedValue.value, id: key, type: 'checkbox', onChange: handleChange, checked: mergedChecked.value, disabled: mergedDisabled.value }, null, 8 /* PatchFlags.PROPS */, [
                    'value',
                    'checked',
                    'disabled'
                ]),
                createElementVNode('label', { class: 'mc-checkbox__label', for: key }, [
                    createElementVNode('span', {
                        style: {
                            background: mergedDisabled.value ? 'rgba(0, 0, 0, 0.02)' : '',
                            borderColor: mergedDisabled.value ? '#cccfdb' : ''
                        }
                    }, [createVNode(indeterminate.value ? IconIndeterminateMark : IconCheckMark)], 4 /* PatchFlags.STYLE */),
                    labelVNode.value
                ])
            ], 16 /* PatchFlags.FULL_PROPS */);
        };
    }
});
