import { defineComponent, provide, toRefs, ref, createVNode, renderSlot, nextTick, onUnmounted, watch, computed } from 'vue';
import { useEventBus } from '@vueuse/core';
import McCheckbox from './Checkbox';
import { flatten } from '../_utils_';
import { checkboxGroupInjectionKey, checkboxIKey, checkboxGroupPros } from './interface';
export default defineComponent({
    name: 'CheckboxGroup',
    props: checkboxGroupPros,
    emits: ['update:value'],
    setup(props, { slots, emit, expose }) {
        const { value: valueVM, options, checkedColor, max, disabled } = toRefs(props);
        const internalValue = ref([]);
        const checkboxGroupElRef = ref();
        const checkboxCount = computed(() => {
            var _a, _b;
            const slotsCount = slots.default ? flatten(slots.default(), checkboxIKey).length : 0;
            const optionsCount = (_b = (_a = options === null || options === void 0 ? void 0 : options.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
            return slotsCount + optionsCount;
        });
        const checkedCount = computed(() => {
            var _a, _b;
            return (_b = (_a = mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
        });
        const status = computed(() => {
            return {
                all: checkedCount.value === checkboxCount.value,
                indeterminate: checkedCount.value > 0 && checkedCount.value < checkboxCount.value
            };
        });
        const mergedValue = valueVM.value ? valueVM : internalValue;
        const callUpdateValue = (value) => {
            emit('update:value', mergedValue.value, value);
        };
        // due with max props, some logic...
        if ((mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) && (max === null || max === void 0 ? void 0 : max.value)) {
            watch([checkboxCount, checkedCount, max], () => {
                void nextTick(() => {
                    var _a;
                    if (((_a = mergedValue.value) === null || _a === void 0 ? void 0 : _a.length) === max.value) {
                        BusMaxControl.emit(true);
                    }
                    else {
                        BusMaxControl.emit(false);
                    }
                });
            }, {
                immediate: true
            });
        }
        const updateGroupValue = (value, call = true) => {
            if (mergedValue.value) {
                const index = mergedValue.value.indexOf(value !== null && value !== void 0 ? value : '');
                if (index === -1) {
                    mergedValue.value.push(value !== null && value !== void 0 ? value : '');
                }
                else {
                    mergedValue.value.splice(index, 1);
                }
                call && callUpdateValue(value);
            }
            else {
                call && callUpdateValue(value);
            }
        };
        const SelectAllEventBusKey = Symbol();
        const MaxControlEventBusKey = Symbol();
        const BusSelectAll = useEventBus(SelectAllEventBusKey);
        const BusMaxControl = useEventBus(MaxControlEventBusKey);
        provide(checkboxGroupInjectionKey, {
            groupValue: mergedValue,
            groupCheckedColor: checkedColor,
            groupDisabled: disabled,
            updateGroupValue,
            BusSelectAll,
            BusMaxControl
        });
        onUnmounted(() => {
            BusMaxControl.reset();
            BusSelectAll.reset();
        });
        expose({
            selectAll(selectDisabled = true) {
                BusSelectAll.emit(selectDisabled);
                (mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) && callUpdateValue();
            },
            clear() {
                if (mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) {
                    mergedValue.value.length = 0;
                    callUpdateValue();
                }
            },
            status,
            el: checkboxGroupElRef
        });
        return () => {
            var _a, _b;
            return createVNode('div', {
                ref: checkboxGroupElRef,
                class: 'mc-checkbox-group'
            }, [
                ...((_b = (_a = options.value) === null || _a === void 0 ? void 0 : _a.map(option => {
                    const { value, label, disabled } = option;
                    if (!label || typeof label === 'string') {
                        return createVNode(McCheckbox, { value, label, disabled });
                    }
                    else {
                        return createVNode(McCheckbox, { value, disabled }, { default: label });
                    }
                })) !== null && _b !== void 0 ? _b : []),
                renderSlot(slots, 'default')
            ]);
        };
    }
});
