import { createGlobalState, useStorage, createEventHook } from '@vueuse/core';
const useGlobalLanguageState = createGlobalState(() => useStorage('meetcode-ui-language-local-storage', 'zh-CN'));
export const globalLanguage = useGlobalLanguageState();
/**
 * User's integrate i18n controller
 */
export function useI18nController(initialLang) {
    const languageChangeEventHook = createEventHook();
    const setLanguage = (lang = 'zh-CN') => {
        if (globalLanguage.value !== lang) {
            globalLanguage.value = lang;
            languageChangeEventHook.trigger(lang);
        }
    };
    if (initialLang)
        setLanguage(initialLang);
    return {
        current: globalLanguage,
        setLanguage,
        switchLanguage() {
            if (globalLanguage.value === 'en-US') {
                setLanguage('zh-CN');
            }
            else {
                setLanguage('en-US');
            }
        },
        onLanguageChange: languageChangeEventHook.on
    };
}
