import { defineComponent, toRefs, computed, onMounted } from 'vue';
import { getSlotFirstVNode, useThemeRegister, createComponentVNode, createElementVNode } from '../_utils_';
import { mainCssr } from './styles';
import { iconProps } from './interface';
export default defineComponent({
    name: 'Icon',
    props: iconProps,
    setup(props, { slots }) {
        // theme register
        onMounted(() => {
            useThemeRegister({
                key: 'Icon',
                main: mainCssr
            });
        });
        const { size, color, spin, speed, icon } = toRefs(props);
        const spinningSpeed = computed(() => {
            switch (speed.value) {
                case 'slow':
                    return '2s';
                case 'normal':
                    return '1.6s';
                case 'fast':
                    return '1.2s';
                default:
                    return '1.6s';
            }
        });
        const cssVars = computed(() => {
            var _a;
            return {
                '--icon-color': (_a = color.value) !== null && _a !== void 0 ? _a : 'initial',
                '--icon-font-size': size.value ? `${size.value}px` : 'initial',
                '--icon-spinning-speed': spinningSpeed.value
            };
        });
        return () => createElementVNode('i', {
            role: 'img',
            class: ['mc-icon', { 'mc-icon--spinning': spin.value }],
            style: cssVars.value
        }, [icon.value ? createComponentVNode(icon.value) : getSlotFirstVNode(slots.default)], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */);
    }
});
