import type { ElementClassSet, ElementStyleSet } from '../_utils_';
import type { InjectionKey, PropType, CSSProperties } from 'vue';
export declare const layoutInjectionKey: InjectionKey<unknown>;
export declare const layoutIKey: unique symbol;
export declare const layoutHeaderIKey: unique symbol;
export declare const layoutContentIKey: unique symbol;
export declare const layoutFooterIKey: unique symbol;
export declare const layoutSiderIKey: unique symbol;
export declare const basicColumnLayoutComponentIKey: unique symbol;
export declare const basicRowLayoutComponentIKey: unique symbol;
export interface LayoutProps {
    preset?: 'holy' | 'full' | 'two-column' | 'three-column';
    siderRight?: boolean;
    siderWidth?: string | number;
    leftSiderWidth?: string | number;
    rightSiderWidth?: string | number;
    fixedSider?: boolean;
    fixedLeftSider?: boolean;
    fixedRightSider?: boolean;
    fixedHeader?: boolean;
    fixedFooter?: boolean;
    siderStyle?: ElementStyleSet;
    leftSiderStyle?: ElementStyleSet;
    rightSiderStyle?: ElementStyleSet;
    headerStyle?: ElementStyleSet;
    contentStyle?: ElementStyleSet;
    footerStyle?: ElementStyleSet;
    siderClass?: ElementClassSet;
    leftSiderClass?: ElementClassSet;
    rightSiderClass?: ElementClassSet;
    headerClass?: ElementClassSet;
    contentClass?: ElementClassSet;
    footerClass?: ElementClassSet;
    showSider?: boolean;
    showLeftSider?: boolean;
    showRightSider?: boolean;
    showHeader?: boolean;
    showContent?: boolean;
    showFooter?: boolean;
}
export interface LayoutSiderProps {
    width?: string | number;
    bordered?: boolean;
    collapsed?: boolean;
    collapsable?: boolean;
    collapsedWidth?: string | number;
    triggerPosition?: {
        top?: CSSProperties['top'];
        bottom?: CSSProperties['bottom'];
    };
    triggerType?: 'button' | 'bar';
    transitionMode?: 'width' | 'transform';
    onBeforeToggle: (isCollapsed: boolean) => Promise<boolean | undefined | void> | boolean | undefined | void;
}
export interface LayoutHeaderProps {
    bordered?: boolean;
}
export interface LayoutFooterProps {
    bordered?: boolean;
}
export declare const layoutProps: {
    preset: {
        type: PropType<"holy" | "full" | "two-column" | "three-column" | undefined>;
        default: undefined;
    };
    siderRight: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    siderWidth: {
        type: PropType<string | number | undefined>;
        default: number;
    };
    leftSiderWidth: {
        type: PropType<string | number | undefined>;
        default: number;
    };
    rightSiderWidth: {
        type: PropType<string | number | undefined>;
        default: number;
    };
    fixedSider: {
        type: PropType<boolean | undefined>;
        default: undefined;
    };
    fixedLeftSider: {
        type: PropType<boolean | undefined>;
        default: undefined;
    };
    fixedRightSider: {
        type: PropType<boolean | undefined>;
        default: undefined;
    };
    fixedHeader: {
        type: PropType<boolean | undefined>;
        default: undefined;
    };
    fixedFooter: {
        type: PropType<boolean | undefined>;
        default: undefined;
    };
    siderStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    leftSiderStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    rightSiderStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    headerStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    contentStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    footerStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    siderClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    leftSiderClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    rightSiderClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    headerClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    contentClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    footerClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    showSider: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showLeftSider: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showRightSider: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showHeader: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showContent: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showFooter: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
export declare const layoutSiderProps: {
    width: {
        type: PropType<string | number | undefined>;
        default: number;
    };
    bordered: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    collapsed: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    collapsable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    collapsedWidth: {
        type: PropType<string | number | undefined>;
        default: number;
    };
    triggerPosition: {
        type: PropType<{
            top?: CSSProperties['top'];
            bottom?: CSSProperties['bottom'];
        } | undefined>;
        default: undefined;
    };
    triggerType: {
        type: PropType<"bar" | "button" | undefined>;
        default: string;
    };
    transitionMode: {
        type: PropType<"transform" | "width" | undefined>;
        default: string;
    };
    onBeforeToggle: {
        type: PropType<(isCollapsed: boolean) => Promise<boolean | undefined | void> | boolean | undefined | void>;
        default: undefined;
    };
};
export declare const layoutHeaderProps: {
    bordered: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
export declare const layoutFooterProps: {
    bordered: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
