var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { defineComponent, getCurrentInstance, renderSlot, toRefs, computed, ref } from 'vue';
import { cssUnitTransform } from '../../_utils_';
import { createElementVNode, createComponentVNode } from '../../_utils_';
import { layoutSiderIKey, layoutIKey, layoutSiderProps } from '../interface';
import { McButton } from '../../button';
import { McIcon } from '../../icon';
import { ChevronBackSharp } from '@vicons/ionicons5';
export default defineComponent({
    name: 'LayoutSider',
    iKey: layoutSiderIKey,
    props: layoutSiderProps,
    emits: ['toggled'],
    setup(props, { slots, emit, expose }) {
        var _a;
        const { parent } = (_a = getCurrentInstance()) !== null && _a !== void 0 ? _a : {};
        if (parent && parent.type.iKey !== layoutIKey) {
            throw new Error('[McLayoutSider]: McLayoutHeader must be placed inside McLayout.');
        }
        const { width, bordered, collapsed, collapsable, collapsedWidth, triggerPosition, triggerType, transitionMode, onBeforeToggle } = toRefs(props);
        const isCollapsed = ref(!!collapsed.value);
        const mergedWidth = computed(() => cssUnitTransform(isCollapsed.value ? collapsedWidth.value : width.value));
        const cssVars = computed(() => {
            var _a;
            const { top, bottom } = (_a = triggerPosition.value) !== null && _a !== void 0 ? _a : {};
            return {
                '--layout-sider-width': mergedWidth.value,
                '--layout-sider-collapse-button-trigger-top': top || '50%',
                '--layout-sider-collapse-button-trigger-bottom': bottom || '',
                '--layout-sider-scroll-area-min-width': transitionMode.value === 'transform' ? cssUnitTransform(width.value) : ''
            };
        });
        const callToggle = () => __awaiter(this, void 0, void 0, function* () {
            var _b;
            const callback = yield ((_b = onBeforeToggle.value) === null || _b === void 0 ? void 0 : _b.call(onBeforeToggle, isCollapsed.value));
            if (callback)
                return;
            isCollapsed.value = !isCollapsed.value;
            emit('toggled', isCollapsed.value);
        });
        const buttonTriggerVNode = () => {
            return createComponentVNode(McButton, { circle: true, size: 'small' }, {
                icon: () => createComponentVNode(McIcon, null, {
                    default: () => createComponentVNode(ChevronBackSharp)
                })
            });
        };
        const collapseTriggerVNode = () => {
            return createElementVNode('div', { class: `mc-layout-sider__collapse-${triggerType.value}-trigger`, onClick: callToggle }, [slots.trigger ? renderSlot(slots, 'trigger') : triggerType.value === 'button' ? buttonTriggerVNode() : null]);
        };
        expose({
            toggle: () => {
                isCollapsed.value = !isCollapsed.value;
            }
        });
        // main logic...
        return () => {
            return createElementVNode('aside', {
                class: ['mc-layout-sider', bordered.value ? 'mc-layout-sider--bordered' : '', isCollapsed.value ? 'mc-layout-sider--collapsed' : ''],
                style: cssVars.value
            }, [createElementVNode('div', { class: 'mc-layout-sider__scroll-area' }, [renderSlot(slots, 'default')]), collapsable.value ? collapseTriggerVNode() : null]);
        };
    }
});
