import { defineComponent, renderSlot, createVNode, inject, computed, toRefs, getCurrentInstance, mergeProps } from 'vue';
import { checkParent, createComponentVNode, createElementVNode } from '../../_utils_';
import { not, or } from '@vueuse/core';
import { menuIKey, menuItemGroupIKey, menuItemIKey, menuItemProps, menuInjectionKey, subMenuInjectionKey, menuGroupInjectionKey } from '../interface';
import { McTooltip } from '../../tooltip';
export default defineComponent({
    name: 'MenuItem',
    props: menuItemProps,
    iKey: menuItemIKey,
    setup(props, { slots, attrs }) {
        var _a, _b, _c, _d;
        const instance = getCurrentInstance();
        const key = (_a = instance === null || instance === void 0 ? void 0 : instance.vnode.key) !== null && _a !== void 0 ? _a : '';
        const isParentMenu = computed(() => checkParent(menuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentMenuItemGroup = computed(() => checkParent(menuItemGroupIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const { indent, disabled } = toRefs(props);
        const { activeKey, updateKey, padding: menuPadding, isCollapsed, isDisabled: isMenuDisabled } = (_b = inject(menuInjectionKey, null)) !== null && _b !== void 0 ? _b : {};
        const { padding: subMenuPadding, isDisabled: isSubMenuDisabled, hidePopover: hideSubMenuPopover } = (_c = inject(subMenuInjectionKey, null)) !== null && _c !== void 0 ? _c : {};
        const { padding: menuItemGroupPadding, isDisabled: isMenuItemGroupDisabled, hidePopover: hideMenuItemGroupPopover } = (_d = inject(menuGroupInjectionKey, null)) !== null && _d !== void 0 ? _d : {};
        const isActive = computed(() => !!(key && key === (activeKey === null || activeKey === void 0 ? void 0 : activeKey.value)));
        const selfPadding = computed(() => (typeof indent.value === 'number' ? indent.value : isParentMenuItemGroup.value ? ((menuItemGroupPadding === null || menuItemGroupPadding === void 0 ? void 0 : menuItemGroupPadding.value) || 0) + 16 : ((isParentMenu.value ? menuPadding === null || menuPadding === void 0 ? void 0 : menuPadding.value : subMenuPadding === null || subMenuPadding === void 0 ? void 0 : subMenuPadding.value) || 0) + 32));
        const mergedDisabled = or(isMenuDisabled, isParentMenuItemGroup.value ? isMenuItemGroupDisabled === null || isMenuItemGroupDisabled === void 0 ? void 0 : isMenuItemGroupDisabled.value : isSubMenuDisabled === null || isSubMenuDisabled === void 0 ? void 0 : isSubMenuDisabled.value, disabled);
        const cssVars = computed(() => {
            return {
                '--menu-item-padding-left': `${selfPadding.value}px`
            };
        });
        // main logic...
        return () => createComponentVNode(McTooltip, { disabled: or(not(isCollapsed), not(isParentMenu), mergedDisabled).value, content: () => renderSlot(slots, 'default'), placement: 'right' }, {
            default: () => createElementVNode('li', {
                class: ['mc-menu-item', isActive.value ? 'mc-menu-item--active' : '', disabled.value ? 'mc-menu-item--disabled' : ''],
                style: cssVars.value,
                onClick: () => {
                    if (mergedDisabled.value)
                        return;
                    key && (updateKey === null || updateKey === void 0 ? void 0 : updateKey(key));
                    hideSubMenuPopover === null || hideSubMenuPopover === void 0 ? void 0 : hideSubMenuPopover();
                    hideMenuItemGroupPopover === null || hideMenuItemGroupPopover === void 0 ? void 0 : hideMenuItemGroupPopover();
                }
            }, [slots.icon ? createVNode('div', { class: 'mc-menu-item__icon' }, [renderSlot(slots, 'icon')]) : null, createVNode('div', mergeProps({ class: 'mc-menu-item__content' }, attrs), [renderSlot(slots, 'default')])], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 16 /* PatchFlags.FULL_PROPS */),
            _: 3 /* SlotFlags.FORWARDED */
        }, 8 /* PatchFlags.PROPS */, ['disabled', 'content']);
    }
});
