import { defineComponent, toRefs, renderSlot, inject, provide, computed, ref, getCurrentInstance, mergeProps } from 'vue';
import { checkParent, flattenWithOptions, propsMergeSlots, createComponentVNode, createElementVNode, createDirectives } from '../../_utils_';
import { and, not, or } from '@vueuse/core';
import { menuIKey, subMenuIKey, menuItemGroupIKey, menuItemIKey, subMenuInjectionKey, menuInjectionKey, menuGroupInjectionKey, menuItemGroupProps } from '../interface';
import { McPopover } from '../../popover';
import { McFadeInExpandTransition } from '../../_transition_';
export default defineComponent({
    name: 'MenuItemGroup',
    iKey: menuItemGroupIKey,
    props: menuItemGroupProps,
    setup(props, { slots, attrs }) {
        var _a, _b;
        const instance = getCurrentInstance();
        const isParentMenu = computed(() => checkParent(menuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentSubMenu = computed(() => checkParent(subMenuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const { indent, disabled } = toRefs(props);
        const { activeKey, padding: menuPadding, isCollapsed: isMenuCollapsed, isHorizontal: isMenuHorizontal, isDisabled: isMenuDisabled } = (_a = inject(menuInjectionKey, null)) !== null && _a !== void 0 ? _a : {};
        const { padding: subMenuPadding, isDisabled: isSubMenuDisabled } = (_b = inject(subMenuInjectionKey, null)) !== null && _b !== void 0 ? _b : {};
        const hasCollapsed = ref(isMenuCollapsed === null || isMenuCollapsed === void 0 ? void 0 : isMenuCollapsed.value);
        const isActive = computed(() => {
            const keys = flattenWithOptions({ slots, key: menuItemIKey, infinity: true }).map(item => {
                return item.key;
            });
            return !!((activeKey === null || activeKey === void 0 ? void 0 : activeKey.value) && keys.includes(activeKey.value));
        });
        const selfPadding = computed(() => (typeof indent.value === 'number' ? indent.value : isParentSubMenu.value ? ((subMenuPadding === null || subMenuPadding === void 0 ? void 0 : subMenuPadding.value) || 0) + 16 : ((menuPadding === null || menuPadding === void 0 ? void 0 : menuPadding.value) || 0) + 32));
        const mergedDisabled = or(isMenuDisabled, isSubMenuDisabled, disabled);
        const menuPopoverPlacement = computed(() => {
            if (isMenuHorizontal === null || isMenuHorizontal === void 0 ? void 0 : isMenuHorizontal.value) {
                return 'bottom';
            }
            else if (isMenuCollapsed === null || isMenuCollapsed === void 0 ? void 0 : isMenuCollapsed.value) {
                return 'right-start';
            }
        });
        const menuPopoverDisabled = or(mergedDisabled, not(and(isParentMenu, or(isMenuHorizontal, isMenuCollapsed))));
        const menuPopoverRef = ref();
        const cssVars = computed(() => {
            return {
                '--menu-item-group-padding-left': `${selfPadding.value}px`
            };
        });
        const handleHidePopover = () => {
            var _a;
            (_a = menuPopoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
        };
        provide(menuGroupInjectionKey, {
            padding: selfPadding,
            isDisabled: disabled,
            hidePopover: handleHidePopover
        });
        // main logic...
        return () => createElementVNode('li', {
            class: [
                'mc-menu-item-group',
                and(hasCollapsed, isParentMenu).value ? 'mc-menu-item-group--collapsed' : '',
                and(isActive, not(menuPopoverDisabled)).value ? 'mc-menu-item-group--child-active' : '',
                disabled.value ? 'mc-menu-item-group--disabled' : ''
            ]
        }, [
            createComponentVNode(McPopover, {
                ref_key: 'menuPopoverRef',
                ref: menuPopoverRef,
                disabled: menuPopoverDisabled.value,
                placement: menuPopoverPlacement.value,
                withArrow: false,
                style: { padding: '4px 0', width: '200px', [menuPopoverPlacement.value === 'bottom' ? 'marginTop' : 'marginLeft']: '4px' },
                class: ['mc-menu-item-group mc-menu-item-group--dropdown']
            }, {
                content: () => createElementVNode('ul', mergeProps({ class: 'mc-menu-item-group-children' }, attrs), [renderSlot(slots, 'default')]),
                default: () => createElementVNode('div', { class: 'mc-menu-item-group-title', style: cssVars.value }, [createElementVNode('span', { class: 'mc-menu-item-group-title__content' }, [propsMergeSlots(props, slots, 'title')])], 16 /* PatchFlags.FULL_PROPS */)
            }, 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['disabled', 'placement']),
            and(isMenuHorizontal, isParentMenu).value
                ? null
                : createComponentVNode(McFadeInExpandTransition, {
                    onAfterLeave: () => (hasCollapsed.value = true),
                    onEnter: () => (hasCollapsed.value = false)
                }, {
                    default: () => createDirectives('v-if', {
                        condition: not(and(isMenuCollapsed, isParentMenu)).value,
                        node: createElementVNode('ul', mergeProps({ class: 'mc-menu-item-group-children' }, attrs), [renderSlot(slots, 'default')])
                    }),
                    _: 3 /* SlotFlags.FORWARDED */
                })
        ], 2 /* PatchFlags.CLASS */);
    }
});
