import { defineComponent, renderSlot, toRefs, ref, provide, inject, computed, getCurrentInstance, mergeProps } from 'vue';
import { checkParent, flattenWithOptions, propsMergeSlots, createComponentVNode, createElementVNode, createDirectives } from '../../_utils_';
import { and, not, or } from '@vueuse/core';
import { menuIKey, menuItemIKey, menuItemGroupIKey, subMenuIKey, menuInjectionKey, subMenuInjectionKey, menuGroupInjectionKey, subMenuProps } from '../interface';
import { findParent } from './utils';
import { McIcon } from '../../icon';
import { McPopover } from '../../popover';
import { McFadeInExpandTransition } from '../../_transition_';
import { ChevronUpOutline, ChevronForwardOutline } from '@vicons/ionicons5';
export default defineComponent({
    name: 'SubMenu',
    iKey: subMenuIKey,
    props: subMenuProps,
    setup(props, { slots, attrs }) {
        var _a, _b, _c, _d;
        const instance = getCurrentInstance();
        const key = (_a = instance === null || instance === void 0 ? void 0 : instance.vnode.key) !== null && _a !== void 0 ? _a : '';
        const isParentMenu = computed(() => checkParent(menuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentSubMenu = computed(() => checkParent(subMenuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentMenuItemGroup = computed(() => checkParent(menuItemGroupIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const { unique, indent, disabled } = toRefs(props);
        const { activeKey, expandedKeys, updateExpandKeys, padding: menuPadding, keyTree, options, isUnique: isMenuUnique, isCollapsed: isMenuCollapsed, isHorizontal: isMenuHorizontal, isDisabled: isMenuDisabled } = (_b = inject(menuInjectionKey, null)) !== null && _b !== void 0 ? _b : {};
        const { padding: subMenuPadding, isUnique: isSubMenuUnique, isDisabled: isSubMenuDisabled, hidePopover: hideSubMenuPopover } = (_c = inject(subMenuInjectionKey, null)) !== null && _c !== void 0 ? _c : {};
        const { padding: menuItemGroupPadding, isDisabled: isMenuItemGroupDisabled, hidePopover: hideMenuItemGroupPopover } = (_d = inject(menuGroupInjectionKey, null)) !== null && _d !== void 0 ? _d : {};
        const isExpanded = computed(() => expandedKeys === null || expandedKeys === void 0 ? void 0 : expandedKeys.value.includes(key || ''));
        const isActive = computed(() => {
            const keys = flattenWithOptions({ slots, key: menuItemIKey, infinity: true }).map(item => {
                return item.key;
            });
            return !!((activeKey === null || activeKey === void 0 ? void 0 : activeKey.value) && keys.includes(activeKey.value));
        });
        const selfPadding = computed(() => (typeof indent.value === 'number' ? indent.value : isParentMenuItemGroup.value ? ((menuItemGroupPadding === null || menuItemGroupPadding === void 0 ? void 0 : menuItemGroupPadding.value) || 0) + 16 : ((isParentMenu.value ? menuPadding === null || menuPadding === void 0 ? void 0 : menuPadding.value : subMenuPadding === null || subMenuPadding === void 0 ? void 0 : subMenuPadding.value) || 0) + 32));
        const mergedDisabled = or(isMenuDisabled, isParentMenuItemGroup.value ? isMenuItemGroupDisabled === null || isMenuItemGroupDisabled === void 0 ? void 0 : isMenuItemGroupDisabled.value : isSubMenuDisabled === null || isSubMenuDisabled === void 0 ? void 0 : isSubMenuDisabled.value, disabled);
        const watchUnique = or(and(isMenuUnique, isParentMenu), and(isSubMenuUnique, isParentSubMenu));
        const menuPopoverPlacement = computed(() => {
            if (isMenuHorizontal === null || isMenuHorizontal === void 0 ? void 0 : isMenuHorizontal.value) {
                return isParentMenu.value ? 'bottom' : 'right-start';
            }
            else if (isMenuCollapsed === null || isMenuCollapsed === void 0 ? void 0 : isMenuCollapsed.value) {
                return 'right-start';
            }
        });
        const menuPopoverDisabled = or(mergedDisabled, and(not(isMenuCollapsed), not(isMenuHorizontal)));
        const menuPopoverRef = ref();
        const cssVars = computed(() => {
            return {
                '--menu-submenu-padding-left': `${selfPadding.value}px`
            };
        });
        const handleHidePopover = () => {
            var _a;
            (_a = menuPopoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
            hideSubMenuPopover === null || hideSubMenuPopover === void 0 ? void 0 : hideSubMenuPopover();
            hideMenuItemGroupPopover === null || hideMenuItemGroupPopover === void 0 ? void 0 : hideMenuItemGroupPopover();
        };
        provide(subMenuInjectionKey, {
            padding: selfPadding,
            isDisabled: disabled,
            isUnique: unique,
            hidePopover: handleHidePopover
        });
        // main logic...
        return () => createElementVNode('li', { class: ['mc-sub-menu', and(not(isExpanded), menuPopoverDisabled).value ? 'mc-sub-menu--collapsed' : '', isActive.value ? 'mc-sub-menu--child-active' : '', disabled.value ? 'mc-sub-menu--disabled' : ''] }, [
            createComponentVNode(McPopover, {
                ref: menuPopoverRef,
                disabled: menuPopoverDisabled.value,
                placement: menuPopoverPlacement.value,
                withArrow: false,
                style: { padding: '4px 0', width: '200px', [menuPopoverPlacement.value === 'bottom' ? 'marginTop' : 'marginLeft']: '4px' },
                class: ['mc-sub-menu mc-sub-menu--dropdown', `mc-sub-menu--placement-${menuPopoverPlacement.value}`]
            }, {
                content: () => createElementVNode('ul', mergeProps({ class: 'mc-sub-menu-children', style: { margin: 0 } }, attrs), [renderSlot(slots, 'default')]),
                default: () => createElementVNode('div', {
                    class: 'mc-sub-menu-title',
                    style: cssVars.value,
                    onClick: () => {
                        var _a, _b, _c, _d;
                        if (or(isMenuCollapsed, isMenuHorizontal, mergedDisabled).value)
                            return;
                        if (or(not(watchUnique), isExpanded).value) {
                            updateExpandKeys === null || updateExpandKeys === void 0 ? void 0 : updateExpandKeys(key);
                        }
                        else {
                            const keys = isParentMenu.value ? keyTree.map(item => item.children && item.key).filter(Boolean) : (_d = (_c = (_b = findParent((_a = options === null || options === void 0 ? void 0 : options.value) !== null && _a !== void 0 ? _a : keyTree, key)) === null || _b === void 0 ? void 0 : _b.children) === null || _c === void 0 ? void 0 : _c.map((item) => item.key)) !== null && _d !== void 0 ? _d : [];
                            updateExpandKeys === null || updateExpandKeys === void 0 ? void 0 : updateExpandKeys([...expandedKeys === null || expandedKeys === void 0 ? void 0 : expandedKeys.value.filter(key => !keys.includes(key)), key]);
                        }
                    }
                }, [
                    slots.icon ? createElementVNode('div', { class: 'mc-sub-menu-title__icon' }, [renderSlot(slots, 'icon')]) : null,
                    createElementVNode('span', { class: 'mc-sub-menu-title__content' }, [propsMergeSlots(props, slots, 'title')]),
                    and(isMenuHorizontal, isParentMenu).value
                        ? null
                        : createComponentVNode(McIcon, { class: 'mc-sub-menu-title__arrow' }, {
                            default: () => createComponentVNode(or(isMenuCollapsed, menuPopoverPlacement.value === 'right-start').value ? ChevronForwardOutline : ChevronUpOutline)
                        })
                ], 16 /* PatchFlags.FULL_PROPS */),
                _: 3 /* SlotFlags.FORWARDED */
            }, 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['disabled', 'placement']),
            (isMenuHorizontal === null || isMenuHorizontal === void 0 ? void 0 : isMenuHorizontal.value)
                ? null
                : createComponentVNode(McFadeInExpandTransition, null, {
                    default: () => createDirectives('v-if', {
                        condition: and(isExpanded, not(isMenuCollapsed)).value,
                        node: createElementVNode('ul', mergeProps({ class: 'mc-sub-menu-children' }, attrs), [renderSlot(slots, 'default')])
                    }),
                    _: 3 /* SlotFlags.FORWARDED */
                })
        ], 2 /* PatchFlags.CLASS */);
    }
});
