import { createVNode } from 'vue';
import { flatten } from '../../_utils_';
import { subMenuIKey, menuItemIKey, menuItemGroupIKey } from '../interface';
import McMenuItem from './MenuItem';
import McMenuItemGroup from './MenuItemGroup';
import McSubMenu from './SubMenu';
export function createKeyTree(children, result = []) {
    children.forEach(vnode => {
        var _a, _b, _c, _d;
        const { iKey } = vnode.type;
        const subChildren = (_b = (_a = vnode.children).default) === null || _b === void 0 ? void 0 : _b.call(_a);
        if (iKey === menuItemIKey) {
            result.push({ key: (_c = vnode.key) !== null && _c !== void 0 ? _c : '' });
        }
        else if (iKey === subMenuIKey) {
            result.push({
                key: (_d = vnode.key) !== null && _d !== void 0 ? _d : '',
                children: createKeyTree(subChildren)
            });
        }
        else if (iKey === menuItemGroupIKey) {
            const items = flatten(subChildren, [subMenuIKey, menuItemIKey]);
            createKeyTree(items, result);
        }
    });
    return result;
}
export function createKeyTreeByOptions(options, result = []) {
    options.forEach(option => {
        const { key = '', children, group } = option;
        if (group) {
            const items = (children !== null && children !== void 0 ? children : []).map(({ key = '', children }) => ({ key, children }));
            createKeyTreeByOptions(items, result);
        }
        else if (children) {
            result.push({
                key,
                children: createKeyTreeByOptions(children !== null && children !== void 0 ? children : [])
            });
        }
        else {
            result.push({ key });
        }
    });
    return result;
}
export function createMenu(option) {
    const { key, label, icon, indent, disabled, unique, group, children, style, class: itemClass } = option;
    if (group) {
        return createVNode(McMenuItemGroup, { indent, disabled, title: label, style, class: itemClass }, {
            default: () => (children !== null && children !== void 0 ? children : []).map(item => createMenu(item))
        });
    }
    else if (children) {
        return createVNode(McSubMenu, { key, indent, disabled, unique, title: label, style, class: itemClass }, {
            icon,
            default: () => children.map(item => createMenu(item))
        });
    }
    else {
        return createVNode(McMenuItem, { key, indent, disabled, style, class: itemClass }, {
            icon,
            default: typeof label === 'string' ? () => label : label
        });
    }
}
export function findPath(tree, key, path = []) {
    for (let i = 0; i < tree.length; i++) {
        const tempPath = [...path];
        tempPath.push(tree[i].key);
        if (tree[i].key === key)
            return tempPath;
        if (tree[i].children) {
            const result = findPath(tree[i].children, key, tempPath);
            if (result)
                return result;
        }
    }
}
export function findNode(tree, key) {
    for (let i = 0; i < tree.length; i++) {
        if (tree[i].key === key)
            return tree[i];
        if (tree[i].children) {
            const result = findNode(tree[i].children, key);
            if (result)
                return result;
        }
    }
}
export function findParent(tree, key) {
    for (let i = 0; i < tree.length; i++) {
        if (tree[i].children) {
            const index = tree[i].children.findIndex((item) => item.key === key);
            if (index > -1)
                return tree[i];
            const result = findParent(tree[i].children, key);
            if (result)
                return result;
        }
    }
}
