import { ref, reactive, createApp, TransitionGroup, watch } from 'vue';
import { useThemeRegister, createComponentVNode, createDirectives, createKey, reactiveOmit, responsiveTarget } from '../_utils_';
import { VLazyTeleport } from 'vueuc';
import MessageEntity from './MessageEntity';
import { mainCssr, lightCssr, darkCssr } from './styles';
import { omit } from 'lodash-es';
export default function useMessage(config = {}) {
    useThemeRegister({
        key: 'Message',
        main: mainCssr,
        light: lightCssr,
        dark: darkCssr
    });
    const items = reactive([]);
    const itemRefsMap = new WeakMap();
    const isMounted = ref(false);
    const getMessageVNode = (item, index) => {
        var _a, _b, _c, _d;
        const { key, type, options } = item;
        const { className, onClose } = options;
        return createComponentVNode(MessageEntity, Object.assign(Object.assign({ key,
            type, 'data-index': index, card: (_a = config.card) !== null && _a !== void 0 ? _a : false, duration: (_b = config.duration) !== null && _b !== void 0 ? _b : 3000, closable: (_c = config.closable) !== null && _c !== void 0 ? _c : false, hoverAlive: (_d = config.hoverAlive) !== null && _d !== void 0 ? _d : true }, omit(options, 'message', 'className')), { class: className, onClose: () => {
                onClose === null || onClose === void 0 ? void 0 : onClose();
                close(key);
            }, onVnodeMounted(vnode) {
                var _a;
                const exposedInst = (_a = vnode.component) === null || _a === void 0 ? void 0 : _a.exposed;
                if (exposedInst) {
                    itemRefsMap.set(item, exposedInst);
                }
            },
            onVnodeUnmounted() {
                itemRefsMap.delete(item);
            } }), {
            default: () => { var _a; return (typeof options.message === 'string' ? options.message : (_a = options.message) === null || _a === void 0 ? void 0 : _a.call(options)); }
        }, 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['type', 'duration', 'closable', 'hoverAlive', 'html', 'card', 'icon', 'action', 'key']);
    };
    const MessageEnvironment = () => {
        return isMounted.value
            ? createComponentVNode(VLazyTeleport, {
                to: 'body',
                show: isMounted.value
            }, {
                default: () => {
                    var _a, _b, _c;
                    return createComponentVNode(TransitionGroup, {
                        name: 'mc-message-slide-down',
                        class: 'mc-message-global-container',
                        appear: true,
                        tag: 'div',
                        'v-placement': (_a = config.placement) !== null && _a !== void 0 ? _a : 'top-center',
                        style: [{ '--message-item-gap': `${(_b = config.itemGap) !== null && _b !== void 0 ? _b : 8}px` }, (_c = config.containerStyle) !== null && _c !== void 0 ? _c : {}],
                        onEnter: (el) => {
                            var _a, _b, _c;
                            if (((_b = (_a = el === null || el === void 0 ? void 0 : el.dataset) === null || _a === void 0 ? void 0 : _a.index) !== null && _b !== void 0 ? _b : 0) > ((_c = config.max) !== null && _c !== void 0 ? _c : Infinity) - 1) {
                                close(0);
                            }
                        },
                        onAfterLeave: () => {
                            if (items.length === 0 && isMounted.value) {
                                unmount();
                            }
                        }
                    }, {
                        default: () => createDirectives('v-for', items, (item, index) => getMessageVNode(item, index), true)
                    }, 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['v-placement']);
                }
            })
            : null;
    };
    let hostElement = null;
    let app = null;
    const mount = () => {
        isMounted.value = true;
        hostElement = document.createElement('div');
        app = createApp(createComponentVNode(MessageEnvironment));
        app === null || app === void 0 ? void 0 : app.mount(hostElement);
    };
    const unmount = () => {
        isMounted.value = false;
        app === null || app === void 0 ? void 0 : app.unmount();
        app = null;
        hostElement = null;
    };
    const close = (key) => {
        if (typeof key === 'string') {
            const index = items.findIndex(item => item.key === key);
            index > -1 && items.splice(index, 1);
        }
        else {
            items.splice(key, 1);
        }
    };
    const create = (type, options, isAsync) => {
        const message = reactive({
            key: createKey('message'),
            type,
            options
        });
        items.push(message);
        const proxyMessage = new Proxy(message, {
            set(target, p, value) {
                if (p === 'key') {
                    return false;
                }
                else if (p === 'type') {
                    return Reflect.set(target, p, value);
                }
                else {
                    return Reflect.set(target.options, p, value);
                }
            },
            get(target, p) {
                var _a;
                if (p === 'key' || p === 'type') {
                    return Reflect.get(target, p);
                }
                else if (p === 'close' || p === 'el') {
                    return (_a = itemRefsMap.get(target)) === null || _a === void 0 ? void 0 : _a[p];
                }
                else {
                    return Reflect.get(target.options, p);
                }
            }
        });
        return (isAsync
            ? new Promise(resolve => {
                const originalOnCloseHandler = proxyMessage.onClose;
                proxyMessage.onClose = () => {
                    originalOnCloseHandler === null || originalOnCloseHandler === void 0 ? void 0 : originalOnCloseHandler();
                    resolve(proxyMessage);
                };
            })
            : proxyMessage);
    };
    function createMessageApi(type, isAsync) {
        function ApiConstructor(maybeOptions, options = {}) {
            if (typeof maybeOptions === 'string') {
                return create(type, Object.assign({ message: maybeOptions }, options), isAsync);
            }
            else {
                return create(type, Object.assign(Object.assign({ message: '' }, maybeOptions), options), isAsync);
            }
        }
        return ApiConstructor;
    }
    // @ts-ignore
    const McMessage = (options) => {
        const reactiveOptions = responsiveTarget(options);
        return create(reactiveOptions.type, reactiveOmit(reactiveOptions, 'type'), false);
    };
    // @ts-ignore
    const McAsyncMessage = (options) => {
        const reactiveOptions = responsiveTarget(options);
        return create(reactiveOptions.type, reactiveOmit(reactiveOptions, 'type'), true);
    };
    ['text', 'success', 'warning', 'info', 'error', 'loading'].forEach(type => {
        McMessage[type] = createMessageApi(type, false);
        McAsyncMessage[type] = createMessageApi(type, true);
    });
    watch(items, val => {
        if (items.length > 0 && !isMounted.value) {
            mount();
        }
    });
    return { McMessage, McAsyncMessage, close };
}
