import type { VNodeChild, CSSProperties, PropType } from 'vue';
export declare type MessageCloseImpl = () => void | Promise<void>;
export declare type MessageType = 'text' | 'success' | 'warning' | 'info' | 'error' | 'loading';
export interface MessageOptions {
    type: MessageType;
    message?: string | (() => VNodeChild);
    className?: string;
    style?: string | CSSProperties;
    duration?: number;
    closable?: boolean;
    hoverAlive?: boolean;
    html?: string;
    card?: boolean;
    itemGap?: number;
    icon?: () => VNodeChild;
    action?: () => VNodeChild;
    onClose?: MessageCloseImpl;
}
export declare type MessageApiOptions = Partial<Omit<MessageOptions, 'type'>>;
export declare type Message = {
    readonly key: string;
    type: MessageType;
    options: MessageApiOptions;
};
export declare type ConfigurableMessage = Required<MessageOptions> & Readonly<MessageExposeInstance>;
export declare type MessageApiReturnType<S extends boolean> = S extends true ? Promise<ConfigurableMessage> : ConfigurableMessage;
export declare type CreateMessageApiReturnType<S extends boolean> = {
    (options: MessageApiOptions): MessageApiReturnType<S>;
    (content: string): MessageApiReturnType<S>;
    (content: string, options: MessageApiOptions): MessageApiReturnType<S>;
};
export declare type MessageApi<S extends boolean = false> = {
    (options: MessageOptions): MessageApiReturnType<S>;
    text: CreateMessageApiReturnType<S>;
    success: CreateMessageApiReturnType<S>;
    warning: CreateMessageApiReturnType<S>;
    info: CreateMessageApiReturnType<S>;
    error: CreateMessageApiReturnType<S>;
    loading: CreateMessageApiReturnType<S>;
};
export interface MessageExposeInstance {
    close?: MessageCloseImpl;
    el?: HTMLElement;
}
export interface MessageProps {
    type?: MessageType;
    duration?: number;
    closable?: boolean;
    hoverAlive?: boolean;
    html?: string;
    card?: boolean;
    icon?: () => VNodeChild;
    action?: () => VNodeChild;
}
export interface MessageGlobalConfig {
    max?: number;
    placement?: 'top-left' | 'top-center' | 'top-right' | 'middle-left' | 'middle-center' | 'middle-right' | 'bottom-left' | 'bottom-center' | 'bottom-right';
    duration?: number;
    itemGap?: number;
    card?: boolean;
    hoverAlive?: boolean;
    closable?: boolean;
    containerStyle?: string | CSSProperties;
}
export declare const messageProps: {
    type: {
        type: PropType<MessageType | undefined>;
        default: string;
    };
    duration: {
        type: PropType<number | undefined>;
        default: number;
    };
    closable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    hoverAlive: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    html: {
        type: PropType<MessageType | undefined>;
        default: undefined;
    };
    card: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    icon: {
        type: PropType<(() => VNodeChild) | undefined>;
        default: undefined;
    };
    action: {
        type: PropType<(() => VNodeChild) | undefined>;
        default: undefined;
    };
};
