var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { defineComponent, createVNode, toRefs, computed, renderSlot, ref, Transition, watch, createTextVNode, mergeProps, provide } from 'vue';
import { createKey, useThemeRegister, useI18n, useSharedItems } from '../_utils_';
import { onClickOutside, useMouse, useMagicKeys, pausableWatch } from '@vueuse/core';
import { VLazyTeleport } from 'vueuc';
import { modalProps, modalInjectionKey } from './interface';
import { McIcon } from '../icon';
import { McButton } from '../button';
import { Close as IconClose } from '@vicons/ionicons5';
import { mainCssr, lightCssr, darkCssr } from './styles';
export default defineComponent({
    name: 'Modal',
    inheritAttrs: false,
    props: modalProps,
    emits: ['update:show', 'wrapper-click', 'shortcut-stroke', 'after-enter', 'after-leave', 'before-enter', 'cancel', 'confirm'],
    setup(props, { slots, attrs, emit, expose }) {
        // theme register
        useThemeRegister({
            key: 'Modal',
            main: mainCssr,
            light: lightCssr,
            dark: darkCssr
        });
        const { show, width, height, appearFromCursor, wrapperClosable, shortcutKey, closeOnShortcut, closable, headerStyle, bodyStyle, footerStyle, headerClass, bodyClass, footerClass, title, showHeader, showFooter, cancelText, confirmText, pure, position, animation, onBeforeLeave, appearX, appearY } = toRefs(props);
        const key = createKey('modal');
        const modalElRef = ref();
        const modalAppearXRef = ref(0);
        const modalAppearYRef = ref(0);
        const modalTransformOrigin = computed(() => (appearFromCursor.value ? `${modalAppearXRef.value}px ${modalAppearYRef.value}px` : 'center center'));
        const modalTransitionName = computed(() => (appearFromCursor.value ? 'mc-modal-scale' : ['scale', 'slide'].includes(animation.value) ? `mc-modal-${animation.value}` : 'mc-modal-scale'));
        const isTopModal = computed(() => topItem.value === key);
        const isMountModal = ref(true);
        const { add, remove, topItem } = useSharedItems();
        const { x, y } = useMouse();
        const { i18n } = useI18n('modal');
        const magicKeys = useMagicKeys({
            passive: false,
            onEventFired(event) {
                isTopModal.value && event.stopImmediatePropagation();
            }
        });
        const keys = computed(() => Array.from(magicKeys.current));
        let closeAction;
        watch(show, () => {
            if (show.value) {
                isMountModal.value = true;
                modalAppearXRef.value = appearX.value || x.value;
                modalAppearYRef.value = appearY.value || y.value;
                add(key);
            }
            else {
                remove(key);
            }
        }, { immediate: true });
        const { pause, resume } = pausableWatch(magicKeys[shortcutKey.value], v => {
            if (v && show.value && closeOnShortcut.value && isTopModal.value) {
                callShortcutStroke();
                callUpdateShow(false);
            }
        });
        watch(isTopModal, () => {
            if (!isTopModal.value) {
                pause();
            }
            else {
                resume();
            }
        }, { immediate: true });
        onClickOutside(modalElRef, event => {
            if (wrapperClosable.value && isTopModal.value) {
                callWrapperClick(event);
                callUpdateShow(false);
            }
        });
        const callUpdateShow = (val) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!val) {
                const callback = yield ((_a = onBeforeLeave.value) === null || _a === void 0 ? void 0 : _a.call(onBeforeLeave, closeAction));
                if (callback)
                    return;
            }
            emit('update:show', val);
        });
        const callWrapperClick = (e) => {
            closeAction = 'wrapper';
            emit('wrapper-click', e);
        };
        const callShortcutStroke = () => {
            closeAction = 'shortcut';
            emit('shortcut-stroke', keys.value);
        };
        const callAfterEnter = () => {
            emit('after-enter');
        };
        const callAfterLeave = () => {
            isMountModal.value = false;
            emit('after-leave');
        };
        const callBeforeEnter = () => {
            emit('before-enter');
        };
        const callOnCancel = () => {
            closeAction = 'cancel';
            emit('cancel');
        };
        const callOnConfirm = () => {
            closeAction = 'confirm';
            emit('confirm');
        };
        const handleCancel = () => {
            callOnCancel();
            callUpdateShow(false);
        };
        const handleConfirm = () => {
            callOnConfirm();
            callUpdateShow(false);
        };
        const modalPosition = computed(() => {
            const { top, right, bottom, left } = position.value || {};
            return {
                marginTop: top !== undefined ? (typeof top === 'number' ? `${top}px` : top) : 'auto',
                marginRight: right !== undefined ? (typeof right === 'number' ? `${right}px` : right) : 'auto',
                marginBottom: bottom !== undefined ? (typeof bottom === 'number' ? `${bottom}px` : bottom) : 'auto',
                marginLeft: left !== undefined ? (typeof left === 'number' ? `${left}px` : left) : 'auto'
            };
        });
        const cssVars = computed(() => {
            return {
                '--modal-width': typeof width.value === 'number' ? `${width.value}px` : width.value,
                '--modal-height': typeof height.value === 'number' ? `${height.value}px` : height.value
            };
        });
        const headerVNode = computed(() => {
            var _a;
            const titleVNode = createVNode('div', { class: 'mc-modal-title' }, [typeof title.value === 'string' ? createTextVNode(title.value) : (_a = title.value) === null || _a === void 0 ? void 0 : _a.call(title)]);
            const closeIconVNode = closable.value
                ? createVNode(McButton, {
                    render: 'text',
                    class: 'mc-modal-close-button',
                    style: { padding: '0 6px' },
                    onClick: () => {
                        closeAction = 'close';
                        callUpdateShow(false);
                    }
                }, {
                    icon: () => createVNode(McIcon, { size: 20 }, { default: () => createVNode(IconClose) })
                })
                : null;
            return !pure.value && showHeader.value ? createVNode('div', { class: ['mc-modal__header', headerClass.value], style: headerStyle.value }, [slots.header ? renderSlot(slots, 'header') : titleVNode, closeIconVNode]) : null;
        });
        const footerVNode = computed(() => {
            return !pure.value && showFooter.value
                ? createVNode('div', { class: ['mc-modal__footer', footerClass.value], style: footerStyle.value }, slots.footer
                    ? [renderSlot(slots, 'footer')]
                    : [
                        cancelText.value !== null ? createVNode(McButton, { ghost: true, onClick: handleCancel }, { default: () => cancelText.value || i18n('CancelButtonText') }) : null,
                        confirmText.value !== null ? createVNode(McButton, { style: { marginLeft: '16px' }, type: 'success', onClick: handleConfirm }, { default: () => confirmText.value || i18n('ConfirmButtonText') }) : null
                    ])
                : null;
        });
        const modalVNode = computed(() => {
            const mergedProps = mergeProps({
                ref: modalElRef,
                class: ['mc-modal', pure.value ? 'mc-modal--pure' : ''],
                style: Object.assign(Object.assign({}, cssVars.value), modalPosition.value)
            }, attrs);
            return createVNode('div', { class: 'mc-modal-wrapper', style: `transform-origin: ${modalTransformOrigin.value}` }, [
                createVNode('div', mergedProps, [headerVNode.value, createVNode('div', { class: ['mc-modal__body', bodyClass.value], style: bodyStyle.value }, [renderSlot(slots, 'default')]), footerVNode.value])
            ]);
        });
        const modalContainerVNode = computed(() => {
            return createVNode('div', { class: 'mc-modal-container' }, [
                createVNode(Transition, { name: 'mc-modal-mask-fade', appear: true }, {
                    default: () => (show.value ? createVNode('div', { class: 'mc-modal-mask' }) : null)
                }),
                createVNode(Transition, { name: modalTransitionName.value, appear: true, onBeforeEnter: callBeforeEnter, onAfterEnter: callAfterEnter, onAfterLeave: callAfterLeave }, {
                    default: () => (show.value ? modalVNode.value : null)
                })
            ]);
        });
        provide(modalInjectionKey, modalElRef);
        expose({
            hide: () => callUpdateShow(false),
            el: modalElRef
        });
        return () => (isMountModal.value ? createVNode(VLazyTeleport, { to: 'body', show: show.value }, { default: () => modalContainerVNode.value }) : null);
    }
});
