import { PropType, VNodeChild, Ref, InjectionKey, CSSProperties } from 'vue';
import { ElementClassSet } from '../_utils_';
export declare type OnBeforeLeaveImpl = (action: ModalCloseAction) => Promise<boolean | undefined | void> | boolean | undefined | void;
export declare type OnCancelImpl = () => Promise<boolean | undefined | void> | boolean | undefined | void;
export declare type OnConfirmImpl = () => Promise<boolean | undefined | void> | boolean | undefined | void;
export declare type ModalCloseAction = 'wrapper' | 'close' | 'shortcut' | 'cancel' | 'confirm';
export declare type ModalInjection = Ref<HTMLElement | undefined> | null;
export interface ModalPosition {
    top?: number | string;
    right?: number | string;
    bottom?: number | string;
    left?: number | string;
}
export interface ModalExposeInstance {
    hide: () => void;
    el: HTMLElement;
}
export interface ModalProps {
    show?: boolean;
    width?: number | string;
    height?: number | string;
    appearFromCursor?: boolean;
    wrapperClosable?: boolean;
    shortcutKey?: string;
    closeOnShortcut?: boolean;
    closable?: boolean;
    headerStyle?: string | CSSProperties;
    bodyStyle?: string | CSSProperties;
    footerStyle?: string | CSSProperties;
    maskStyle?: string | CSSProperties;
    headerClass?: ElementClassSet;
    bodyClass?: ElementClassSet;
    footerClass?: ElementClassSet;
    title?: string | (() => VNodeChild);
    showHeader?: boolean;
    showFooter?: boolean;
    cancelText?: string | null;
    confirmText?: string | null;
    pure?: boolean;
    position?: ModalPosition;
    animation?: 'scale' | 'slide';
    onBeforeLeave?: OnBeforeLeaveImpl;
    appearX?: number;
    appearY?: number;
}
export interface ModalObjectEmits {
    'onUpdate:show'?: (val: boolean) => void;
    onWrapperClick?: (e: MouseEvent) => void;
    onShortcutStroke?: (keys: string[]) => void;
    onAfterEnter?: () => void;
    onAfterLeave?: () => void;
    onBeforeEnter?: () => void;
    onCancel?: () => void;
    onConfirm?: () => void;
}
export declare const modalProps: {
    show: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    width: {
        type: PropType<string | number | undefined>;
        default: number;
    };
    height: {
        type: PropType<string | number | undefined>;
        default: string;
    };
    appearFromCursor: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    wrapperClosable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    shortcutKey: {
        type: PropType<string | undefined>;
        default: string;
    };
    closeOnShortcut: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    closable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    headerStyle: {
        type: PropType<string | CSSProperties | undefined>;
        default: undefined;
    };
    bodyStyle: {
        type: PropType<string | CSSProperties | undefined>;
        default: undefined;
    };
    footerStyle: {
        type: PropType<string | CSSProperties | undefined>;
        default: undefined;
    };
    headerClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    bodyClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    footerClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    title: {
        type: PropType<string | (() => VNodeChild) | undefined>;
        default: string;
    };
    showHeader: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showFooter: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    cancelText: {
        type: PropType<string | null | undefined>;
        default: undefined;
    };
    confirmText: {
        type: PropType<string | null | undefined>;
        default: undefined;
    };
    pure: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    position: {
        type: PropType<ModalPosition | undefined>;
        default: undefined;
    };
    animation: {
        type: PropType<"scale" | "slide" | undefined>;
        default: string;
    };
    onBeforeLeave: {
        type: PropType<OnBeforeLeaveImpl | undefined>;
        default: undefined;
    };
    appearX: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    appearY: {
        type: PropType<number | undefined>;
        default: undefined;
    };
};
export declare const modalInjectionKey: InjectionKey<ModalInjection>;
