var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { defineComponent, ref, toRefs, renderSlot, createVNode, Fragment, mergeProps, onMounted } from 'vue';
import { getSlotFirstVNode, propsMergeSlots, useThemeRegister } from '../_utils_';
import { omit } from 'lodash-es';
import { AlertCircle as IconAlert } from '@vicons/ionicons5';
import { McPopover } from '../popover';
import { popoverProps, popoverEmits } from '../popover/interface';
import { McButton } from '../button';
import { McIcon } from '../icon';
import { popconfirmProps, popconfirmEmits } from './interface';
import { mainCssr } from './styles';
const defaultPropsOverride = {
    trigger: {
        type: String,
        default: 'click'
    }
};
export default defineComponent({
    name: 'Popconfirm',
    props: Object.assign(Object.assign(Object.assign({}, popoverProps), popconfirmProps), defaultPropsOverride),
    emits: [...popoverEmits, ...popconfirmEmits],
    setup(props, { slots }) {
        // theme register
        onMounted(() => {
            useThemeRegister({
                key: 'Popconfirm',
                main: mainCssr
            });
        });
        const { cancelText, cancelDisabled, confirmText, confirmDisabled, hideIcon, onCancel, onConfirm } = toRefs(props);
        const popoverRef = ref();
        const handleCancel = () => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            if (onCancel.value) {
                const { value: cancel } = onCancel;
                const callback = yield cancel();
                if (!callback || callback === undefined) {
                    (_a = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
                }
            }
            else {
                (_b = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _b === void 0 ? void 0 : _b.hide();
            }
        });
        const handleConfirm = () => __awaiter(this, void 0, void 0, function* () {
            var _c, _d;
            if (onConfirm.value) {
                const { value: confirm } = onConfirm;
                const callback = yield confirm();
                if (!callback || callback === undefined) {
                    (_c = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _c === void 0 ? void 0 : _c.hide();
                }
            }
            else {
                (_d = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _d === void 0 ? void 0 : _d.hide();
            }
        });
        const getActionVNode = () => {
            const showCancel = cancelText.value !== null;
            const showConfirm = confirmText.value !== null;
            const hasActionSlot = !!slots.action;
            const needActionBlock = showCancel || showConfirm || hasActionSlot;
            return needActionBlock
                ? createVNode('div', {
                    class: 'mc-popconfirm__action'
                }, [
                    hasActionSlot
                        ? renderSlot(slots, 'action')
                        : createVNode(Fragment, null, [
                            showCancel ? createVNode(McButton, { size: 'small', ghost: true, disabled: cancelDisabled.value, onClick: handleCancel }, { default: () => cancelText.value }) : null,
                            showConfirm ? createVNode(McButton, { size: 'small', type: 'success', disabled: confirmDisabled.value, style: { marginLeft: '8px' }, onClick: handleConfirm }, { default: () => confirmText.value }) : null
                        ])
                ])
                : null;
        };
        return () => {
            const iconVNode = hideIcon.value ? null : getSlotFirstVNode(slots.icon) || createVNode(McIcon, { size: 22, color: '#f59e0b', style: { 'margin-right': '8px' } }, { default: () => createVNode(IconAlert) });
            const mergedProps = mergeProps(omit(props, Object.keys(popconfirmProps)), {
                ref: popoverRef,
                class: 'mc-popconfirm'
            });
            return createVNode(McPopover, mergedProps, {
                default: () => renderSlot(slots, 'default'),
                content: () => createVNode(Fragment, null, [
                    createVNode('div', {
                        class: 'mc-popconfirm__content'
                    }, [iconVNode, propsMergeSlots(props, slots, 'content')]),
                    getActionVNode()
                ])
            });
        };
    }
});
