import { ref, reactive, toRefs, nextTick, renderSlot, mergeProps, defineComponent, toRaw, onMounted, computed, isReactive } from 'vue';
import { useThemeRegister, createComponentVNode, createElementVNode, createDirectives } from '../_utils_';
import { CheckmarkSharp as IconCheck } from '@vicons/ionicons5';
import { useVirtualList } from '@vueuse/core';
import { omit } from 'lodash-es';
import { McPopover } from '../popover';
import { popoverProps, popoverEmits } from '../popover/interface';
import { McIcon } from '../icon';
import { popselectProps, popselectEmits } from './interface';
import { mainCssr, lightCssr, darkCssr } from './styles';
const defaultPropsOverride = {
    placement: {
        type: String,
        default: 'bottom'
    }
};
export default defineComponent({
    name: 'Popselect',
    props: Object.assign(Object.assign(Object.assign({}, popoverProps), popselectProps), defaultPropsOverride),
    emits: [...popoverEmits, ...popselectEmits],
    setup(props, { slots, attrs, emit }) {
        // theme register
        onMounted(() => {
            useThemeRegister({
                key: 'Popselect',
                main: mainCssr,
                light: lightCssr,
                dark: darkCssr
            });
        });
        const { value: valueVM, options, multiple, maxHeight, autoClose, autoScroll, truncate, matchTrigger, itemHeight, itemStyle } = toRefs(props);
        const popoverRef = ref();
        const cssVars = computed(() => {
            return {
                '--popselect-inner-max-width': typeof truncate.value === 'number' ? `${truncate.value}px` : '200px'
            };
        });
        let scrollToOption;
        const handleShow = () => {
            nextTick(() => {
                var _a;
                const index = (_a = options.value) === null || _a === void 0 ? void 0 : _a.findIndex(e => e.value === (multiple.value ? valueVM.value[0] : valueVM.value));
                index && scrollToOption(index);
            });
        };
        const handleHide = () => {
            if ((autoClose.value === undefined && !multiple.value) || autoClose.value) {
                requestAnimationFrame(() => {
                    var _a;
                    (_a = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
                });
            }
        };
        const createOptionVNode = (data) => {
            var _a;
            const { label, value, disabled } = data;
            const isDisabled = !!disabled;
            const isSelected = multiple.value ? valueVM.value.includes(value) : valueVM.value === value;
            const checkIconVNode = multiple.value && isSelected ? createComponentVNode(McIcon, { size: 16, style: 'margin-left: 8px' }, { default: () => createComponentVNode(IconCheck) }) : null;
            const option = toRaw((_a = options.value) === null || _a === void 0 ? void 0 : _a.find(e => e.value === value));
            const handleUpdateValue = (value) => {
                if (multiple.value) {
                    const index = valueVM.value.indexOf(value);
                    if (index === -1) {
                        valueVM.value.push(value);
                    }
                    else {
                        valueVM.value.splice(index, 1);
                    }
                    emit('update:value', toRaw(valueVM.value), option);
                }
                else {
                    emit('update:value', value, option);
                }
            };
            return createElementVNode('div', {
                class: ['mc-popselect-option', { 'mc-popselect-option--selected': isSelected, 'mc-popselect-option--disabled': isDisabled }],
                style: itemStyle.value,
                onClick: () => {
                    if (isDisabled)
                        return;
                    valueVM.value !== void 0 && handleUpdateValue(value);
                    emit('select', value, option);
                    handleHide();
                }
            }, [
                createElementVNode('div', {
                    class: 'mc-popselect-option__inner'
                }, [createElementVNode('div', { class: { truncate: truncate.value } }, [typeof label === 'string' ? label : label()], 2 /* PatchFlags.CLASS */), checkIconVNode])
            ], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */);
        };
        return () => {
            var _a;
            const subHeight = itemHeight.value;
            const listHeight = options.value ? Math.min(options.value.length * subHeight, (_a = maxHeight.value) !== null && _a !== void 0 ? _a : 0) : 0;
            const { list, containerProps, wrapperProps, scrollTo } = useVirtualList(isReactive(options.value) ? options.value : reactive(options.value), {
                // Keep `itemHeight` in sync with the item's row.
                itemHeight: subHeight
            });
            scrollToOption = scrollTo;
            const mergedProps = mergeProps(omit(props, Object.keys(popselectProps)), {
                ref_key: 'popoverRef',
                ref: popoverRef,
                class: 'mc-popselect',
                style: Object.assign({ padding: '0px', minWidth: matchTrigger.value ? 'none' : '110px' }, cssVars.value)
            });
            return createComponentVNode(McPopover, Object.assign(Object.assign({}, mergedProps), { onShow: (...args) => {
                    autoScroll.value && handleShow();
                    attrs.show && attrs.onShow(...args);
                } }), {
                default: () => renderSlot(slots, 'default'),
                content: () => createElementVNode('div', Object.assign(Object.assign({}, containerProps), { class: 'mc-popselect__content', style: { height: listHeight + 'px', overflow: 'auto', padding: '4px 4px 0px 4px' } }), [
                    createElementVNode('div', wrapperProps.value, createDirectives('v-for', list.value, item => createOptionVNode(item.data)), 4 /* PatchFlags.STYLE */)
                ], 4 /* PatchFlags.STYLE */)
            }, 16 /* PatchFlags.FULL_PROPS */);
        };
    }
});
