var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ref, toRefs, computed, watch, mergeProps, provide, createVNode, createTextVNode, nextTick, defineComponent, onMounted } from 'vue';
import { flatten, getSlotFirstVNode, kebabCaseEscape, useThemeRegister } from '../_utils_';
import { useElementBounding, throttledWatch } from '@vueuse/core';
import { tabsInjectionKey, tabPaneIKey, tabIKey, tabsProps } from './interface';
import McTab from './src/Tab';
import { mainCssr, lightCssr, darkCssr } from './styles';
export default defineComponent({
    name: 'Tabs',
    props: tabsProps,
    emits: ['update:value', 'tab-switch', 'tab-click'],
    setup(props, { slots, emit, expose }) {
        var _a;
        // theme register
        onMounted(() => {
            useThemeRegister({
                key: 'Tabs',
                main: mainCssr,
                light: lightCssr,
                dark: darkCssr
            });
        });
        const { value: valueVM, defaultTab, type, showLine, center, stretch, tabGap, animation, activeColor, barPosition, headerStyle, headerClass, contentStyle, contentClass, onBeforeTabSwitch } = toRefs(props);
        const activeTabName = ref((valueVM === null || valueVM === void 0 ? void 0 : valueVM.value) || ((_a = defaultTab === null || defaultTab === void 0 ? void 0 : defaultTab.value) !== null && _a !== void 0 ? _a : ''));
        const activeTabVNode = ref();
        const tabsElRef = ref();
        const headerElRef = ref();
        const barElRef = ref();
        const barUpdatedTimer = ref();
        const cssVars = computed(() => {
            var _a;
            return {
                // '--tab-default-color': defaultColor.value,
                '--tab-active-color': activeColor.value,
                '--tab-gap': stretch.value ? 0 : ((_a = tabGap.value) !== null && _a !== void 0 ? _a : 0) / 2 + 'px'
            };
        });
        if (valueVM) {
            watch(valueVM, name => {
                name && handleBeforeTabSwitch(name);
            });
        }
        if (type.value === 'bar' || type.value === 'segment') {
            if (animation.value === 'slide') {
                watch(activeTabVNode, () => {
                    nextTick(() => {
                        updateBarStyle();
                    });
                });
                const { top, right, width, height } = useElementBounding(headerElRef);
                throttledWatch([top, right, width, height], () => {
                    clearBarUpdatedTimer();
                    const { value: barEl } = barElRef;
                    barEl.style.transition = '0s';
                    updateBarStyle();
                }, {
                    throttle: 32
                });
            }
        }
        const callUpdateTab = (name) => {
            activeTabName.value = name;
            valueVM && emit('update:value', name);
            emit('tab-switch', name);
        };
        const handleTabClick = (name) => {
            emit('tab-click', name);
            handleBeforeTabSwitch(name);
        };
        const handleBeforeTabSwitch = (name) => __awaiter(this, void 0, void 0, function* () {
            // callback only when the value is changed
            if (activeTabName.value !== name) {
                if (onBeforeTabSwitch === null || onBeforeTabSwitch === void 0 ? void 0 : onBeforeTabSwitch.value) {
                    const { value: beforeTabSwitch } = onBeforeTabSwitch;
                    const callback = yield beforeTabSwitch(activeTabName.value, name);
                    if (callback || callback === undefined) {
                        callUpdateTab(name);
                    }
                }
                else {
                    callUpdateTab(name);
                }
            }
        });
        const clearBarUpdatedTimer = () => {
            window.clearTimeout(barUpdatedTimer.value);
            barUpdatedTimer.value = null;
        };
        const updateBarStyle = () => {
            if (!activeTabVNode.value)
                return;
            const { value: { el: activeTabEl } } = activeTabVNode;
            const { value: barEl } = barElRef;
            const { offsetWidth, offsetLeft } = activeTabEl;
            barEl.style.left = `${offsetLeft}px`;
            barEl.style.maxWidth = `${offsetWidth}px`;
            barUpdatedTimer.value = window.setTimeout(() => {
                barEl.style.transition = '0.2s';
            }, 64);
        };
        provide(tabsInjectionKey, activeTabName);
        const getTabVNode = (maybeTabPane) => {
            var _a, _b;
            const { children, props, type } = maybeTabPane;
            const { name, tabLabel = '', tabStyle = '', tabClass = '', disabled = false } = (_a = kebabCaseEscape(props)) !== null && _a !== void 0 ? _a : {};
            const isTab = type.iKey === tabIKey;
            const isActive = activeTabName.value === name;
            const isDisabled = typeof disabled === 'boolean' ? disabled : disabled === '';
            const tabVNode = createVNode(McTab, mergeProps(isTab ? (_b = maybeTabPane.props) !== null && _b !== void 0 ? _b : {} : { style: tabStyle }, {
                class: [{ 'mc-tabs-tab--active': isActive, 'mc-tabs-tab--disabled': isDisabled }, tabClass],
                onClick: () => {
                    if (isDisabled)
                        return;
                    name !== undefined && handleTabClick(name);
                }
            }), {
                default: () => {
                    var _a, _b, _c, _d;
                    if (isTab) {
                        return (_b = (_a = children === null || children === void 0 ? void 0 : children.default) === null || _a === void 0 ? void 0 : _a.call(children)) !== null && _b !== void 0 ? _b : null;
                    }
                    else {
                        return (_d = (_c = children === null || children === void 0 ? void 0 : children.tab) === null || _c === void 0 ? void 0 : _c.call(children)) !== null && _d !== void 0 ? _d : [createVNode('span', { class: 'mc-tabs-tab__label' }, [typeof tabLabel === 'string' ? createTextVNode(tabLabel) : tabLabel()])];
                    }
                }
            });
            if (isActive) {
                activeTabVNode.value = tabVNode;
            }
            return tabVNode;
        };
        const lineBarVNode = computed(() => {
            if (type.value === 'bar') {
                if (animation.value === 'slide') {
                    return createVNode('div', {
                        ref: barElRef,
                        class: ['mc-tabs__header-bar', `mc-tabs__header-bar--${barPosition.value}`]
                    });
                }
            }
            else if (type.value === 'segment') {
                if (animation.value === 'slide') {
                    return createVNode('div', {
                        ref: barElRef,
                        class: ['mc-tabs__header-bar']
                    });
                }
            }
            return null;
        });
        const tabsHeaderVNode = computed(() => {
            var _a, _b;
            // use IKey, ensure non-SpecificVNode element won't be rendered in header
            const [firstTabPane, maybeTabPanes] = getSlotFirstVNode(slots.default, [tabPaneIKey, tabIKey], true);
            if (!maybeTabPanes || maybeTabPanes.length === 0)
                return null;
            activeTabName.value = activeTabName.value || ((_b = (_a = firstTabPane === null || firstTabPane === void 0 ? void 0 : firstTabPane.props) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : '');
            return createVNode('div', {
                class: ['mc-tabs__header-scroll-content', headerClass === null || headerClass === void 0 ? void 0 : headerClass.value],
                style: headerStyle === null || headerStyle === void 0 ? void 0 : headerStyle.value
            }, maybeTabPanes.map(maybeTabPane => getTabVNode(maybeTabPane)));
        });
        expose({
            switchTo: handleBeforeTabSwitch,
            el: tabsElRef
        });
        return () => createVNode('div', {
            ref: tabsElRef,
            class: ['mc-tabs', `mc-tabs--${type.value}`],
            style: cssVars.value
        }, [
            createVNode('div', {
                ref: headerElRef,
                class: [
                    'mc-tabs__header',
                    {
                        'mc-tabs__header--center': center.value,
                        'mc-tabs__header--stretch': stretch.value,
                        'mc-tabs__header--with-line': type.value === 'segment' || type.value === 'empty' ? false : showLine.value
                    },
                    type.value === 'bar' || type.value === 'segment' ? `mc-tabs__header--bar-${animation.value}` : ''
                ]
            }, [lineBarVNode.value, tabsHeaderVNode.value]),
            slots.default && flatten(slots.default(), tabPaneIKey).length > 0
                ? createVNode('div', {
                    class: ['mc-tabs__content', contentClass === null || contentClass === void 0 ? void 0 : contentClass.value],
                    style: contentStyle.value
                }, flatten(slots.default(), tabIKey, true))
                : null
        ]);
    }
});
