import { InjectionKey, Ref, CSSProperties, VNodeChild, PropType } from 'vue';
import type { ElementClassSet, ElementStyleSet } from '../_utils_';
export declare const tabsInjectionKey: InjectionKey<Ref<string | number>>;
export declare const tabPaneIKey: unique symbol;
export declare const tabIKey: unique symbol;
export declare type TabsType = 'bar' | 'empty' | 'card' | 'segment';
export declare type TabPaneName = string | number;
export declare type OnBeforeTabSwitchImpl = (from: TabPaneName, to: TabPaneName) => Promise<boolean | undefined | void> | boolean | undefined | void;
export declare type MaybeTabPaneProps = TabPaneProps & TabProps & Record<string, unknown>;
export interface TabsExposeInstance {
    el: HTMLElement;
    switchTo: (key: string | number) => void;
}
export interface TabsProps {
    value?: TabPaneName;
    defaultTab?: TabPaneName;
    type?: TabsType;
    showLine?: boolean;
    stretch?: boolean;
    center?: boolean;
    tabGap?: number;
    animation?: 'slide' | 'scale' | 'fade';
    activeColor?: string;
    barPosition?: 'bottom' | 'top';
    headerStyle?: ElementStyleSet;
    headerClass?: ElementClassSet;
    contentStyle?: ElementStyleSet;
    contentClass?: ElementClassSet;
    onBeforeTabSwitch?: OnBeforeTabSwitchImpl;
}
export interface TabPaneProps {
    name?: TabPaneName;
    tabLabel?: string | (() => VNodeChild);
    tabStyle?: string | CSSProperties;
    tabClass?: string;
    disabled?: boolean;
    preload?: boolean;
    lazy?: boolean;
}
export interface TabProps {
    name?: string | number;
    disabled?: boolean;
}
export declare const tabsProps: {
    value: {
        type: PropType<TabPaneName | undefined>;
        default: undefined;
    };
    defaultTab: {
        type: PropType<TabPaneName | undefined>;
        default: undefined;
    };
    type: {
        type: PropType<TabsType | undefined>;
        default: string;
    };
    showLine: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    stretch: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    center: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    tabGap: {
        type: PropType<number | undefined>;
        default: number;
    };
    animation: {
        type: PropType<"scale" | "slide" | "fade" | undefined>;
        default: string;
    };
    activeColor: {
        type: PropType<string | undefined>;
        default: string;
    };
    barPosition: {
        type: PropType<"bottom" | "top" | undefined>;
        default: string;
    };
    headerStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    headerClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    contentStyle: {
        type: PropType<ElementStyleSet | undefined>;
        default: undefined;
    };
    contentClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    onBeforeTabSwitch: {
        type: PropType<OnBeforeTabSwitchImpl | undefined>;
        default: undefined;
    };
};
export declare const tabPaneProps: {
    name: {
        type: PropType<TabPaneName | undefined>;
        default: undefined;
    };
    tabLabel: {
        type: PropType<string | (() => VNodeChild) | undefined>;
        default: undefined;
    };
    tabStyle: {
        type: PropType<string | CSSProperties | undefined>;
        default: undefined;
    };
    tabClass: {
        type: PropType<string | undefined>;
        default: undefined;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    preload: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    lazy: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
export declare const tabProps: {
    name: {
        type: PropType<string | number | undefined>;
        default: undefined;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
