import { ref, toRefs, inject, computed, createVNode, createCommentVNode, renderSlot, vShow, withDirectives, defineComponent } from 'vue';
import { watchOnce } from '@vueuse/core';
import { tabsInjectionKey, tabPaneIKey, tabPaneProps } from '../interface';
export default defineComponent({
    name: 'TabPane',
    iKey: tabPaneIKey,
    props: tabPaneProps,
    setup(props, { slots }) {
        const { name, preload, lazy } = toRefs(props);
        const valueRef = inject(tabsInjectionKey, null);
        if (!valueRef) {
            throw new Error('[McTabPane]: McTabPane must be placed inside McTabs.');
        }
        const isActive = computed(() => {
            return (name === null || name === void 0 ? void 0 : name.value) === (valueRef === null || valueRef === void 0 ? void 0 : valueRef.value);
        });
        const hasRendered = ref(isActive.value);
        if (lazy.value && !hasRendered.value) {
            watchOnce(isActive, () => {
                hasRendered.value = true;
            });
        }
        return () => {
            const tabPaneVNode = createVNode('div', { class: 'mc-tab-pane' }, [renderSlot(slots, 'default')]);
            if (preload.value) {
                return withDirectives(tabPaneVNode, [[vShow, isActive.value]]);
            }
            else {
                if (lazy.value) {
                    return hasRendered.value ? withDirectives(tabPaneVNode, [[vShow, isActive.value]]) : createCommentVNode('v-if', true);
                }
                else {
                    return isActive.value ? tabPaneVNode : createCommentVNode('v-if', true);
                }
            }
        };
    }
});
