import { computed, toRefs, renderSlot, createVNode, defineComponent, onMounted } from 'vue';
import { getSlotFirstVNode, useColorFactory, useThemeRegister } from '../_utils_';
import { textLinkProps } from './interface';
import { mainCssr } from './styles/index';
const BASE_COLOR_MAP = {
    primary: {
        color: '#3b82f6'
    },
    success: {
        color: '#16a34a'
    },
    warning: {
        color: '#fb923c'
    },
    danger: {
        color: '#dc2626'
    },
    info: {
        color: '#6B7280'
    }
};
export default defineComponent({
    name: 'TextLink',
    props: textLinkProps,
    setup(props, { slots }) {
        onMounted(() => {
            useThemeRegister({
                key: 'TextLink',
                main: mainCssr
            });
        });
        const { type, to, underline, trigger, color, hoverColor, block, raw } = toRefs(props);
        const showUnderline = computed(() => {
            if (!underline.value)
                return '';
            switch (trigger.value) {
                case 'always':
                    return 'underline';
                case 'hover':
                    return 'underline-hover';
                default:
                    return '';
            }
        });
        const cssVars = computed(() => {
            const compositeInputColor = {
                color: color.value || BASE_COLOR_MAP[type.value].color
            };
            const { default: defaultColorSet, hover: hoverColorSet, active: activeColorSet, disabled: disabledColorSet } = useColorFactory(compositeInputColor, {
                hover: [0, 0, 0, 0.12]
            });
            return {
                '--text-link-default-color': color.value || defaultColorSet.color,
                '--text-link-hover-color': hoverColor.value || hoverColorSet.color,
                '--text-link-active-color': activeColorSet.color,
                '--text-link-disabled-color': disabledColorSet.color
            };
        });
        return () => {
            var _a, _b;
            const textContent = (_b = (_a = getSlotFirstVNode(slots.default)) === null || _a === void 0 ? void 0 : _a.children) !== null && _b !== void 0 ? _b : '';
            const isEmail = typeof textContent === 'string' && /^([A-Za-z0-9_\-\.])+\@([A-Za-z0-9_\-\.])+\.([A-Za-z]{2,4})$/.test(textContent);
            return createVNode('a', {
                class: ['mc-text-link', { 'mc-text-link--block': block.value }, showUnderline.value],
                style: cssVars.value,
                href: isEmail ? (!raw.value ? `mailto:${textContent}` : '#') : to.value
            }, [renderSlot(slots, 'default')]);
        };
    }
});
