import { computed, watch } from 'vue';
import { createGlobalState, useStorage, createEventHook, usePreferredDark } from '@vueuse/core';
import { defineTheme } from './_utils_';
const globalThemeState = createGlobalState(() => useStorage('meetcode-ui-theme-local-storage', 'light'));
export const globalTheme = globalThemeState();
export const isLight = computed(() => globalTheme.value === 'light');
export const isDark = computed(() => globalTheme.value === 'dark');
/**
 * User's integrate theme controller
 */
export function useThemeController(options = {}) {
    const { initialTheme, useOsTheme } = options;
    const themeChangeEventHook = createEventHook();
    const setTheme = (theme = 'light') => {
        if (globalTheme.value !== theme) {
            globalTheme.value = theme;
            themeChangeEventHook.trigger(theme);
        }
    };
    if (initialTheme) {
        setTheme(initialTheme);
    }
    else if (useOsTheme) {
        const isDark = usePreferredDark();
        watch(isDark, colorScheme => {
            if (colorScheme) {
                setTheme('dark');
            }
            else {
                setTheme('light');
            }
        }, { immediate: true });
    }
    return {
        current: globalTheme,
        isDark,
        isLight,
        setTheme,
        switchTheme() {
            if (isLight.value) {
                setTheme('dark');
            }
            else {
                setTheme('light');
            }
        },
        onThemeChange: themeChangeEventHook.on,
        defineTheme
    };
}
