import { defineComponent, renderSlot, mergeProps, onMounted } from 'vue';
import { propsMergeSlots, useThemeRegister, createComponentVNode } from '../_utils_';
import { omit } from 'lodash-es';
import { McPopover } from '../popover';
import { popoverProps } from '../popover/interface';
import { tooltipProps } from './interface';
import { lightCssr, darkCssr } from './styles';
export default defineComponent({
    name: 'Tooltip',
    props: Object.assign(Object.assign({}, popoverProps), tooltipProps),
    setup(props, { slots }) {
        // theme register
        onMounted(() => {
            useThemeRegister({
                key: 'Tooltip',
                light: lightCssr,
                dark: darkCssr
            });
        });
        return () => {
            const mergedProps = mergeProps(omit(props, Object.keys(tooltipProps)), {
                class: 'mc-tooltip'
            });
            return createComponentVNode(McPopover, mergedProps, {
                default: () => renderSlot(slots, 'default'),
                content: () => propsMergeSlots(props, slots, 'content'),
                _: 3 /* SlotFlags.FORWARDED */
            }, 16 /* PatchFlags.FULL_PROPS */);
        };
    }
});
