"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.camelToKebab = exports.kebabToCamel = exports.kebabCaseEscape = void 0;
/**
 * convert kebab case keyed object into camel case keyed object
 *
 * @param targetObj
 * @returns e.g. `{ 'current-count': 1 } --> { currentCount: 1 }`
 */
function kebabCaseEscape(targetObj) {
    if (typeof targetObj !== 'object' || !targetObj)
        return targetObj;
    const obj = {};
    for (const [key, value] of Object.entries(targetObj)) {
        if (key.indexOf('-') > -1) {
            const escapeKey = kebabToCamel(key);
            obj[escapeKey] = value;
        }
        else {
            obj[key] = value;
        }
    }
    return obj;
}
exports.kebabCaseEscape = kebabCaseEscape;
/**
 * convert kebab case string into camel case.
 *
 * e.g. `hello-world --> helloWorld`
 */
function kebabToCamel(str) {
    return str.split('-').reduce((prev, cur) => {
        return `${prev}${cur.charAt(0).toUpperCase()}${cur.slice(1)}`;
    });
}
exports.kebabToCamel = kebabToCamel;
/**
 * convert camel case string into kebab case.
 *
 * e.g. `helloWorld --> hello-world`
 */
function camelToKebab(str) {
    return str.replace(/([A-Z])/g, '-$1').toLowerCase();
}
exports.camelToKebab = camelToKebab;
