import { PatchFlags, SlotFlags } from './tsutils';
import type { Component, VNodeProps, VNode, VNodeChild, VNodeArrayChildren, HTMLAttributes, Slots, CSSProperties } from 'vue';
import type { IntrinsicElementAttributes, ElementClassSet, ElementStyleSet, SpecificVNode, Merge } from './tsutils';
declare type ComponentProps<T extends object> = (T & Record<string, unknown> & VNodeProps & Partial<{
    class: ElementClassSet;
    style: ElementStyleSet;
}>) | null;
declare type ElementProps<T extends keyof IntrinsicElementAttributes> = (IntrinsicElementAttributes[T] & Record<string, unknown> & VNodeProps) | null;
declare type ComponentChildren<S extends string> = SlotChildren<S> | ArrayChildren | null;
declare type SlotChildren<S extends string> = Partial<Merge<Record<S, () => VNodeChild>, {
    _: SlotFlags;
}>>;
declare type ArrayChildren = unknown[];
interface DirectivesConfig {
    condition?: boolean;
    node: VNodeChild | (() => VNodeChild);
    commentText?: string;
}
/**
 * 创建组件VNode
 * @param type 自定义组件
 * @param props 组件Props
 * @param children
 * @param patchFlag PatchFlags
 * @param dynamicProps
 * @returns VNode
 */
export declare function createComponentVNode<T extends object, S extends string = 'default'>(type: Component, props?: ComponentProps<T>, children?: ComponentChildren<S>, patchFlag?: PatchFlags, dynamicProps?: (keyof (T & HTMLAttributes & VNodeProps))[] | null): SpecificVNode<T>;
/**
 * 创建内置Dom元素VNode
 * @param type 元素类型
 * @param props 内置Props
 * @param children
 * @param patchFlag PatchFlags
 * @param dynamicProps
 * @returns VNode
 */
export declare function createElementVNode<T extends keyof IntrinsicElementAttributes>(type: T, props?: ElementProps<T>, children?: unknown, patchFlag?: PatchFlags, dynamicProps?: (keyof (IntrinsicElementAttributes[T] & VNodeProps))[] | null): VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>;
/**
 * 创建文本VNode
 */
export declare function createTextVNode(source: string | object | unknown, patch?: boolean): VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>;
/**
 * 创建Fragment
 * @param children Array
 * @param patchFlag 64 | 128 | 256
 * @returns Fragment VNode
 * @example
 * ```html
 * <div v-for="item in data">{{ item.id }}</div>
 * ```
 *
 * ```ts
 * createFragment(
 *     renderList(data, item => createElementVNode('div', null, item.id, PatchFlags.TEXT)),
 *     PatchFlags.UNKEYED_FRAGMENT
 * )
 * ```
 */
export declare function createFragment(children?: unknown[], patchFlag?: PatchFlags.STABLE_FRAGMENT | PatchFlags.KEYED_FRAGMENT | PatchFlags.UNKEYED_FRAGMENT): VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>;
/**
 * 创建基础过渡效果
 * @param name
 * @param effect
 * @param slots
 * @returns
 */
export declare function createTransition(name: string, effect: CSSProperties, slots: SlotChildren<'default'>): SpecificVNode<{
    name: string;
    enterFrom: CSSProperties;
    leaveTo: CSSProperties;
}>;
export declare function renderSlot(slots: Slots, name: string, props?: Record<string, unknown>, fallback?: () => VNodeArrayChildren, noSlotted?: boolean): VNode<import("vue").RendererNode, import("vue").RendererElement, {
    [key: string]: any;
}>;
/**
 * 通过 `v-for` `v-if` `v-show` 指令创建VNode
 * @param type
 * @param config
 */
export declare function createDirectives(type: 'v-if' | 'v-show', config: DirectivesConfig): VNodeChild;
export declare function createDirectives<S extends unknown>(type: 'v-for', source: S[], cb: (value: S, index: number) => VNodeChild, keyed?: boolean): VNodeChild;
export {};
