"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThemeMap = exports.useThemeRegister = void 0;
const vue_1 = require("vue");
const theme_1 = require("../theme");
const useLight = [];
const useDark = [];
function unregisterTheme(name) {
    var _a;
    const { states = [] } = (_a = exports.ThemeMap.get(name)) !== null && _a !== void 0 ? _a : {};
    states
        .filter(state => state.isMounted)
        .forEach(state => {
        state.theme.unmount();
        state.isMounted = false;
    });
}
function registerTheme(name, component) {
    var _a, _b, _c, _d;
    function getState(states, name) {
        return states.find(state => state.name === name);
    }
    const { inherit, states = [] } = (_a = exports.ThemeMap.get(name)) !== null && _a !== void 0 ? _a : {};
    const inheritStates = (_c = (inherit ? (_b = exports.ThemeMap.get(inherit)) === null || _b === void 0 ? void 0 : _b.states : [])) !== null && _c !== void 0 ? _c : [];
    if (component) {
        // 增量注册
        const state = (_d = getState(states, component)) !== null && _d !== void 0 ? _d : getState(inheritStates, component);
        if (state) {
            state.theme.mount({ id: state.key });
            state.isMounted = true;
        }
    }
    else {
        // 全量注册
        const mergedStates = [...inheritStates.filter(({ name }) => !!!getState(states, name)), ...states];
        mergedStates
            .filter(state => !state.isMounted)
            .forEach(state => {
            state.theme.mount({ id: state.key });
            state.isMounted = true;
        });
    }
}
function useLightTheme(id, theme) {
    const key = `Mc${id}-light`;
    if (useLight.findIndex(l => l.key === key) > -1)
        return;
    const state = {
        key,
        name: id,
        isMounted: false,
        isStatic: false,
        theme
    };
    useLight.push(state);
}
function useDarkTheme(id, theme) {
    const key = `Mc${id}-dark`;
    if (useDark.findIndex(d => d.key === key) > -1)
        return;
    const state = {
        key,
        name: id,
        isMounted: false,
        isStatic: false,
        theme
    };
    useDark.push(state);
}
function useThemeRegister(theme) {
    const { key, main, light, dark } = theme;
    main === null || main === void 0 ? void 0 : main.mount({ id: `Mc${key}` });
    light && useLightTheme(key, light);
    dark && useDarkTheme(key, dark);
    registerTheme(theme_1.globalTheme.value, key);
}
exports.useThemeRegister = useThemeRegister;
exports.ThemeMap = new Map();
exports.ThemeMap.set('light', { states: useLight });
exports.ThemeMap.set('dark', { states: useDark });
(0, vue_1.watch)(theme_1.globalTheme, (newTheme, oldTheme) => {
    unregisterTheme(oldTheme || '');
    registerTheme(newTheme);
}, { immediate: true });
