"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const core_1 = require("@vueuse/core");
const interface_1 = require("./interface");
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Anchor',
    props: interface_1.anchorProps,
    emits: ['change'],
    setup(props, { emit, expose }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Anchor',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const { options, bound, offsetTop, offsetBottom, type, showTrack, showMarker } = (0, vue_1.toRefs)(props);
        const activeKey = (0, vue_1.ref)();
        const indicatorBarOffsetTop = (0, vue_1.ref)(0);
        const topList = (0, vue_1.shallowReactive)({});
        const anchorElRef = (0, vue_1.ref)();
        const anchorLinkElRefs = (0, core_1.useTemplateRefsList)();
        const { top: anchorTop } = (0, core_1.useElementBounding)(anchorElRef);
        const callChange = (target) => {
            emit('change', target);
        };
        const setActiveLink = (key) => {
            activeKey.value = key;
            const activeLinkEl = anchorLinkElRefs.value.find(link => link.hash === key);
            activeLinkEl && setIndicatorBarOffset(activeLinkEl);
        };
        const setIndicatorBarOffset = (target) => {
            const { top } = target.getBoundingClientRect();
            indicatorBarOffsetTop.value = top - anchorTop.value;
        };
        const createAnchorLink = (option) => {
            const { href, title, children } = option;
            return (0, _utils_1.createElementVNode)('div', {
                class: ['mc-anchor-link', activeKey.value === href ? 'mc-anchor-link--active' : '']
            }, [
                (0, _utils_1.createElementVNode)('a', {
                    ref: anchorLinkElRefs.value.set,
                    class: 'mc-anchor-link-title',
                    href,
                    onClick: e => {
                        e.preventDefault();
                        const { hash } = e.target;
                        if (hash === activeKey.value)
                            return;
                        window.location.hash = hash;
                        const target = document.getElementById(hash.slice(1));
                        target === null || target === void 0 ? void 0 : target.scrollIntoView();
                        callChange(hash);
                        setIndicatorBarOffset(e.target);
                    }
                }, [typeof title === 'string' ? title : title === null || title === void 0 ? void 0 : title()], 8 /* PatchFlags.PROPS */, ['href']),
                (0, _utils_1.createDirectives)('v-for', children !== null && children !== void 0 ? children : [], item => createAnchorLink(item))
            ], 2 /* PatchFlags.CLASS */);
        };
        const setTargetElement = (options) => {
            const flattenData = (0, _utils_1.flattenTree)(options, 'children');
            flattenData.forEach(item => {
                const id = item.href;
                const el = document.getElementById(id.slice(1));
                if (el) {
                    const { top } = (0, core_1.useElementBounding)(el);
                    topList[id] = top;
                }
            });
        };
        (0, core_1.throttledWatch)(topList, () => {
            // console.log(Object.entries(topList).map(([key, value]) => value.value));
            const activeLink = Object.entries(topList).find(([key, item]) => item.value <= bound.value + offsetTop.value && item.value >= bound.value - offsetBottom.value);
            setActiveLink((activeLink === null || activeLink === void 0 ? void 0 : activeLink[0]) || '');
        }, {
            throttle: 64
        });
        (0, vue_1.watch)(options, () => {
            var _a;
            setTargetElement((_a = options.value) !== null && _a !== void 0 ? _a : []);
        }, {
            deep: true
        });
        (0, vue_1.onMounted)(() => {
            var _a;
            setTargetElement((_a = options.value) !== null && _a !== void 0 ? _a : []);
        });
        expose({
            el: anchorElRef.value,
            scrollTo(href) {
                const target = document.getElementById(href.slice(1));
                target === null || target === void 0 ? void 0 : target.scrollIntoView({
                    behavior: 'smooth'
                });
                callChange(href);
            }
        });
        // main logic...
        return () => {
            var _a;
            return (0, _utils_1.createElementVNode)('div', { ref_key: 'anchorElRef', ref: anchorElRef, class: ['mc-anchor', `mc-anchor-type-${type.value}`] }, [
                (0, _utils_1.createDirectives)('v-for', (_a = options.value) !== null && _a !== void 0 ? _a : [], item => createAnchorLink(item)),
                type.value === 'bar'
                    ? (0, _utils_1.createElementVNode)('div', { class: 'mc-anchor-indicator' }, [
                        showTrack.value ? (0, _utils_1.createElementVNode)('div', { class: 'mc-anchor-indicator-track' }) : null,
                        showMarker.value ? (0, _utils_1.createElementVNode)('div', { class: 'mc-anchor-indicator-marker', style: { top: `${indicatorBarOffsetTop.value + 4}px` } }, null, 4 /* PatchFlags.STYLE */) : null
                    ])
                    : null
            ], 2 /* PatchFlags.CLASS */);
        };
    }
});
