"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const interface_1 = require("./interface");
const core_1 = require("@vueuse/core");
const _utils_1 = require("../_utils_");
const color_1 = require("./color");
const styles_1 = require("./styles");
const SIZE_MAP = {
    mini: {
        height: '20px',
        padding: '0 4px',
        fontSize: '12px',
        iconSize: '14px',
        iconMargin: '2px'
    },
    small: {
        height: '26px',
        padding: '0 8px',
        fontSize: '13px',
        iconSize: '16px',
        iconMargin: '3px'
    },
    medium: {
        height: '32px',
        padding: '0 12px',
        fontSize: '14px',
        iconSize: '18px',
        iconMargin: '4px'
    },
    large: {
        height: '38px',
        padding: '0 16px',
        fontSize: '16px',
        iconSize: '20px',
        iconMargin: '5px'
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'Button',
    iKey: interface_1.buttonIKey,
    props: interface_1.buttonProps,
    setup(props, { slots, expose }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Button',
                main: styles_1.mainCssr
            });
        });
        const { type, size, disabled, ghost, dashed, render, round, circle, block, loading, iconRight, color, textColor, borderColor, colorSet, textColorSet, borderColorSet } = (0, vue_1.toRefs)(props);
        const buttonElRef = (0, vue_1.ref)();
        const isRippling = (0, vue_1.ref)(false);
        const isDefault = (0, vue_1.computed)(() => type.value === 'default');
        const isCustom = (0, vue_1.computed)(() => type.value === 'custom');
        const isNotNormal = (0, vue_1.computed)(() => render.value !== 'normal');
        const isTransparent = (0, core_1.or)(ghost, dashed, isNotNormal.value);
        const customWithoutColor = (0, core_1.and)(isCustom, (0, core_1.not)(color));
        const useDefaultColor = (0, core_1.and)(customWithoutColor, (0, core_1.not)(textColor));
        const useDefaultBorderColor = (0, core_1.and)(customWithoutColor, (0, core_1.not)(borderColor));
        const useDefaultBackgroundColor = (0, core_1.and)(customWithoutColor);
        const cssVars = (0, vue_1.computed)(() => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z;
            const { value: buttonColor } = color_1.buttonColorMap;
            const compositeInputColor = type.value === 'custom'
                ? {
                    color: textColor.value || color.value || buttonColor.custom.color,
                    borderColor: borderColor.value || color.value || buttonColor.custom.borderColor,
                    backgroundColor: color.value || buttonColor.custom.backgroundColor
                }
                : {
                    color: buttonColor[type.value].color,
                    borderColor: buttonColor[type.value].borderColor,
                    backgroundColor: buttonColor[type.value].backgroundColor
                };
            const { default: defaultColorSet, hover: hoverColorSet, active: activeColorSet, disabled: disabledColorSet } = (0, _utils_1.useColorFactory)(compositeInputColor);
            const { default: defaultButtonDefaultColorSet, hover: defaultButtonHoverColorSet, active: defaultButtonActiveColorSet, disabled: defaultButtonDisabledColorSet } = color_1.defaultButtonColorMap.value;
            const buttonSizeSet = SIZE_MAP[size.value];
            const sizeVars = {
                '--button-width': circle.value ? buttonSizeSet.height : block.value ? '100%' : 'initial',
                '--button-height': buttonSizeSet.height,
                '--button-padding': buttonSizeSet.padding,
                '--button-font-size': buttonSizeSet.fontSize,
                '--button-icon-size': buttonSizeSet.iconSize,
                '--button-icon-margin': buttonSizeSet.iconMargin,
                '--button-radius': circle.value ? '50%' : round.value ? buttonSizeSet.height : '3px'
            };
            const colorVars = isCustom.value
                ? {
                    '--button-default-color': (_b = (_a = textColorSet.value) === null || _a === void 0 ? void 0 : _a.default) !== null && _b !== void 0 ? _b : (useDefaultColor.value ? '#000' : (0, core_1.or)(textColor, isTransparent).value ? defaultColorSet.color : '#fff'),
                    '--button-default-border-color': (_d = (_c = borderColorSet.value) === null || _c === void 0 ? void 0 : _c.default) !== null && _d !== void 0 ? _d : (isNotNormal.value ? 'transparent' : useDefaultBorderColor.value ? '#e0e0e6' : defaultColorSet.borderColor),
                    '--button-default-background-color': (_f = (_e = colorSet.value) === null || _e === void 0 ? void 0 : _e.default) !== null && _f !== void 0 ? _f : (isTransparent.value ? 'transparent' : useDefaultBackgroundColor.value ? '#fff' : defaultColorSet.backgroundColor),
                    '--button-hover-color': (_h = (_g = textColorSet.value) === null || _g === void 0 ? void 0 : _g.hover) !== null && _h !== void 0 ? _h : (useDefaultColor.value ? '#059669' : (0, core_1.or)(textColor, isTransparent).value ? hoverColorSet.color : '#fff'),
                    '--button-hover-border-color': (_k = (_j = borderColorSet.value) === null || _j === void 0 ? void 0 : _j.hover) !== null && _k !== void 0 ? _k : (isNotNormal.value ? 'transparent' : useDefaultBorderColor.value ? '#10b981' : hoverColorSet.borderColor),
                    '--button-hover-background-color': (_m = (_l = colorSet.value) === null || _l === void 0 ? void 0 : _l.hover) !== null && _m !== void 0 ? _m : (isTransparent.value ? 'transparent' : useDefaultBackgroundColor.value ? '#fff' : hoverColorSet.backgroundColor),
                    '--button-active-color': (_p = (_o = textColorSet.value) === null || _o === void 0 ? void 0 : _o.active) !== null && _p !== void 0 ? _p : (useDefaultColor.value ? '#15803d' : (0, core_1.or)(textColor, isTransparent).value ? activeColorSet.color : '#fff'),
                    '--button-active-border-color': (_r = (_q = borderColorSet.value) === null || _q === void 0 ? void 0 : _q.active) !== null && _r !== void 0 ? _r : (isNotNormal.value ? 'transparent' : useDefaultBorderColor.value ? '#15803d' : activeColorSet.borderColor),
                    '--button-active-background-color': (_t = (_s = colorSet.value) === null || _s === void 0 ? void 0 : _s.active) !== null && _t !== void 0 ? _t : (isTransparent.value ? 'transparent' : useDefaultBackgroundColor.value ? '#fff' : activeColorSet.backgroundColor),
                    '--button-disabled-color': (_v = (_u = textColorSet.value) === null || _u === void 0 ? void 0 : _u.disabled) !== null && _v !== void 0 ? _v : (useDefaultColor.value ? '#aaa' : (0, core_1.or)(textColor, isTransparent).value ? disabledColorSet.color : '#fff'),
                    '--button-disabled-border-color': (_x = (_w = borderColorSet.value) === null || _w === void 0 ? void 0 : _w.disabled) !== null && _x !== void 0 ? _x : (isNotNormal.value ? 'transparent' : useDefaultBorderColor.value ? '#eee' : disabledColorSet.borderColor),
                    '--button-disabled-background-color': (_z = (_y = colorSet.value) === null || _y === void 0 ? void 0 : _y.disabled) !== null && _z !== void 0 ? _z : (isTransparent.value ? 'transparent' : useDefaultBackgroundColor.value ? '#fff' : disabledColorSet.backgroundColor)
                }
                : {
                    '--button-default-color': isDefault.value ? defaultButtonDefaultColorSet.color : isTransparent.value ? defaultColorSet.color : defaultButtonDefaultColorSet.backgroundColor,
                    '--button-default-border-color': isNotNormal.value ? 'transparent' : isDefault.value ? defaultButtonDefaultColorSet.borderColor : defaultColorSet.borderColor,
                    '--button-default-background-color': isTransparent.value ? 'transparent' : isDefault.value ? defaultButtonDefaultColorSet.backgroundColor : defaultColorSet.backgroundColor,
                    '--button-hover-color': isDefault.value ? defaultButtonHoverColorSet.color : isTransparent.value ? hoverColorSet.color : defaultButtonHoverColorSet.backgroundColor,
                    '--button-hover-border-color': isNotNormal.value ? 'transparent' : isDefault.value ? defaultButtonHoverColorSet.borderColor : hoverColorSet.borderColor,
                    '--button-hover-background-color': isTransparent.value ? 'transparent' : isDefault.value ? defaultButtonHoverColorSet.backgroundColor : hoverColorSet.backgroundColor,
                    '--button-active-color': isDefault.value ? defaultButtonActiveColorSet.color : isTransparent.value ? activeColorSet.color : defaultButtonActiveColorSet.backgroundColor,
                    '--button-active-border-color': isNotNormal.value ? 'transparent' : isDefault.value ? defaultButtonActiveColorSet.borderColor : activeColorSet.borderColor,
                    '--button-active-background-color': isTransparent.value ? 'transparent' : isDefault.value ? defaultButtonActiveColorSet.backgroundColor : activeColorSet.backgroundColor,
                    '--button-disabled-color': isDefault.value ? defaultButtonDisabledColorSet.color : isTransparent.value ? disabledColorSet.color : defaultButtonDisabledColorSet.backgroundColor,
                    '--button-disabled-border-color': isNotNormal.value ? 'transparent' : isDefault.value ? defaultButtonDisabledColorSet.borderColor : disabledColorSet.borderColor,
                    '--button-disabled-background-color': isTransparent.value ? 'transparent' : isDefault.value ? defaultButtonDisabledColorSet.backgroundColor : disabledColorSet.backgroundColor
                };
            return Object.assign(Object.assign(Object.assign({}, colorVars), sizeVars), { '--button-ripple-color': (0, _utils_1.setColorAlpha)(colorVars['--button-active-border-color'], 0), '--button-flex-direction': iconRight.value ? 'row-reverse' : 'row' });
        });
        const iconVNode = (0, vue_1.computed)(() => {
            return loading.value
                ? (0, vue_1.createVNode)('span', { class: ['mc-button__icon-loading', iconRight.value ? 'right' : 'left'] }, null, 2 /* PatchFlags.CLASS */)
                : slots.icon
                    ? (0, vue_1.createVNode)('span', { class: ['mc-button__icon', iconRight.value ? 'right' : 'left'] }, [(0, vue_1.renderSlot)(slots, 'icon')], 2 /* PatchFlags.CLASS */)
                    : null;
        });
        const contentVNode = (0, vue_1.computed)(() => {
            return slots.default ? (0, vue_1.createVNode)('span', { class: 'mc-button__content' }, [(0, vue_1.renderSlot)(slots, 'default')]) : null;
        });
        // expose({
        //     el: buttonElRef
        // });
        return () => (0, vue_1.createVNode)('button', {
            ref_key: 'buttonElRef',
            ref: buttonElRef,
            class: [
                'mc-button',
                `mc-button--${render.value}`,
                `mc-button--${type.value}`,
                `mc-button--${size.value}`,
                {
                    'mc-button--block': block.value,
                    'mc-button--disabled': disabled.value,
                    'mc-button--ghost': ghost.value,
                    'mc-button--dashed': dashed.value,
                    'mc-button--round': round.value,
                    'mc-button--circle': circle.value,
                    'mc-button--rippling': isRippling.value
                }
            ],
            style: cssVars.value,
            disabled: disabled.value,
            onMousedown() {
                if (isRippling.value)
                    isRippling.value = false;
            },
            onClick() {
                isRippling.value = true;
            },
            onAnimationend(e) {
                if (e.animationName === 'mc-button-border-ripple-out') {
                    isRippling.value = false;
                }
            }
        }, [iconVNode.value, contentVNode.value], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['disabled']);
    }
});
