"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const _utils_1 = require("../_utils_");
const styles_1 = require("./styles");
const icon_1 = require("./icon");
const interface_1 = require("./interface");
exports.default = (0, vue_1.defineComponent)({
    name: 'Checkbox',
    iKey: interface_1.checkboxIKey,
    props: interface_1.checkboxProps,
    emits: ['update:value'],
    setup(props, { slots, attrs, emit }) {
        var _a;
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Checkbox',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const key = (0, _utils_1.createKey)('checkbox');
        const { value: valueVM, label, size, checkedValue, uncheckedValue, disabled, indeterminate, checkedColor } = (0, vue_1.toRefs)(props);
        const { groupValue, groupCheckedColor, groupDisabled, updateGroupValue, BusSelectAll, BusMaxControl } = (_a = (0, vue_1.inject)(interface_1.checkboxGroupInjectionKey, null)) !== null && _a !== void 0 ? _a : {};
        const checkboxElRef = (0, vue_1.ref)();
        const internalValue = (0, vue_1.ref)(false);
        const internalDisabled = (0, vue_1.ref)(false);
        const scaleRatio = (0, vue_1.computed)(() => {
            switch (size.value) {
                case 'small':
                    return 0.8;
                case 'medium':
                    return 0.9;
                case 'large':
                    return 1;
            }
        });
        const mergedValue = (0, core_1.isDefined)(valueVM) ? valueVM : internalValue;
        const mergedDisabled = (0, core_1.or)(groupDisabled, disabled, internalDisabled);
        const mergedChecked = (0, vue_1.computed)(() => {
            if (groupValue === null || groupValue === void 0 ? void 0 : groupValue.value) {
                return groupValue.value.indexOf(mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) > -1;
            }
            return (mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) === checkedValue.value;
        });
        const mergedCheckedColor = (0, core_1.and)(groupCheckedColor, (0, core_1.not)(checkedColor)).value ? groupCheckedColor : checkedColor;
        const cssVars = (0, vue_1.computed)(() => {
            var _a, _b;
            return {
                '--checkbox-checked-color': (_a = mergedCheckedColor === null || mergedCheckedColor === void 0 ? void 0 : mergedCheckedColor.value) !== null && _a !== void 0 ? _a : '#10b981',
                '--checkbox-hover-color': ((_b = mergedCheckedColor === null || mergedCheckedColor === void 0 ? void 0 : mergedCheckedColor.value) !== null && _b !== void 0 ? _b : '#10b981') + '0f',
                '--checkbox-scale-size': `scale(${scaleRatio.value})`
            };
        });
        const callUpdateValue = (value) => {
            if (valueVM.value === undefined) {
                mergedValue.value = value;
            }
            emit('update:value', value);
        };
        const updateInternalDisabled = (reachMax) => {
            if (reachMax) {
                !mergedChecked.value && (internalDisabled.value = true);
            }
            else {
                internalDisabled.value && (internalDisabled.value = false);
            }
        };
        const handleChange = () => {
            if (updateGroupValue) {
                updateGroupValue(mergedValue.value);
            }
            else {
                callUpdateValue(mergedChecked.value ? uncheckedValue.value : checkedValue.value);
            }
        };
        if (BusMaxControl && BusSelectAll) {
            BusMaxControl.on(updateInternalDisabled);
            BusSelectAll.on((selectDisabled) => {
                if (!mergedChecked.value) {
                    if (!selectDisabled) {
                        !mergedDisabled.value && (updateGroupValue === null || updateGroupValue === void 0 ? void 0 : updateGroupValue(mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value, false));
                    }
                    else {
                        updateGroupValue === null || updateGroupValue === void 0 ? void 0 : updateGroupValue(mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value, false);
                    }
                }
            });
        }
        const labelVNode = (0, vue_1.computed)(() => {
            if (slots.default) {
                return (0, vue_1.createVNode)('span', null, [(0, vue_1.renderSlot)(slots, 'default')]);
            }
            else if (label === null || label === void 0 ? void 0 : label.value) {
                return (0, vue_1.createVNode)('span', null, [label.value]);
            }
            else {
                return null;
            }
        });
        return () => {
            const mergedProps = (0, vue_1.mergeProps)({
                ref: checkboxElRef,
                class: ['mc-checkbox', { 'mc-checkbox--checked': mergedChecked.value, 'mc-checkbox--disabled': mergedDisabled.value }],
                style: cssVars.value
            }, attrs);
            return (0, _utils_1.createElementVNode)('div', mergedProps, [
                (0, _utils_1.createElementVNode)('input', { class: 'mc-checkbox__input', value: mergedValue.value, id: key, type: 'checkbox', onChange: handleChange, checked: mergedChecked.value, disabled: mergedDisabled.value }, null, 8 /* PatchFlags.PROPS */, [
                    'value',
                    'checked',
                    'disabled'
                ]),
                (0, _utils_1.createElementVNode)('label', { class: 'mc-checkbox__label', for: key }, [
                    (0, _utils_1.createElementVNode)('span', {
                        style: {
                            background: mergedDisabled.value ? 'rgba(0, 0, 0, 0.02)' : '',
                            borderColor: mergedDisabled.value ? '#cccfdb' : ''
                        }
                    }, [(0, vue_1.createVNode)(indeterminate.value ? icon_1.IconIndeterminateMark : icon_1.IconCheckMark)], 4 /* PatchFlags.STYLE */),
                    labelVNode.value
                ])
            ], 16 /* PatchFlags.FULL_PROPS */);
        };
    }
});
