"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const Checkbox_1 = __importDefault(require("./Checkbox"));
const _utils_1 = require("../_utils_");
const interface_1 = require("./interface");
exports.default = (0, vue_1.defineComponent)({
    name: 'CheckboxGroup',
    props: interface_1.checkboxGroupPros,
    emits: ['update:value'],
    setup(props, { slots, emit, expose }) {
        const { value: valueVM, options, checkedColor, max, disabled } = (0, vue_1.toRefs)(props);
        const internalValue = (0, vue_1.ref)([]);
        const checkboxGroupElRef = (0, vue_1.ref)();
        const checkboxCount = (0, vue_1.computed)(() => {
            var _a, _b;
            const slotsCount = slots.default ? (0, _utils_1.flatten)(slots.default(), interface_1.checkboxIKey).length : 0;
            const optionsCount = (_b = (_a = options === null || options === void 0 ? void 0 : options.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
            return slotsCount + optionsCount;
        });
        const checkedCount = (0, vue_1.computed)(() => {
            var _a, _b;
            return (_b = (_a = mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
        });
        const status = (0, vue_1.computed)(() => {
            return {
                all: checkedCount.value === checkboxCount.value,
                indeterminate: checkedCount.value > 0 && checkedCount.value < checkboxCount.value
            };
        });
        const mergedValue = valueVM.value ? valueVM : internalValue;
        const callUpdateValue = (value) => {
            emit('update:value', mergedValue.value, value);
        };
        // due with max props, some logic...
        if ((mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) && (max === null || max === void 0 ? void 0 : max.value)) {
            (0, vue_1.watch)([checkboxCount, checkedCount, max], () => {
                void (0, vue_1.nextTick)(() => {
                    var _a;
                    if (((_a = mergedValue.value) === null || _a === void 0 ? void 0 : _a.length) === max.value) {
                        BusMaxControl.emit(true);
                    }
                    else {
                        BusMaxControl.emit(false);
                    }
                });
            }, {
                immediate: true
            });
        }
        const updateGroupValue = (value, call = true) => {
            if (mergedValue.value) {
                const index = mergedValue.value.indexOf(value !== null && value !== void 0 ? value : '');
                if (index === -1) {
                    mergedValue.value.push(value !== null && value !== void 0 ? value : '');
                }
                else {
                    mergedValue.value.splice(index, 1);
                }
                call && callUpdateValue(value);
            }
            else {
                call && callUpdateValue(value);
            }
        };
        const SelectAllEventBusKey = Symbol();
        const MaxControlEventBusKey = Symbol();
        const BusSelectAll = (0, core_1.useEventBus)(SelectAllEventBusKey);
        const BusMaxControl = (0, core_1.useEventBus)(MaxControlEventBusKey);
        (0, vue_1.provide)(interface_1.checkboxGroupInjectionKey, {
            groupValue: mergedValue,
            groupCheckedColor: checkedColor,
            groupDisabled: disabled,
            updateGroupValue,
            BusSelectAll,
            BusMaxControl
        });
        (0, vue_1.onUnmounted)(() => {
            BusMaxControl.reset();
            BusSelectAll.reset();
        });
        expose({
            selectAll(selectDisabled = true) {
                BusSelectAll.emit(selectDisabled);
                (mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) && callUpdateValue();
            },
            clear() {
                if (mergedValue === null || mergedValue === void 0 ? void 0 : mergedValue.value) {
                    mergedValue.value.length = 0;
                    callUpdateValue();
                }
            },
            status,
            el: checkboxGroupElRef
        });
        return () => {
            var _a, _b;
            return (0, vue_1.createVNode)('div', {
                ref: checkboxGroupElRef,
                class: 'mc-checkbox-group'
            }, [
                ...((_b = (_a = options.value) === null || _a === void 0 ? void 0 : _a.map(option => {
                    const { value, label, disabled } = option;
                    if (!label || typeof label === 'string') {
                        return (0, vue_1.createVNode)(Checkbox_1.default, { value, label, disabled });
                    }
                    else {
                        return (0, vue_1.createVNode)(Checkbox_1.default, { value, disabled }, { default: label });
                    }
                })) !== null && _b !== void 0 ? _b : []),
                (0, vue_1.renderSlot)(slots, 'default')
            ]);
        };
    }
});
