"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const core_1 = require("@vueuse/core");
const icon_1 = require("../icon");
const button_1 = require("../button");
const ionicons5_1 = require("@vicons/ionicons5");
const vueuc_1 = require("vueuc");
const interface_1 = require("./interface");
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Drawer',
    inheritAttrs: false,
    props: interface_1.drawerProps,
    setup(props, { slots, attrs, emit, expose }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Drawer',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const { show, size, appearDirection, wrapperClosable, shortcutKey, closeOnShortcut, closable, headerStyle, bodyStyle, headerClass, bodyClass, title, showHeader, pure, onBeforeLeave } = (0, vue_1.toRefs)(props);
        const key = (0, _utils_1.createKey)('drawer');
        const drawerElRef = (0, vue_1.ref)(null);
        const isMountDrawer = (0, vue_1.ref)(true);
        const isTopDrawer = (0, vue_1.computed)(() => topItem.value === key);
        const { add, remove, topItem } = (0, _utils_1.useSharedItems)();
        const magicKeys = (0, core_1.useMagicKeys)({
            passive: false,
            onEventFired(event) {
                isTopDrawer.value && event.stopImmediatePropagation();
            }
        });
        const keys = (0, vue_1.computed)(() => Array.from(magicKeys.current));
        let closeAction;
        (0, vue_1.watch)(show, () => {
            if (show.value) {
                isMountDrawer.value = true;
                add(key);
            }
            else {
                remove(key);
            }
        }, { immediate: true });
        const { pause, resume } = (0, core_1.pausableWatch)(magicKeys[shortcutKey.value], v => {
            if (v && show.value && closeOnShortcut.value && isTopDrawer.value) {
                callShortcutStroke();
                callUpdateShow(false);
            }
        });
        (0, vue_1.watch)(isTopDrawer, () => {
            if (!isTopDrawer.value) {
                pause();
            }
            else {
                resume();
            }
        }, { immediate: true });
        (0, core_1.onClickOutside)(drawerElRef, event => {
            if (wrapperClosable.value && isTopDrawer.value) {
                callWrapperClick(event);
                callUpdateShow(false);
            }
        });
        const callUpdateShow = (val) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!val) {
                const callback = yield ((_a = onBeforeLeave.value) === null || _a === void 0 ? void 0 : _a.call(onBeforeLeave, closeAction));
                if (callback)
                    return;
            }
            emit('update:show', val);
        });
        const callWrapperClick = (e) => {
            closeAction = 'wrapper';
            emit('wrapper-click', e);
        };
        const callShortcutStroke = () => {
            closeAction = 'shortcut';
            emit('shortcut-stroke', keys.value);
        };
        const callAfterEnter = () => {
            emit('after-enter');
        };
        const callAfterLeave = () => {
            isMountDrawer.value = false;
            emit('after-leave');
        };
        const callBeforeEnter = () => {
            emit('before-enter');
        };
        const cssVars = (0, vue_1.computed)(() => {
            const drawerSize = typeof size.value === 'number' ? `${size.value}px` : size.value;
            const drawerWidth = ['left', 'right'].includes(appearDirection.value) ? drawerSize : '100vw';
            const drawerHeight = ['top', 'bottom'].includes(appearDirection.value) ? drawerSize : '100vh';
            return {
                '--drawer-width': drawerWidth,
                '--drawer-height': drawerHeight,
                '--drawer-size': drawerSize
            };
        });
        const headerVNode = (0, vue_1.computed)(() => {
            var _a;
            const titleVNode = (0, vue_1.createVNode)('div', { class: 'mc-drawer-title' }, [typeof title.value === 'string' ? (0, vue_1.createTextVNode)(title.value) : (_a = title.value) === null || _a === void 0 ? void 0 : _a.call(title)]);
            const closeIconVNode = closable.value
                ? (0, vue_1.createVNode)(button_1.McButton, {
                    render: 'text',
                    class: 'mc-drawer-close-button',
                    style: { padding: '0 6px' },
                    onClick: () => {
                        closeAction = 'close';
                        callUpdateShow(false);
                    }
                }, {
                    icon: () => (0, vue_1.createVNode)(icon_1.McIcon, { size: 20 }, { default: () => (0, vue_1.createVNode)(ionicons5_1.Close) })
                })
                : null;
            return !pure.value && showHeader.value ? (0, vue_1.createVNode)('div', { class: ['mc-drawer__header', headerClass.value], style: headerStyle.value }, [slots.header ? (0, vue_1.renderSlot)(slots, 'header') : titleVNode, closeIconVNode]) : null;
        });
        const drawerVNode = (0, vue_1.computed)(() => {
            const mergedProps = (0, vue_1.mergeProps)({
                ref: drawerElRef,
                class: ['mc-drawer', pure.value ? 'mc-drawer--pure' : '', `mc-drawer--${appearDirection.value}`]
            }, attrs);
            return (0, vue_1.createVNode)('div', { class: 'mc-drawer-wrapper', style: cssVars.value }, [
                (0, vue_1.createVNode)('div', mergedProps, [headerVNode.value, (0, vue_1.createVNode)('div', { class: ['mc-drawer__body', bodyClass.value], style: bodyStyle.value }, [(0, vue_1.renderSlot)(slots, 'default')])])
            ]);
        });
        const drawerContainerVNode = (0, vue_1.computed)(() => {
            return (0, vue_1.createVNode)('div', { class: 'mc-drawer-container' }, [
                (0, vue_1.createVNode)(vue_1.Transition, { name: 'mc-drawer-mask-fade', appear: true }, {
                    default: () => (show.value ? (0, vue_1.createVNode)('div', { class: 'mc-drawer-mask' }) : null)
                }),
                (0, vue_1.createVNode)(vue_1.Transition, { name: `mc-drawer-appear-${appearDirection.value}`, appear: true, onBeforeEnter: callBeforeEnter, onAfterEnter: callAfterEnter, onAfterLeave: callAfterLeave }, {
                    default: () => (show.value ? drawerVNode.value : null)
                })
            ]);
        });
        (0, vue_1.provide)(interface_1.drawerInjectionKey, drawerElRef);
        expose({
            hide: () => callUpdateShow(false),
            el: drawerElRef
        });
        return () => (isMountDrawer.value ? (0, vue_1.createVNode)(vueuc_1.VLazyTeleport, { to: 'body', show: show.value }, { default: () => drawerContainerVNode.value }) : null);
    }
});
