import { PropType, VNodeChild, Ref, InjectionKey, CSSProperties } from 'vue';
import { ElementClassSet } from '../_utils_';
export declare type DrawerCloseAction = 'wrapper' | 'close' | 'shortcut';
export declare type DrawerAppearDirection = 'top' | 'right' | 'bottom' | 'left';
export declare type OnBeforeLeaveImpl = (action: DrawerCloseAction) => Promise<boolean | undefined | void> | boolean | undefined | void;
export declare type DrawerInjection = Ref<HTMLElement | null> | null;
export interface DrawerExposeInstance {
    hide: () => void;
    el: HTMLElement;
}
export interface DrawerProps {
    show?: boolean;
    size?: number | string;
    appearDirection?: DrawerAppearDirection;
    wrapperClosable?: boolean;
    shortcutKey?: string;
    closeOnShortcut?: boolean;
    closable?: boolean;
    headerStyle?: string | Partial<CSSProperties>;
    bodyStyle?: string | Partial<CSSProperties>;
    maskStyle?: string | Partial<CSSProperties>;
    headerClass?: ElementClassSet;
    bodyClass?: ElementClassSet;
    title?: string | (() => VNodeChild);
    showHeader?: boolean;
    pure?: boolean;
    onBeforeLeave?: OnBeforeLeaveImpl;
}
export interface DrawerObjectEmits {
    'onUpdate:show'?: (val: boolean) => void;
    onWrapperClick?: (e: MouseEvent) => void;
    onShortcutStroke?: (keys: string[]) => void;
    onAfterEnter?: () => void;
    onAfterLeave?: () => void;
    onBeforeEnter?: () => void;
}
export declare const drawerProps: {
    show: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    size: {
        type: PropType<string | number | undefined>;
        default: string;
    };
    appearDirection: {
        type: PropType<DrawerAppearDirection | undefined>;
        default: string;
    };
    wrapperClosable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    shortcutKey: {
        type: PropType<string | undefined>;
        default: string;
    };
    closeOnShortcut: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    closable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    headerStyle: {
        type: PropType<string | Partial<CSSProperties> | undefined>;
        default: undefined;
    };
    bodyStyle: {
        type: PropType<string | Partial<CSSProperties> | undefined>;
        default: undefined;
    };
    headerClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    bodyClass: {
        type: PropType<ElementClassSet>;
        default: undefined;
    };
    title: {
        type: PropType<string | (() => VNodeChild) | undefined>;
        default: string;
    };
    showHeader: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    pure: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    onBeforeLeave: {
        type: PropType<OnBeforeLeaveImpl | undefined>;
        default: undefined;
    };
};
export declare const drawerInjectionKey: InjectionKey<DrawerInjection>;
