"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const _utils_1 = require("../_utils_");
const styles_1 = require("./styles");
const interface_1 = require("./interface");
const icon_1 = require("../icon");
const _internal_1 = require("../_internal_");
const ionicons5_1 = require("@vicons/ionicons5");
const SIZE_MAP = {
    small: {
        wrapperPaddingX: 10,
        fontSize: 12,
        innerPaddingY: 4.5,
        innerLineHeight: 17,
        padding: '10px',
        wordCountFontSize: '10px',
        addonMargin: '4px'
    },
    medium: {
        wrapperPaddingX: 12,
        fontSize: 14,
        innerPaddingY: 5.5,
        innerLineHeight: 21,
        padding: '12px',
        wordCountFontSize: '12px',
        addonMargin: '4px'
    },
    large: {
        wrapperPaddingX: 14,
        fontSize: 16,
        innerPaddingY: 6.5,
        innerLineHeight: 25,
        padding: '14px',
        wordCountFontSize: '14px',
        addonMargin: '6px'
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'Input',
    props: interface_1.inputProps,
    emits: ['update:value', 'focus', 'blur', 'change', 'input', 'select', 'clear', 'password-visible-change'],
    setup(props, { slots, emit, expose }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Input',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const instance = (0, vue_1.getCurrentInstance)();
        const { value: valueVM, type, size, placeholder, disabled, focusOnTyping, autosize, resizable, clearable, wordCount, loading, passwordVisible, minRows, maxRows, maxLength, inputLimits, composed, inputCount, separator, rules } = (0, vue_1.toRefs)(props);
        // TODO: Error throwing utils
        // diagnosis composed input
        if (composed.value) {
            if (!Array.isArray(valueVM.value)) {
                throw new Error('[McInput]: Make sure provide an Array when using composed input.');
            }
            else if (valueVM.value.length !== inputCount.value) {
                throw new Error(`[McInput]: Make sure the length of the binding value you provide is equal to the inputCount value. value length: ${valueVM.value.length}, inputCount: ${inputCount.value}`);
            }
        }
        const internalValue = composed.value ? (0, vue_1.ref)(new Array(inputCount.value).fill('')) : (0, vue_1.ref)('');
        const mergedValue = (0, core_1.isDefined)(valueVM) ? valueVM : internalValue;
        const valueLength = (0, vue_1.computed)(() => (composed.value ? mergedValue.value.reduce((pre, cur) => pre + cur).length : mergedValue.value.toString().length));
        const inputRootElRef = (0, vue_1.ref)();
        const inputElRef = (0, vue_1.ref)();
        const textareaElRef = (0, vue_1.ref)();
        const inputInst = (0, vue_1.computed)(() => (type.value === 'textarea' ? textareaElRef.value : inputElRef.value));
        const inputElRefsList = (0, vue_1.ref)();
        const textareaElRefsList = (0, vue_1.ref)();
        const inputInstsList = (0, vue_1.computed)(() => (type.value === 'textarea' ? textareaElRefsList.value : inputElRefsList.value));
        const isHovered = (0, vue_1.ref)(false);
        const isFocused = (0, vue_1.ref)(false);
        const isShowPassword = (0, vue_1.ref)(false);
        const isCompositing = composed.value ? (0, vue_1.ref)(new Array(inputCount.value).fill(false)) : (0, vue_1.ref)(false);
        const showPlaceholder = (0, core_1.and)((0, core_1.not)(mergedValue), (0, core_1.or)(placeholder, slots['placeholder']), (0, core_1.not)(isCompositing));
        const showInputSuffix = (0, core_1.and)(type.value !== 'textarea', (0, core_1.or)(wordCount, loading, (0, core_1.and)(clearable, (0, core_1.not)(disabled)), (0, core_1.and)(type.value === 'password', passwordVisible.value !== 'none', (0, core_1.not)(disabled)), slots['suffix']));
        const showTextareaSuffix = (0, core_1.and)(type.value === 'textarea', wordCount);
        const textareaMinHeight = (0, vue_1.computed)(() => SIZE_MAP[size.value].innerLineHeight * (minRows.value || 1) + SIZE_MAP[size.value].innerPaddingY * 2);
        const textareaMaxHeight = (0, vue_1.computed)(() => SIZE_MAP[size.value].innerLineHeight * (maxRows.value || 1) + SIZE_MAP[size.value].innerPaddingY * 2);
        const textareaAutosizeHeight = (0, vue_1.ref)(textareaMinHeight.value);
        const inputAutosizeWidth = (0, vue_1.ref)(0);
        const validateHook = (0, core_1.createEventHook)();
        const errorMessage = (0, vue_1.ref)('');
        const isValid = (0, vue_1.ref)(true);
        const cssVars = (0, vue_1.computed)(() => {
            var _a;
            const { fontSize, innerPaddingY, innerLineHeight, wrapperPaddingX, padding, wordCountFontSize, addonMargin } = (_a = SIZE_MAP[size.value]) !== null && _a !== void 0 ? _a : SIZE_MAP.medium;
            return {
                '--input-textarea-resizable': resizable.value ? 'vertical' : 'none',
                '--input-textarea-min-height': `${textareaMinHeight.value}px`,
                '--input-textarea-max-height': maxRows.value ? `${textareaMaxHeight.value}px` : '',
                '--input-textarea-autosize-height': `${textareaAutosizeHeight.value}px`,
                '--input-autosize-width': inputAutosizeWidth.value ? `${inputAutosizeWidth.value}px` : '0px',
                '--input-font-size': `${fontSize}px`,
                '--input-el-padding': `${innerPaddingY}px 0px`,
                '--input-el-line-height': `${innerLineHeight}px`,
                '--input-wrapper-padding': `0px ${wrapperPaddingX}px`,
                '--input-padding': padding,
                '--input-height': `${innerPaddingY * 2 + innerLineHeight}px`,
                '--input-word-count-font-size': wordCountFontSize,
                '--input-prefix-margin': addonMargin,
                '--input-suffix-margin': addonMargin,
                '--input-border-color': isValid.value ? '#e0e0e6' : '#dc2626',
                '--input-active-border-color': isValid.value ? '#10b981' : '#dc2626',
                '--input-state-border-shadow-color': (0, _utils_1.setColorAlpha)(isValid.value ? '#10b981' : '#dc2626', 0.4)
            };
        });
        // some helpers
        const forceSetTextareaHeight = () => {
            var _a;
            (_a = inputInst.value) === null || _a === void 0 ? void 0 : _a.setAttribute('style', `height: ${textareaAutosizeHeight.value}px`);
        };
        const adjustTextareaHeight = () => {
            var _a;
            textareaAutosizeHeight.value = 0;
            if (resizable.value)
                (_a = inputInst.value) === null || _a === void 0 ? void 0 : _a.setAttribute('style', `height: ${textareaMinHeight.value}px`);
            (0, vue_1.nextTick)(() => {
                if (inputInst.value) {
                    const { scrollHeight } = inputInst.value;
                    textareaAutosizeHeight.value = scrollHeight;
                    // force set height when resize textarea manually
                    if (resizable.value) {
                        (0, vue_1.nextTick)(() => {
                            forceSetTextareaHeight();
                        });
                    }
                }
            });
        };
        const adjustInputWidth = () => {
            inputAutosizeWidth.value = 0;
            // TODO: autosize when composed
            (0, vue_1.nextTick)(() => {
                if (inputInst.value) {
                    const { scrollWidth } = inputInst.value;
                    inputAutosizeWidth.value = scrollWidth;
                }
            });
        };
        const autosizeWatcher = () => {
            if ((0, core_1.not)(autosize).value)
                return;
            if (type.value === 'textarea') {
                adjustTextareaHeight();
            }
            else {
                adjustInputWidth();
            }
        };
        const checkInput = (value, event) => {
            var _a;
            const result = [];
            for (const rule of (_a = inputLimits.value) !== null && _a !== void 0 ? _a : []) {
                switch (rule) {
                    case 'number':
                        result.push(/^\d+$/.test(value));
                        break;
                    case 'not-special':
                        result.push(!/[\W]/.test(value));
                        break;
                    case 'trim':
                        result.push(!value.startsWith(' ') && !value.endsWith(' '));
                        break;
                    case 'not-space':
                        result.push(value.indexOf(' ') === -1);
                        break;
                    default:
                        if (typeof rule === 'function') {
                            result.push(!!rule(value, event));
                        }
                        else if (Object.prototype.toString.call(rule) === '[object RegExp]') {
                            result.push(!!!value.replace(rule, ''));
                        }
                        else {
                            result.push(true);
                        }
                }
            }
            return result.every(flag => flag === true);
        };
        const validateInput = (rule, value = mergedValue.value || '', index) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const { regExp, validator, required, message } = rule;
            if (validator) {
                const result = yield validator(value, index);
                updateValidation(Object.prototype.toString.call(result) !== '[object Error]', (_a = result === null || result === void 0 ? void 0 : result.message) !== null && _a !== void 0 ? _a : '');
            }
            else if (regExp && typeof value === 'string') {
                const result = regExp.test(value);
                updateValidation(result, message || '');
            }
            else if (required) {
                const result = !!value;
                updateValidation(result, message || '');
            }
        });
        // watchers
        (0, vue_1.watch)(mergedValue, autosizeWatcher, {
            immediate: true
        });
        (0, vue_1.watch)(isShowPassword, flag => {
            emit('password-visible-change', flag);
        });
        (0, core_1.onStartTyping)(() => {
            if ((0, core_1.and)((0, core_1.not)(isFocused), focusOnTyping, (0, core_1.not)(disabled)).value) {
                focus(composed.value ? 0 : void 0);
            }
        });
        validateHook.on(({ trigger, value, index }) => {
            var _a, _b;
            const rulesList = (_b = (trigger ? (_a = rules.value) === null || _a === void 0 ? void 0 : _a.filter(rule => { var _a; return (_a = rule.trigger) === null || _a === void 0 ? void 0 : _a.includes(trigger); }) : rules.value)) !== null && _b !== void 0 ? _b : [];
            rulesList.forEach(rule => validateInput(rule, value, index));
        });
        const updateValue = (value) => {
            if ((0, core_1.isDefined)(valueVM)) {
                emit('update:value', value);
            }
            else {
                mergedValue.value = value;
            }
            emit('input', value);
        };
        const updateMultipleValue = (value, index) => {
            if ((0, core_1.isDefined)(valueVM)) {
                if (index !== void 0) {
                    valueVM.value[index] = value;
                }
                else if (value === '') {
                    valueVM.value.fill('');
                }
                emit('update:value', (0, vue_1.toRaw)(valueVM.value), index);
                emit('input', (0, vue_1.toRaw)(valueVM.value), index);
            }
            else {
                if (index !== void 0) {
                    mergedValue.value[index] = value;
                }
                else if (value === '') {
                    mergedValue.value.fill('');
                }
                emit('input', value, index);
            }
        };
        const updateValidation = (status, message) => {
            isValid.value = status;
            errorMessage.value = message;
        };
        const focus = (index) => {
            var _a, _b;
            if (index === void 0) {
                (_a = inputInst.value) === null || _a === void 0 ? void 0 : _a.focus();
            }
            else {
                (_b = inputElRefsList.value) === null || _b === void 0 ? void 0 : _b[index].focus();
            }
        };
        const blur = (index) => {
            var _a, _b;
            if (index === void 0) {
                (_a = inputInst.value) === null || _a === void 0 ? void 0 : _a.blur();
            }
            else {
                (_b = inputElRefsList.value) === null || _b === void 0 ? void 0 : _b[index].blur();
            }
        };
        const select = (index) => {
            var _a, _b;
            if (index === void 0) {
                (_a = inputInst.value) === null || _a === void 0 ? void 0 : _a.select();
            }
            else {
                (_b = inputElRefsList.value) === null || _b === void 0 ? void 0 : _b[index].select();
            }
        };
        // event methods
        const handleFocus = (payload, index) => {
            isFocused.value = true;
            validateHook.trigger({ trigger: 'focus', value: mergedValue.value, index });
            emit('focus', index);
        };
        const handleBlur = (payload, index) => {
            isFocused.value = false;
            validateHook.trigger({ trigger: 'blur', value: mergedValue.value, index });
            emit('blur', index);
        };
        const handleChange = (payload, index) => {
            var _a, _b, _c;
            const value = (composed.value ? (_b = (_a = inputInstsList.value) === null || _a === void 0 ? void 0 : _a[index]) === null || _b === void 0 ? void 0 : _b.value : (_c = inputInst.value) === null || _c === void 0 ? void 0 : _c.value) || '';
            validateHook.trigger({ trigger: 'change', value, index });
            emit('change', value, index);
        };
        const handleInput = (payload, index) => {
            var _a, _b, _c, _d;
            if (composed.value ? isCompositing.value[index] : isCompositing.value)
                return;
            const value = (composed.value ? (_b = (_a = inputInstsList.value) === null || _a === void 0 ? void 0 : _a[index]) === null || _b === void 0 ? void 0 : _b.value : (_c = inputInst.value) === null || _c === void 0 ? void 0 : _c.value) || '';
            if (checkInput(value, payload) || value === '') {
                validateHook.trigger({ trigger: 'input', value, index });
                composed.value ? updateMultipleValue(value, index) : updateValue(value);
            }
            else {
                (_d = instance === null || instance === void 0 ? void 0 : instance.proxy) === null || _d === void 0 ? void 0 : _d.$forceUpdate();
            }
        };
        const handleSelect = (payload, index) => {
            var _a, _b;
            const { selectionStart, selectionEnd, value = '' } = (_b = (composed.value ? (_a = inputInstsList.value) === null || _a === void 0 ? void 0 : _a[index] : inputInst.value)) !== null && _b !== void 0 ? _b : {};
            const selectedValue = value.slice(selectionStart || 0, selectionEnd || 0);
            validateHook.trigger({ trigger: 'select', value: selectedValue, index });
            emit('select', selectedValue, index);
        };
        const handleClear = () => {
            validateHook.trigger({ trigger: 'clear', value: '' });
            composed.value ? updateMultipleValue('') : updateValue('');
            emit('clear');
        };
        const handleCompositionStart = (payload, index) => {
            if (index !== void 0) {
                isCompositing.value[index] = true;
            }
            else {
                isCompositing.value = true;
            }
        };
        const handleCompositionEnd = (payload, index) => {
            if (index !== void 0) {
                isCompositing.value[index] = false;
            }
            else {
                isCompositing.value = false;
            }
            handleInput(payload, index);
        };
        // vnodes
        const inputVNode = (value, index) => {
            return (0, _utils_1.createElementVNode)('input', {
                ref_for: composed.value,
                ref_key: composed.value ? 'inputElRefsList' : 'inputElRef',
                ref: composed.value ? inputElRefsList : inputElRef,
                class: 'mc-input-el',
                type: isShowPassword.value ? 'text' : type.value,
                value,
                maxlength: maxLength.value,
                disabled: disabled.value,
                onFocus: payload => handleFocus(payload, index),
                onBlur: payload => handleBlur(payload, index),
                onChange: payload => handleChange(payload, index),
                onInput: payload => handleInput(payload, index),
                onSelect: payload => handleSelect(payload, index),
                onCompositionstart: payload => handleCompositionStart(payload, index),
                onCompositionend: payload => handleCompositionEnd(payload, index)
            }, null, 8 /* PatchFlags.PROPS */, ['type', 'maxlength', 'value', 'disabled']);
        };
        const textareaVNode = () => {
            return (0, _utils_1.createElementVNode)('textarea', {
                ref_key: 'textareaElRef',
                ref: textareaElRef,
                class: 'mc-input-el',
                value: mergedValue.value,
                maxlength: maxLength.value,
                disabled: disabled.value,
                onFocus: handleFocus,
                onBlur: handleBlur,
                onChange: handleChange,
                onInput: handleInput,
                onSelect: handleSelect,
                onCompositionstart: handleCompositionStart,
                onCompositionend: handleCompositionEnd
            }, null, 8 /* PatchFlags.PROPS */, ['maxlength', 'value', 'disabled']);
        };
        const innerVNode = (value, placeholder, index) => {
            const mergedShowPlaceholder = composed.value ? (0, core_1.and)((0, core_1.not)(mergedValue.value[index]), !!placeholder, (0, core_1.not)(isCompositing.value[index])).value : showPlaceholder.value;
            return (0, _utils_1.createElementVNode)('div', { class: 'mc-input__inner' }, [
                type.value === 'textarea' ? textareaVNode() : inputVNode(value, index),
                mergedShowPlaceholder ? (0, _utils_1.createElementVNode)('div', { class: 'mc-input-placeholder' }, [placeholder]) : null
            ]);
        };
        const passwordEyeVnode = () => {
            let visibleSwitchEvent = {};
            if (passwordVisible.value === 'click') {
                visibleSwitchEvent = {
                    onClick: () => {
                        isShowPassword.value = !isShowPassword.value;
                    }
                };
            }
            else if (passwordVisible.value === 'hover') {
                visibleSwitchEvent = {
                    onMouseenter: () => {
                        isShowPassword.value = true;
                    },
                    onMouseleave: () => {
                        isShowPassword.value = false;
                    }
                };
            }
            else if (passwordVisible.value === 'mousedown') {
                const mouseupEvent = () => {
                    isShowPassword.value = false;
                    document.removeEventListener('mouseup', mouseupEvent);
                };
                visibleSwitchEvent = {
                    onMousedown: () => {
                        isShowPassword.value = true;
                        document.addEventListener('mouseup', mouseupEvent);
                    }
                };
            }
            return (0, _utils_1.createComponentVNode)(icon_1.McIcon, Object.assign({ icon: isShowPassword.value ? ionicons5_1.EyeOutline : ionicons5_1.EyeOffOutline, class: 'mc-input-eye-icon' }, visibleSwitchEvent), null, 16 /* PatchFlags.FULL_PROPS */);
        };
        const wordCountVNode = () => {
            return (0, _utils_1.createElementVNode)('span', { class: 'mc-input-word-count' }, [
                maxLength.value ? `${valueLength.value} / ${maxLength.value}` : (0, _utils_1.createFragment)([(0, _utils_1.createTextVNode)(`${valueLength.value} / `, true), (0, _utils_1.createComponentVNode)(icon_1.McIcon, { icon: ionicons5_1.Infinite, style: 'margin-left: 3px' }, null, -1 /* PatchFlags.HOISTED */)])
            ]);
        };
        const clearIconVNode = () => {
            const showIcon = (0, core_1.and)(valueLength.value > 0, (0, core_1.or)(isFocused, isHovered)).value;
            return (0, _utils_1.createComponentVNode)(icon_1.McIcon, {
                icon: ionicons5_1.CloseCircle,
                class: 'mc-input-clear-icon',
                style: showIcon ? { opacity: 1, cursor: 'pointer' } : {},
                onClick: handleClear
            }, null, 8 /* PatchFlags.PROPS */, ['style']);
        };
        const multipleInnerVNode = () => {
            function createCountArray(value, count) {
                return Array.isArray(value) ? (value.length >= count ? value.slice(0, count) : [...value, ...new Array(count - value.length).fill('')]) : new Array(count).fill(value !== null && value !== void 0 ? value : '');
            }
            const separators = createCountArray(separator.value, (inputCount.value || 2) - 1);
            const placeholders = createCountArray(placeholder.value, inputCount.value || 2);
            // TODO: add key
            return (0, _utils_1.createFragment)(new Array(2 * inputCount.value - 1).fill(null).map((item, index) => {
                if (index % 2 === 0)
                    return innerVNode(mergedValue.value[index >> 1], placeholders[index >> 1], index >> 1);
                return (0, _utils_1.createElementVNode)('div', { class: 'mc-input__separator' }, separators[(index - 1) >> 1], 1 /* PatchFlags.TEXT */);
            }));
        };
        expose({
            el: inputRootElRef,
            focus,
            blur,
            select,
            resize: () => {
                if (composed.value)
                    return;
                autosizeWatcher();
            },
            setPasswordVisible: (visible) => {
                isShowPassword.value = visible;
            },
            reset: () => {
                updateValidation(true, '');
            },
            validate(trigger, callback) {
                return __awaiter(this, void 0, void 0, function* () {
                    if (typeof trigger === 'string') {
                        validateHook.trigger({ trigger, value: trigger === 'clear' ? '' : mergedValue.value });
                        yield (0, vue_1.nextTick)();
                        callback && callback(isValid.value);
                    }
                    else {
                        validateHook.trigger({});
                        yield (0, vue_1.nextTick)();
                        trigger && trigger(isValid.value);
                    }
                    return isValid.value;
                });
            }
        });
        return () => (0, _utils_1.createElementVNode)('div', {
            ref_key: 'inputRootElRef',
            ref: inputRootElRef,
            class: [
                'mc-input',
                `mc-input--${type.value}`,
                {
                    'mc-input--disabled': disabled.value,
                    'mc-input--focused': isFocused.value,
                    'mc-input--autosize': autosize.value,
                    'mc-input--resizable': (0, core_1.and)(type.value === 'textarea', resizable).value,
                    'mc-input--with-prepend': !!slots['prepend'],
                    'mc-input--with-append': !!slots['append'],
                    'mc-input--composed': composed.value
                }
            ],
            style: cssVars.value,
            onMousedown: payload => {
                if (disabled.value)
                    return;
                const { tagName } = payload.target;
                if (tagName !== 'INPUT' && tagName !== 'TEXTAREA') {
                    payload.preventDefault();
                    // TODO: when click the composed input, show auto focus on the nearest child input
                    if ((0, core_1.not)(isFocused).value)
                        focus(composed.value ? 0 : void 0);
                }
            },
            onMouseenter: () => {
                if (disabled.value)
                    return;
                isHovered.value = true;
            },
            onMouseleave: () => {
                if (disabled.value)
                    return;
                isHovered.value = false;
            }
        }, [
            slots['prepend'] ? (0, _utils_1.createElementVNode)('div', { class: 'mc-input-prepend' }, [(0, vue_1.renderSlot)(slots, 'prepend')]) : null,
            (0, _utils_1.createElementVNode)('div', { class: 'mc-input-wrapper' }, [
                slots['prefix'] ? (0, _utils_1.createElementVNode)('div', { class: 'mc-input__prefix' }, [(0, vue_1.renderSlot)(slots, 'prefix')]) : null,
                composed.value ? multipleInnerVNode() : innerVNode(mergedValue.value, (0, _utils_1.propsMergeSlots)(props, slots, 'placeholder')),
                showTextareaSuffix.value
                    ? (0, _utils_1.createElementVNode)('div', { class: 'mc-input__suffix' }, [wordCountVNode()])
                    : showInputSuffix.value
                        ? (0, _utils_1.createElementVNode)('div', { class: 'mc-input__suffix' }, [
                            // TODO: add slot data, e.g. isValid
                            slots['suffix'] ? (0, _utils_1.createElementVNode)('span', { class: 'mc-input-suffix-content' }, [(0, vue_1.renderSlot)(slots, 'suffix')]) : null,
                            (0, core_1.and)(clearable, (0, core_1.not)(disabled)).value ? clearIconVNode() : null,
                            (0, core_1.and)(type.value === 'password', passwordVisible.value !== 'none', (0, core_1.not)(disabled)).value ? passwordEyeVnode() : null,
                            loading.value ? (0, _utils_1.createComponentVNode)(_internal_1.McBaseLoading, { size: 14, stroke: 24 }, null, -1 /* PatchFlags.HOISTED */) : null,
                            wordCount.value ? wordCountVNode() : null
                        ])
                        : null
            ]),
            slots['append'] ? (0, _utils_1.createElementVNode)('div', { class: 'mc-input-append' }, [(0, vue_1.renderSlot)(slots, 'append')]) : null,
            (0, _utils_1.createTransition)('SlideInFromTop', { opacity: 0, transform: 'translateY(-3px)' }, {
                default: () => (0, _utils_1.createDirectives)('v-if', {
                    condition: (0, core_1.not)(isValid).value,
                    node: (0, _utils_1.createElementVNode)('div', { class: 'mc-input-valid-message' }, errorMessage.value, 1 /* PatchFlags.TEXT */)
                })
            })
        ], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */);
    }
});
