import type { PropType, RenderFunction } from 'vue';
import type { UISize, UnionOmit } from '../_utils_';
export interface InputSizeSet {
    fontSize: number;
    wordCountFontSize: string;
    padding: string;
    wrapperPaddingX: number;
    innerPaddingY: number;
    innerLineHeight: number;
    addonMargin: string;
}
export interface InputValidRule {
    message?: string;
    regExp?: RegExp;
    validator?: (value: string | string[], index?: number) => Error | boolean | void | Promise<Error | boolean | void>;
    trigger?: InputValidTrigger[];
    required?: boolean;
}
export declare type InputValidTrigger = 'input' | 'change' | 'focus' | 'blur' | 'clear' | 'select';
export declare type InputPlaceholder = string | (() => RenderFunction);
export declare type InputLimitType = 'trim' | 'number' | 'not-special' | 'not-space';
export declare type InputLimitRule = InputLimitType | RegExp | ((value: string, event: Event) => boolean);
export declare type InputSize = UnionOmit<UISize, 'mini'>;
export declare type InputSizeMap = Record<InputSize, InputSizeSet>;
export interface InputProps {
    value?: string | string[];
    type?: 'text' | 'password' | 'textarea';
    size?: InputSize;
    placeholder?: InputPlaceholder | InputPlaceholder[];
    disabled?: boolean;
    focusOnTyping?: boolean;
    autosize?: boolean;
    resizable?: boolean;
    clearable?: boolean;
    wordCount?: boolean;
    loading?: boolean;
    passwordVisible?: 'none' | 'click' | 'hover' | 'mousedown';
    minRows?: number;
    maxRows?: number;
    maxLength?: number;
    inputLimits?: InputLimitRule[];
    composed?: boolean;
    count?: number;
    separator?: string | string[];
    rules?: InputValidRule[];
}
export interface InputExposeInstance {
    focus: () => void;
    blur: () => void;
    select: () => void;
    setPasswordVisible: (visible: boolean) => void;
    resize: () => void;
    reset: () => void;
    validate(callback?: (isValid: boolean) => unknown): Promise<boolean>;
    validate(trigger?: InputValidTrigger, callback?: (isValid: boolean) => unknown): Promise<boolean>;
}
export declare const enum InputEventType {
    INSERT = "insertText",
    DELETE = "deleteContentBackward",
    PASTE = "insertFromPaste",
    INSERT_COMPOSITION = "insertCompositionText"
}
export declare const inputProps: {
    value: {
        type: PropType<string | string[] | undefined>;
        default: undefined;
    };
    type: {
        type: PropType<"textarea" | "text" | "password" | undefined>;
        default: string;
    };
    size: {
        type: PropType<InputSize | undefined>;
        default: string;
    };
    placeholder: {
        type: PropType<InputPlaceholder | InputPlaceholder[] | undefined>;
        default: string;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    focusOnTyping: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    resizable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    clearable: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    autosize: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    wordCount: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    loading: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    passwordVisible: {
        type: PropType<"none" | "hover" | "click" | "mousedown" | undefined>;
        default: string;
    };
    minRows: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    maxRows: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    maxLength: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    inputLimits: {
        type: PropType<InputLimitRule[] | undefined>;
        default: undefined;
    };
    composed: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    inputCount: {
        type: PropType<number | undefined>;
        default: number;
    };
    separator: {
        type: PropType<string | string[] | undefined>;
        default: undefined;
    };
    rules: {
        type: PropType<InputValidRule[] | undefined>;
        default: undefined;
    };
};
