"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const interface_1 = require("./interface");
const preset_1 = require("./preset");
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Layout',
    iKey: interface_1.layoutIKey,
    props: interface_1.layoutProps,
    setup(props, { slots }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Layout',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const { preset, siderRight, siderWidth, leftSiderWidth, rightSiderWidth, fixedSider, fixedLeftSider, fixedRightSider, fixedHeader, fixedFooter, siderStyle, leftSiderStyle, rightSiderStyle, siderClass, leftSiderClass, rightSiderClass, headerStyle, contentStyle, footerStyle, headerClass, contentClass, footerClass, showSider, showLeftSider, showRightSider, showHeader, showContent, showFooter } = (0, vue_1.toRefs)(props);
        const McLayout = (0, vue_1.resolveComponent)('McLayout', true);
        // main logic...
        return () => {
            var _a, _b, _c, _d, _e, _f;
            if (preset.value === 'two-column') {
                return (0, preset_1.createColumnLayout)({
                    siderPlacement: siderRight.value ? 'right' : 'left',
                    fixed: fixedSider.value,
                    siderWidth: siderWidth.value,
                    siderStyle: siderStyle.value,
                    siderClass: siderClass.value,
                    contentStyle: contentStyle.value,
                    contentClass: contentClass.value,
                    showSider: showSider.value,
                    showContent: showContent.value
                }, {
                    sider: () => (0, vue_1.renderSlot)(slots, 'sider'),
                    content: () => (0, vue_1.renderSlot)(slots, 'content')
                });
            }
            else if (preset.value === 'three-column') {
                return (0, preset_1.createColumnLayout)({
                    siderPlacement: 'left',
                    fixed: fixedLeftSider.value,
                    siderWidth: leftSiderWidth.value,
                    siderStyle: leftSiderStyle.value,
                    siderClass: leftSiderClass.value,
                    contentStyle: contentStyle.value,
                    contentClass: contentClass.value,
                    showSider: showLeftSider.value,
                    showContent: showContent.value
                }, {
                    sider: () => (0, vue_1.renderSlot)(slots, 'left-sider'),
                    content: () => (0, preset_1.createColumnLayout)({
                        siderPlacement: 'right',
                        fixed: fixedRightSider.value,
                        siderWidth: rightSiderWidth.value,
                        siderStyle: rightSiderStyle.value,
                        siderClass: rightSiderClass.value,
                        contentStyle: contentStyle.value,
                        contentClass: contentClass.value,
                        showSider: showRightSider.value,
                        showContent: showContent.value
                    }, {
                        sider: () => (0, vue_1.renderSlot)(slots, 'right-sider'),
                        content: () => (0, vue_1.renderSlot)(slots, 'content')
                    })
                });
            }
            else if (preset.value === 'full') {
                return (0, preset_1.createRowLayout)({
                    headerStyle: headerStyle.value,
                    contentStyle: contentStyle.value,
                    footerStyle: footerStyle.value,
                    headerClass: headerClass.value,
                    contentClass: contentClass.value,
                    footerClass: footerClass.value,
                    fixedHeader: fixedHeader.value,
                    fixedFooter: fixedFooter.value,
                    showHeader: showHeader.value,
                    showContent: showContent.value,
                    showFooter: showFooter.value
                }, {
                    header: () => (0, vue_1.renderSlot)(slots, 'header'),
                    content: () => (0, vue_1.renderSlot)(slots, 'content'),
                    footer: () => (0, vue_1.renderSlot)(slots, 'footer')
                });
            }
            else if (preset.value === 'holy') {
                return (0, preset_1.createRowLayout)({
                    headerStyle: headerStyle.value,
                    contentStyle: contentStyle.value,
                    footerStyle: footerStyle.value,
                    headerClass: headerClass.value,
                    contentClass: contentClass.value,
                    footerClass: footerClass.value,
                    fixedHeader: fixedHeader.value,
                    fixedFooter: fixedFooter.value,
                    showHeader: showHeader.value,
                    showContent: showContent.value,
                    showFooter: showFooter.value
                }, {
                    header: () => (0, vue_1.renderSlot)(slots, 'header'),
                    content: () => (0, preset_1.createColumnLayout)({
                        siderPlacement: 'left',
                        fixed: fixedLeftSider.value,
                        siderWidth: leftSiderWidth.value,
                        siderStyle: leftSiderStyle.value,
                        siderClass: leftSiderClass.value,
                        contentStyle: contentStyle.value,
                        contentClass: contentClass.value,
                        showSider: showLeftSider.value,
                        showContent: showContent.value
                    }, {
                        sider: () => (0, vue_1.renderSlot)(slots, 'left-sider'),
                        content: () => (0, preset_1.createColumnLayout)({
                            siderPlacement: 'right',
                            fixed: fixedRightSider.value,
                            siderWidth: rightSiderWidth.value,
                            siderStyle: rightSiderStyle.value,
                            siderClass: rightSiderClass.value,
                            contentStyle: contentStyle.value,
                            contentClass: contentClass.value,
                            showSider: showRightSider.value,
                            showContent: showContent.value
                        }, {
                            sider: () => (0, vue_1.renderSlot)(slots, 'right-sider'),
                            content: () => (0, vue_1.renderSlot)(slots, 'content')
                        })
                    }),
                    footer: () => (0, vue_1.renderSlot)(slots, 'footer')
                });
            }
            const headerVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.layoutHeaderIKey);
            const contentVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.layoutContentIKey);
            const footerVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.layoutFooterIKey);
            const layoutVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.layoutIKey);
            const siderVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.layoutSiderIKey);
            const basicColumnLayoutVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.basicColumnLayoutComponentIKey);
            const basicRowLayoutVNode = (0, _utils_1.getSlotFirstVNode)(slots.default, interface_1.basicRowLayoutComponentIKey);
            const needExtraLayout = siderVNode && !layoutVNode && (headerVNode || footerVNode);
            const children = [
                !needExtraLayout ? headerVNode : null,
                needExtraLayout ? (0, vue_1.createVNode)(McLayout, null, { default: () => [headerVNode, contentVNode, footerVNode] }) : (_b = (_a = (basicColumnLayoutVNode || basicRowLayoutVNode)) !== null && _a !== void 0 ? _a : layoutVNode) !== null && _b !== void 0 ? _b : contentVNode,
                !needExtraLayout ? footerVNode : null
            ];
            if (siderVNode) {
                const siderIndex = (0, _utils_1.getSlotVNodeIndex)(slots.default, (_c = siderVNode === null || siderVNode === void 0 ? void 0 : siderVNode.type) === null || _c === void 0 ? void 0 : _c.iKey);
                const layoutIndex = (0, _utils_1.getSlotVNodeIndex)(slots.default, (_d = layoutVNode === null || layoutVNode === void 0 ? void 0 : layoutVNode.type) === null || _d === void 0 ? void 0 : _d.iKey);
                const contentIndex = (0, _utils_1.getSlotVNodeIndex)(slots.default, (_e = contentVNode === null || contentVNode === void 0 ? void 0 : contentVNode.type) === null || _e === void 0 ? void 0 : _e.iKey);
                const compareCond = layoutIndex > -1 ? siderIndex > layoutIndex : contentIndex > -1 ? siderIndex > contentIndex : siderIndex === ((_f = slots.default) === null || _f === void 0 ? void 0 : _f.call(slots).length) - 1;
                if (compareCond) {
                    children.push((0, vue_1.createVNode)(siderVNode, { class: 'mc-layout-sider--right' }));
                }
                else {
                    children.unshift((0, vue_1.createVNode)(siderVNode, { class: 'mc-layout-sider--left' }));
                }
            }
            return (0, vue_1.createVNode)('div', { class: ['mc-layout', siderVNode ? `mc-layout--with-sider` : ''] }, children);
        };
    }
});
