"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRowLayout = exports.createColumnLayout = void 0;
const vue_1 = require("vue");
const Layout_1 = __importDefault(require("./Layout"));
const LayoutHeader_1 = __importDefault(require("./src/LayoutHeader"));
const LayoutContent_1 = __importDefault(require("./src/LayoutContent"));
const LayoutFooter_1 = __importDefault(require("./src/LayoutFooter"));
const LayoutSider_1 = __importDefault(require("./src/LayoutSider"));
const interface_1 = require("./interface");
const BasicColumnLayoutComponent = (props, { slots, attrs }) => {
    const { siderPlacement = 'left', fixed, siderWidth, siderStyle, contentStyle, contentClass, siderClass, showSider = true, showContent = true } = props;
    const children = [
        showSider
            ? (0, vue_1.createVNode)(LayoutSider_1.default, { width: siderWidth, style: siderStyle, class: siderClass }, {
                default: () => (0, vue_1.renderSlot)(slots, 'sider')
            })
            : null,
        showContent
            ? (0, vue_1.createVNode)(LayoutContent_1.default, { style: contentStyle, class: contentClass }, {
                default: () => (0, vue_1.renderSlot)(slots, 'content')
            })
            : null
    ];
    siderPlacement === 'right' && children.reverse();
    return (0, vue_1.createVNode)(Layout_1.default, { style: { height: '100%' } }, { default: () => children });
};
// @ts-ignore
BasicColumnLayoutComponent.iKey = interface_1.basicColumnLayoutComponentIKey;
const BasicRowLayoutComponent = (props, { slots }) => {
    const { showHeader = true, showContent = true, showFooter = true, fixedHeader, headerStyle, contentStyle, footerStyle, headerClass, contentClass, footerClass } = props;
    return (0, vue_1.createVNode)(Layout_1.default, { class: fixedHeader && showHeader ? 'mc-layout--header-fixed' : '' }, {
        default: () => [
            showHeader ? (0, vue_1.createVNode)(LayoutHeader_1.default, { class: headerClass, style: headerStyle }, { default: () => (0, vue_1.renderSlot)(slots, 'header') }) : null,
            showContent ? (0, vue_1.createVNode)(LayoutContent_1.default, { class: contentClass, style: contentStyle }, { default: () => (0, vue_1.renderSlot)(slots, 'content') }) : null,
            showFooter ? (0, vue_1.createVNode)(LayoutFooter_1.default, { class: footerClass, style: footerStyle }, { default: () => (0, vue_1.renderSlot)(slots, 'footer') }) : null
        ]
    });
};
// @ts-ignore
BasicRowLayoutComponent.iKey = interface_1.basicRowLayoutComponentIKey;
function createColumnLayout(props, slots) {
    return (0, vue_1.createVNode)(BasicColumnLayoutComponent, Object.assign({ class: 'mc-layout-column-preset' }, props), slots);
}
exports.createColumnLayout = createColumnLayout;
function createRowLayout(props, slots) {
    return (0, vue_1.createVNode)(BasicRowLayoutComponent, Object.assign({ class: 'mc-layout-row-preset' }, props), slots);
}
exports.createRowLayout = createRowLayout;
