"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../../_utils_");
const _utils_2 = require("../../_utils_");
const interface_1 = require("../interface");
const button_1 = require("../../button");
const icon_1 = require("../../icon");
const ionicons5_1 = require("@vicons/ionicons5");
exports.default = (0, vue_1.defineComponent)({
    name: 'LayoutSider',
    iKey: interface_1.layoutSiderIKey,
    props: interface_1.layoutSiderProps,
    emits: ['toggled'],
    setup(props, { slots, emit, expose }) {
        var _a;
        const { parent } = (_a = (0, vue_1.getCurrentInstance)()) !== null && _a !== void 0 ? _a : {};
        if (parent && parent.type.iKey !== interface_1.layoutIKey) {
            throw new Error('[McLayoutSider]: McLayoutHeader must be placed inside McLayout.');
        }
        const { width, bordered, collapsed, collapsable, collapsedWidth, triggerPosition, triggerType, transitionMode, onBeforeToggle } = (0, vue_1.toRefs)(props);
        const isCollapsed = (0, vue_1.ref)(!!collapsed.value);
        const mergedWidth = (0, vue_1.computed)(() => (0, _utils_1.cssUnitTransform)(isCollapsed.value ? collapsedWidth.value : width.value));
        const cssVars = (0, vue_1.computed)(() => {
            var _a;
            const { top, bottom } = (_a = triggerPosition.value) !== null && _a !== void 0 ? _a : {};
            return {
                '--layout-sider-width': mergedWidth.value,
                '--layout-sider-collapse-button-trigger-top': top || '50%',
                '--layout-sider-collapse-button-trigger-bottom': bottom || '',
                '--layout-sider-scroll-area-min-width': transitionMode.value === 'transform' ? (0, _utils_1.cssUnitTransform)(width.value) : ''
            };
        });
        const callToggle = () => __awaiter(this, void 0, void 0, function* () {
            var _b;
            const callback = yield ((_b = onBeforeToggle.value) === null || _b === void 0 ? void 0 : _b.call(onBeforeToggle, isCollapsed.value));
            if (callback)
                return;
            isCollapsed.value = !isCollapsed.value;
            emit('toggled', isCollapsed.value);
        });
        const buttonTriggerVNode = () => {
            return (0, _utils_2.createComponentVNode)(button_1.McButton, { circle: true, size: 'small' }, {
                icon: () => (0, _utils_2.createComponentVNode)(icon_1.McIcon, null, {
                    default: () => (0, _utils_2.createComponentVNode)(ionicons5_1.ChevronBackSharp)
                })
            });
        };
        const collapseTriggerVNode = () => {
            return (0, _utils_2.createElementVNode)('div', { class: `mc-layout-sider__collapse-${triggerType.value}-trigger`, onClick: callToggle }, [slots.trigger ? (0, vue_1.renderSlot)(slots, 'trigger') : triggerType.value === 'button' ? buttonTriggerVNode() : null]);
        };
        expose({
            toggle: () => {
                isCollapsed.value = !isCollapsed.value;
            }
        });
        // main logic...
        return () => {
            return (0, _utils_2.createElementVNode)('aside', {
                class: ['mc-layout-sider', bordered.value ? 'mc-layout-sider--bordered' : '', isCollapsed.value ? 'mc-layout-sider--collapsed' : ''],
                style: cssVars.value
            }, [(0, _utils_2.createElementVNode)('div', { class: 'mc-layout-sider__scroll-area' }, [(0, vue_1.renderSlot)(slots, 'default')]), collapsable.value ? collapseTriggerVNode() : null]);
        };
    }
});
