"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const interface_1 = require("./interface");
const utils_1 = require("./src/utils");
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Menu',
    iKey: interface_1.menuIKey,
    props: interface_1.menuProps,
    emits: ['update:value', 'update:expandKeys'],
    setup(props, { slots, emit, expose }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Menu',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const keyTree = slots.default ? (0, utils_1.createKeyTree)(slots.default()) : [];
        const { value: valueVM, expandKeys, disabled, indent, unique, collapsed, collapsedWidth, collapsedIconSize, horizontal, options } = (0, vue_1.toRefs)(props);
        const internalExpandKeys = (0, vue_1.ref)([]);
        const mergedExpandKeys = expandKeys.value ? expandKeys : internalExpandKeys;
        const menuUpdateKey = (0, vue_1.ref)(0);
        const menuElRef = (0, vue_1.ref)();
        const selfPadding = (0, vue_1.computed)(() => indent.value - 32);
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--menu-collapsed-width': collapsedWidth.value + 'px',
                '--menu-collapsed-icon-size': collapsedIconSize.value + 'px',
                '--menu-collapsed-padding': `0px ${(collapsedWidth.value - collapsedIconSize.value) / 2}px`
            };
        });
        // force rerender menu when options changed
        (0, vue_1.watch)(options, () => {
            menuUpdateKey.value++;
        });
        const callUpdateValue = (value) => {
            if (valueVM.value !== value) {
                emit('update:value', value);
            }
        };
        const callUpdateExpandKeys = (key) => {
            if (Array.isArray(key)) {
                mergedExpandKeys.value = key;
            }
            else if (mergedExpandKeys.value) {
                const index = mergedExpandKeys.value.findIndex(item => item === key);
                if (index > -1) {
                    mergedExpandKeys.value.splice(index, 1);
                }
                else {
                    mergedExpandKeys.value.push(key);
                }
            }
            emit('update:expandKeys', mergedExpandKeys.value);
        };
        (0, vue_1.provide)(interface_1.menuInjectionKey, {
            activeKey: valueVM,
            updateKey: callUpdateValue,
            expandedKeys: mergedExpandKeys,
            updateExpandKeys: callUpdateExpandKeys,
            keyTree,
            options,
            padding: selfPadding,
            isDisabled: disabled,
            isUnique: unique,
            isCollapsed: collapsed,
            isHorizontal: horizontal
        });
        expose({
            el: menuElRef,
            expand(key, autoSelect = false) {
                autoSelect && !Array.isArray(key) && callUpdateValue(key);
                const expandKeys = [...new Set((Array.isArray(key) ? key : [key]).map(key => { var _a; return (_a = (0, utils_1.findPath)(keyTree, key)) !== null && _a !== void 0 ? _a : []; }).flat())];
                callUpdateExpandKeys([...new Set([...mergedExpandKeys.value, ...expandKeys])]);
            },
            collapseAll() {
                callUpdateExpandKeys([]);
            }
        });
        // main logic...
        return () => (0, _utils_1.createElementVNode)('ul', {
            ref_key: 'menuElRef',
            ref: menuElRef,
            key: menuUpdateKey.value,
            class: ['mc-menu', collapsed.value ? 'mc-menu--collapsed' : '', horizontal.value ? 'mc-menu--horizontal' : '', disabled.value ? 'mc-menu--disabled' : ''],
            style: cssVars.value
        }, [options.value ? (0, _utils_1.createDirectives)('v-for', options.value, option => (0, utils_1.createMenu)(option)) : (0, vue_1.renderSlot)(slots, 'default')], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['key']);
    }
});
