import type { Key, ElementClassSet, ElementStyleSet } from '../_utils_';
import type { PropType, InjectionKey, Ref, VNodeChild, ComputedRef } from 'vue';
interface MenuInjection {
    activeKey: Ref<Key>;
    updateKey: (key: Key) => void;
    expandedKeys: Ref<Key[]>;
    updateExpandKeys: (key: Key | Key[]) => void;
    keyTree: KeyTree[];
    options: Ref<MenuOption[]>;
    padding: ComputedRef<number>;
    isDisabled: Ref<boolean>;
    isUnique: Ref<boolean>;
    isCollapsed: Ref<boolean>;
    isHorizontal: Ref<boolean>;
}
interface SubMenuInjection {
    padding: ComputedRef<number>;
    isDisabled: Ref<boolean>;
    isUnique: Ref<boolean>;
    hidePopover: () => void;
}
interface MenuGroupInjection {
    padding: ComputedRef<number>;
    isDisabled: Ref<boolean>;
    hidePopover: () => void;
}
export interface KeyTree {
    key: Key;
    children?: KeyTree[];
}
export declare const menuInjectionKey: InjectionKey<MenuInjection>;
export declare const subMenuInjectionKey: InjectionKey<SubMenuInjection>;
export declare const menuGroupInjectionKey: InjectionKey<MenuGroupInjection>;
export declare const menuIKey: unique symbol;
export declare const menuItemIKey: unique symbol;
export declare const menuItemGroupIKey: unique symbol;
export declare const subMenuIKey: unique symbol;
export interface MenuExposeInstance {
    el: HTMLElement;
    expand: (key: Key | Key[], autoSelect?: boolean) => void;
    collapseAll: () => void;
}
export interface MenuOption {
    key?: Key;
    label?: string | (() => VNodeChild);
    icon?: () => VNodeChild;
    indent?: number;
    disabled?: boolean;
    unique?: boolean;
    group?: boolean;
    children?: MenuOption[];
    extra?: Record<string, any>;
    style?: ElementStyleSet;
    class?: ElementClassSet;
}
export interface MenuProps {
    value?: Key;
    expandKeys?: Key[];
    disabled?: boolean;
    indent?: number;
    unique?: boolean;
    collapsed?: boolean;
    collapsedWidth?: number;
    collapsedIconSize?: number;
    horizontal?: boolean;
    options?: MenuOption[];
}
export interface MenuItemProps {
    disabled?: boolean;
    indent?: number | 'auto';
}
export interface MenuItemGroupProps {
    title?: string | (() => VNodeChild);
    disabled?: boolean;
    indent?: number | 'auto';
}
export interface SubMenuProps {
    title?: string | (() => VNodeChild);
    disabled?: boolean;
    indent?: number | 'auto';
    unique?: boolean;
}
export declare const menuProps: {
    value: {
        type: PropType<Key | undefined>;
        default: undefined;
    };
    expandKeys: {
        type: PropType<Key[] | undefined>;
        default: undefined;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    indent: {
        type: PropType<number | undefined>;
        default: number;
    };
    unique: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    collapsed: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    collapsedWidth: {
        type: PropType<number | undefined>;
        default: number;
    };
    collapsedIconSize: {
        type: PropType<number | undefined>;
        default: number;
    };
    horizontal: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    options: {
        type: PropType<MenuOption[] | undefined>;
        default: undefined;
    };
};
export declare const menuItemProps: {
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    indent: {
        type: PropType<number | "auto" | undefined>;
        default: undefined;
    };
};
export declare const menuItemGroupProps: {
    title: {
        type: PropType<string | (() => VNodeChild) | undefined>;
        default: undefined;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    indent: {
        type: PropType<number | "auto" | undefined>;
        default: undefined;
    };
};
export declare const subMenuProps: {
    title: {
        type: PropType<string | (() => VNodeChild) | undefined>;
        default: undefined;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    indent: {
        type: PropType<number | "auto" | undefined>;
        default: undefined;
    };
    unique: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
export {};
