"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../../_utils_");
const core_1 = require("@vueuse/core");
const interface_1 = require("../interface");
const tooltip_1 = require("../../tooltip");
exports.default = (0, vue_1.defineComponent)({
    name: 'MenuItem',
    props: interface_1.menuItemProps,
    iKey: interface_1.menuItemIKey,
    setup(props, { slots, attrs }) {
        var _a, _b, _c, _d;
        const instance = (0, vue_1.getCurrentInstance)();
        const key = (_a = instance === null || instance === void 0 ? void 0 : instance.vnode.key) !== null && _a !== void 0 ? _a : '';
        const isParentMenu = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.menuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentMenuItemGroup = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.menuItemGroupIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const { indent, disabled } = (0, vue_1.toRefs)(props);
        const { activeKey, updateKey, padding: menuPadding, isCollapsed, isDisabled: isMenuDisabled } = (_b = (0, vue_1.inject)(interface_1.menuInjectionKey, null)) !== null && _b !== void 0 ? _b : {};
        const { padding: subMenuPadding, isDisabled: isSubMenuDisabled, hidePopover: hideSubMenuPopover } = (_c = (0, vue_1.inject)(interface_1.subMenuInjectionKey, null)) !== null && _c !== void 0 ? _c : {};
        const { padding: menuItemGroupPadding, isDisabled: isMenuItemGroupDisabled, hidePopover: hideMenuItemGroupPopover } = (_d = (0, vue_1.inject)(interface_1.menuGroupInjectionKey, null)) !== null && _d !== void 0 ? _d : {};
        const isActive = (0, vue_1.computed)(() => !!(key && key === (activeKey === null || activeKey === void 0 ? void 0 : activeKey.value)));
        const selfPadding = (0, vue_1.computed)(() => (typeof indent.value === 'number' ? indent.value : isParentMenuItemGroup.value ? ((menuItemGroupPadding === null || menuItemGroupPadding === void 0 ? void 0 : menuItemGroupPadding.value) || 0) + 16 : ((isParentMenu.value ? menuPadding === null || menuPadding === void 0 ? void 0 : menuPadding.value : subMenuPadding === null || subMenuPadding === void 0 ? void 0 : subMenuPadding.value) || 0) + 32));
        const mergedDisabled = (0, core_1.or)(isMenuDisabled, isParentMenuItemGroup.value ? isMenuItemGroupDisabled === null || isMenuItemGroupDisabled === void 0 ? void 0 : isMenuItemGroupDisabled.value : isSubMenuDisabled === null || isSubMenuDisabled === void 0 ? void 0 : isSubMenuDisabled.value, disabled);
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--menu-item-padding-left': `${selfPadding.value}px`
            };
        });
        // main logic...
        return () => (0, _utils_1.createComponentVNode)(tooltip_1.McTooltip, { disabled: (0, core_1.or)((0, core_1.not)(isCollapsed), (0, core_1.not)(isParentMenu), mergedDisabled).value, content: () => (0, vue_1.renderSlot)(slots, 'default'), placement: 'right' }, {
            default: () => (0, _utils_1.createElementVNode)('li', {
                class: ['mc-menu-item', isActive.value ? 'mc-menu-item--active' : '', disabled.value ? 'mc-menu-item--disabled' : ''],
                style: cssVars.value,
                onClick: () => {
                    if (mergedDisabled.value)
                        return;
                    key && (updateKey === null || updateKey === void 0 ? void 0 : updateKey(key));
                    hideSubMenuPopover === null || hideSubMenuPopover === void 0 ? void 0 : hideSubMenuPopover();
                    hideMenuItemGroupPopover === null || hideMenuItemGroupPopover === void 0 ? void 0 : hideMenuItemGroupPopover();
                }
            }, [slots.icon ? (0, vue_1.createVNode)('div', { class: 'mc-menu-item__icon' }, [(0, vue_1.renderSlot)(slots, 'icon')]) : null, (0, vue_1.createVNode)('div', (0, vue_1.mergeProps)({ class: 'mc-menu-item__content' }, attrs), [(0, vue_1.renderSlot)(slots, 'default')])], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 16 /* PatchFlags.FULL_PROPS */),
            _: 3 /* SlotFlags.FORWARDED */
        }, 8 /* PatchFlags.PROPS */, ['disabled', 'content']);
    }
});
