"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../../_utils_");
const core_1 = require("@vueuse/core");
const interface_1 = require("../interface");
const popover_1 = require("../../popover");
const _transition_1 = require("../../_transition_");
exports.default = (0, vue_1.defineComponent)({
    name: 'MenuItemGroup',
    iKey: interface_1.menuItemGroupIKey,
    props: interface_1.menuItemGroupProps,
    setup(props, { slots, attrs }) {
        var _a, _b;
        const instance = (0, vue_1.getCurrentInstance)();
        const isParentMenu = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.menuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentSubMenu = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.subMenuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const { indent, disabled } = (0, vue_1.toRefs)(props);
        const { activeKey, padding: menuPadding, isCollapsed: isMenuCollapsed, isHorizontal: isMenuHorizontal, isDisabled: isMenuDisabled } = (_a = (0, vue_1.inject)(interface_1.menuInjectionKey, null)) !== null && _a !== void 0 ? _a : {};
        const { padding: subMenuPadding, isDisabled: isSubMenuDisabled } = (_b = (0, vue_1.inject)(interface_1.subMenuInjectionKey, null)) !== null && _b !== void 0 ? _b : {};
        const hasCollapsed = (0, vue_1.ref)(isMenuCollapsed === null || isMenuCollapsed === void 0 ? void 0 : isMenuCollapsed.value);
        const isActive = (0, vue_1.computed)(() => {
            const keys = (0, _utils_1.flattenWithOptions)({ slots, key: interface_1.menuItemIKey, infinity: true }).map(item => {
                return item.key;
            });
            return !!((activeKey === null || activeKey === void 0 ? void 0 : activeKey.value) && keys.includes(activeKey.value));
        });
        const selfPadding = (0, vue_1.computed)(() => (typeof indent.value === 'number' ? indent.value : isParentSubMenu.value ? ((subMenuPadding === null || subMenuPadding === void 0 ? void 0 : subMenuPadding.value) || 0) + 16 : ((menuPadding === null || menuPadding === void 0 ? void 0 : menuPadding.value) || 0) + 32));
        const mergedDisabled = (0, core_1.or)(isMenuDisabled, isSubMenuDisabled, disabled);
        const menuPopoverPlacement = (0, vue_1.computed)(() => {
            if (isMenuHorizontal === null || isMenuHorizontal === void 0 ? void 0 : isMenuHorizontal.value) {
                return 'bottom';
            }
            else if (isMenuCollapsed === null || isMenuCollapsed === void 0 ? void 0 : isMenuCollapsed.value) {
                return 'right-start';
            }
        });
        const menuPopoverDisabled = (0, core_1.or)(mergedDisabled, (0, core_1.not)((0, core_1.and)(isParentMenu, (0, core_1.or)(isMenuHorizontal, isMenuCollapsed))));
        const menuPopoverRef = (0, vue_1.ref)();
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--menu-item-group-padding-left': `${selfPadding.value}px`
            };
        });
        const handleHidePopover = () => {
            var _a;
            (_a = menuPopoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
        };
        (0, vue_1.provide)(interface_1.menuGroupInjectionKey, {
            padding: selfPadding,
            isDisabled: disabled,
            hidePopover: handleHidePopover
        });
        // main logic...
        return () => (0, _utils_1.createElementVNode)('li', {
            class: [
                'mc-menu-item-group',
                (0, core_1.and)(hasCollapsed, isParentMenu).value ? 'mc-menu-item-group--collapsed' : '',
                (0, core_1.and)(isActive, (0, core_1.not)(menuPopoverDisabled)).value ? 'mc-menu-item-group--child-active' : '',
                disabled.value ? 'mc-menu-item-group--disabled' : ''
            ]
        }, [
            (0, _utils_1.createComponentVNode)(popover_1.McPopover, {
                ref_key: 'menuPopoverRef',
                ref: menuPopoverRef,
                disabled: menuPopoverDisabled.value,
                placement: menuPopoverPlacement.value,
                withArrow: false,
                style: { padding: '4px 0', width: '200px', [menuPopoverPlacement.value === 'bottom' ? 'marginTop' : 'marginLeft']: '4px' },
                class: ['mc-menu-item-group mc-menu-item-group--dropdown']
            }, {
                content: () => (0, _utils_1.createElementVNode)('ul', (0, vue_1.mergeProps)({ class: 'mc-menu-item-group-children' }, attrs), [(0, vue_1.renderSlot)(slots, 'default')]),
                default: () => (0, _utils_1.createElementVNode)('div', { class: 'mc-menu-item-group-title', style: cssVars.value }, [(0, _utils_1.createElementVNode)('span', { class: 'mc-menu-item-group-title__content' }, [(0, _utils_1.propsMergeSlots)(props, slots, 'title')])], 16 /* PatchFlags.FULL_PROPS */)
            }, 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['disabled', 'placement']),
            (0, core_1.and)(isMenuHorizontal, isParentMenu).value
                ? null
                : (0, _utils_1.createComponentVNode)(_transition_1.McFadeInExpandTransition, {
                    onAfterLeave: () => (hasCollapsed.value = true),
                    onEnter: () => (hasCollapsed.value = false)
                }, {
                    default: () => (0, _utils_1.createDirectives)('v-if', {
                        condition: (0, core_1.not)((0, core_1.and)(isMenuCollapsed, isParentMenu)).value,
                        node: (0, _utils_1.createElementVNode)('ul', (0, vue_1.mergeProps)({ class: 'mc-menu-item-group-children' }, attrs), [(0, vue_1.renderSlot)(slots, 'default')])
                    }),
                    _: 3 /* SlotFlags.FORWARDED */
                })
        ], 2 /* PatchFlags.CLASS */);
    }
});
