"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../../_utils_");
const core_1 = require("@vueuse/core");
const interface_1 = require("../interface");
const utils_1 = require("./utils");
const icon_1 = require("../../icon");
const popover_1 = require("../../popover");
const _transition_1 = require("../../_transition_");
const ionicons5_1 = require("@vicons/ionicons5");
exports.default = (0, vue_1.defineComponent)({
    name: 'SubMenu',
    iKey: interface_1.subMenuIKey,
    props: interface_1.subMenuProps,
    setup(props, { slots, attrs }) {
        var _a, _b, _c, _d;
        const instance = (0, vue_1.getCurrentInstance)();
        const key = (_a = instance === null || instance === void 0 ? void 0 : instance.vnode.key) !== null && _a !== void 0 ? _a : '';
        const isParentMenu = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.menuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentSubMenu = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.subMenuIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const isParentMenuItemGroup = (0, vue_1.computed)(() => (0, _utils_1.checkParent)(interface_1.menuItemGroupIKey, instance === null || instance === void 0 ? void 0 : instance.parent));
        const { unique, indent, disabled } = (0, vue_1.toRefs)(props);
        const { activeKey, expandedKeys, updateExpandKeys, padding: menuPadding, keyTree, options, isUnique: isMenuUnique, isCollapsed: isMenuCollapsed, isHorizontal: isMenuHorizontal, isDisabled: isMenuDisabled } = (_b = (0, vue_1.inject)(interface_1.menuInjectionKey, null)) !== null && _b !== void 0 ? _b : {};
        const { padding: subMenuPadding, isUnique: isSubMenuUnique, isDisabled: isSubMenuDisabled, hidePopover: hideSubMenuPopover } = (_c = (0, vue_1.inject)(interface_1.subMenuInjectionKey, null)) !== null && _c !== void 0 ? _c : {};
        const { padding: menuItemGroupPadding, isDisabled: isMenuItemGroupDisabled, hidePopover: hideMenuItemGroupPopover } = (_d = (0, vue_1.inject)(interface_1.menuGroupInjectionKey, null)) !== null && _d !== void 0 ? _d : {};
        const isExpanded = (0, vue_1.computed)(() => expandedKeys === null || expandedKeys === void 0 ? void 0 : expandedKeys.value.includes(key || ''));
        const isActive = (0, vue_1.computed)(() => {
            const keys = (0, _utils_1.flattenWithOptions)({ slots, key: interface_1.menuItemIKey, infinity: true }).map(item => {
                return item.key;
            });
            return !!((activeKey === null || activeKey === void 0 ? void 0 : activeKey.value) && keys.includes(activeKey.value));
        });
        const selfPadding = (0, vue_1.computed)(() => (typeof indent.value === 'number' ? indent.value : isParentMenuItemGroup.value ? ((menuItemGroupPadding === null || menuItemGroupPadding === void 0 ? void 0 : menuItemGroupPadding.value) || 0) + 16 : ((isParentMenu.value ? menuPadding === null || menuPadding === void 0 ? void 0 : menuPadding.value : subMenuPadding === null || subMenuPadding === void 0 ? void 0 : subMenuPadding.value) || 0) + 32));
        const mergedDisabled = (0, core_1.or)(isMenuDisabled, isParentMenuItemGroup.value ? isMenuItemGroupDisabled === null || isMenuItemGroupDisabled === void 0 ? void 0 : isMenuItemGroupDisabled.value : isSubMenuDisabled === null || isSubMenuDisabled === void 0 ? void 0 : isSubMenuDisabled.value, disabled);
        const watchUnique = (0, core_1.or)((0, core_1.and)(isMenuUnique, isParentMenu), (0, core_1.and)(isSubMenuUnique, isParentSubMenu));
        const menuPopoverPlacement = (0, vue_1.computed)(() => {
            if (isMenuHorizontal === null || isMenuHorizontal === void 0 ? void 0 : isMenuHorizontal.value) {
                return isParentMenu.value ? 'bottom' : 'right-start';
            }
            else if (isMenuCollapsed === null || isMenuCollapsed === void 0 ? void 0 : isMenuCollapsed.value) {
                return 'right-start';
            }
        });
        const menuPopoverDisabled = (0, core_1.or)(mergedDisabled, (0, core_1.and)((0, core_1.not)(isMenuCollapsed), (0, core_1.not)(isMenuHorizontal)));
        const menuPopoverRef = (0, vue_1.ref)();
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--menu-submenu-padding-left': `${selfPadding.value}px`
            };
        });
        const handleHidePopover = () => {
            var _a;
            (_a = menuPopoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
            hideSubMenuPopover === null || hideSubMenuPopover === void 0 ? void 0 : hideSubMenuPopover();
            hideMenuItemGroupPopover === null || hideMenuItemGroupPopover === void 0 ? void 0 : hideMenuItemGroupPopover();
        };
        (0, vue_1.provide)(interface_1.subMenuInjectionKey, {
            padding: selfPadding,
            isDisabled: disabled,
            isUnique: unique,
            hidePopover: handleHidePopover
        });
        // main logic...
        return () => (0, _utils_1.createElementVNode)('li', { class: ['mc-sub-menu', (0, core_1.and)((0, core_1.not)(isExpanded), menuPopoverDisabled).value ? 'mc-sub-menu--collapsed' : '', isActive.value ? 'mc-sub-menu--child-active' : '', disabled.value ? 'mc-sub-menu--disabled' : ''] }, [
            (0, _utils_1.createComponentVNode)(popover_1.McPopover, {
                ref: menuPopoverRef,
                disabled: menuPopoverDisabled.value,
                placement: menuPopoverPlacement.value,
                withArrow: false,
                style: { padding: '4px 0', width: '200px', [menuPopoverPlacement.value === 'bottom' ? 'marginTop' : 'marginLeft']: '4px' },
                class: ['mc-sub-menu mc-sub-menu--dropdown', `mc-sub-menu--placement-${menuPopoverPlacement.value}`]
            }, {
                content: () => (0, _utils_1.createElementVNode)('ul', (0, vue_1.mergeProps)({ class: 'mc-sub-menu-children', style: { margin: 0 } }, attrs), [(0, vue_1.renderSlot)(slots, 'default')]),
                default: () => (0, _utils_1.createElementVNode)('div', {
                    class: 'mc-sub-menu-title',
                    style: cssVars.value,
                    onClick: () => {
                        var _a, _b, _c, _d;
                        if ((0, core_1.or)(isMenuCollapsed, isMenuHorizontal, mergedDisabled).value)
                            return;
                        if ((0, core_1.or)((0, core_1.not)(watchUnique), isExpanded).value) {
                            updateExpandKeys === null || updateExpandKeys === void 0 ? void 0 : updateExpandKeys(key);
                        }
                        else {
                            const keys = isParentMenu.value ? keyTree.map(item => item.children && item.key).filter(Boolean) : (_d = (_c = (_b = (0, utils_1.findParent)((_a = options === null || options === void 0 ? void 0 : options.value) !== null && _a !== void 0 ? _a : keyTree, key)) === null || _b === void 0 ? void 0 : _b.children) === null || _c === void 0 ? void 0 : _c.map((item) => item.key)) !== null && _d !== void 0 ? _d : [];
                            updateExpandKeys === null || updateExpandKeys === void 0 ? void 0 : updateExpandKeys([...expandedKeys === null || expandedKeys === void 0 ? void 0 : expandedKeys.value.filter(key => !keys.includes(key)), key]);
                        }
                    }
                }, [
                    slots.icon ? (0, _utils_1.createElementVNode)('div', { class: 'mc-sub-menu-title__icon' }, [(0, vue_1.renderSlot)(slots, 'icon')]) : null,
                    (0, _utils_1.createElementVNode)('span', { class: 'mc-sub-menu-title__content' }, [(0, _utils_1.propsMergeSlots)(props, slots, 'title')]),
                    (0, core_1.and)(isMenuHorizontal, isParentMenu).value
                        ? null
                        : (0, _utils_1.createComponentVNode)(icon_1.McIcon, { class: 'mc-sub-menu-title__arrow' }, {
                            default: () => (0, _utils_1.createComponentVNode)((0, core_1.or)(isMenuCollapsed, menuPopoverPlacement.value === 'right-start').value ? ionicons5_1.ChevronForwardOutline : ionicons5_1.ChevronUpOutline)
                        })
                ], 16 /* PatchFlags.FULL_PROPS */),
                _: 3 /* SlotFlags.FORWARDED */
            }, 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */ | 8 /* PatchFlags.PROPS */, ['disabled', 'placement']),
            (isMenuHorizontal === null || isMenuHorizontal === void 0 ? void 0 : isMenuHorizontal.value)
                ? null
                : (0, _utils_1.createComponentVNode)(_transition_1.McFadeInExpandTransition, null, {
                    default: () => (0, _utils_1.createDirectives)('v-if', {
                        condition: (0, core_1.and)(isExpanded, (0, core_1.not)(isMenuCollapsed)).value,
                        node: (0, _utils_1.createElementVNode)('ul', (0, vue_1.mergeProps)({ class: 'mc-sub-menu-children' }, attrs), [(0, vue_1.renderSlot)(slots, 'default')])
                    }),
                    _: 3 /* SlotFlags.FORWARDED */
                })
        ], 2 /* PatchFlags.CLASS */);
    }
});
