"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findParent = exports.findNode = exports.findPath = exports.createMenu = exports.createKeyTreeByOptions = exports.createKeyTree = void 0;
const vue_1 = require("vue");
const _utils_1 = require("../../_utils_");
const interface_1 = require("../interface");
const MenuItem_1 = __importDefault(require("./MenuItem"));
const MenuItemGroup_1 = __importDefault(require("./MenuItemGroup"));
const SubMenu_1 = __importDefault(require("./SubMenu"));
function createKeyTree(children, result = []) {
    children.forEach(vnode => {
        var _a, _b, _c, _d;
        const { iKey } = vnode.type;
        const subChildren = (_b = (_a = vnode.children).default) === null || _b === void 0 ? void 0 : _b.call(_a);
        if (iKey === interface_1.menuItemIKey) {
            result.push({ key: (_c = vnode.key) !== null && _c !== void 0 ? _c : '' });
        }
        else if (iKey === interface_1.subMenuIKey) {
            result.push({
                key: (_d = vnode.key) !== null && _d !== void 0 ? _d : '',
                children: createKeyTree(subChildren)
            });
        }
        else if (iKey === interface_1.menuItemGroupIKey) {
            const items = (0, _utils_1.flatten)(subChildren, [interface_1.subMenuIKey, interface_1.menuItemIKey]);
            createKeyTree(items, result);
        }
    });
    return result;
}
exports.createKeyTree = createKeyTree;
function createKeyTreeByOptions(options, result = []) {
    options.forEach(option => {
        const { key = '', children, group } = option;
        if (group) {
            const items = (children !== null && children !== void 0 ? children : []).map(({ key = '', children }) => ({ key, children }));
            createKeyTreeByOptions(items, result);
        }
        else if (children) {
            result.push({
                key,
                children: createKeyTreeByOptions(children !== null && children !== void 0 ? children : [])
            });
        }
        else {
            result.push({ key });
        }
    });
    return result;
}
exports.createKeyTreeByOptions = createKeyTreeByOptions;
function createMenu(option) {
    const { key, label, icon, indent, disabled, unique, group, children, style, class: itemClass } = option;
    if (group) {
        return (0, vue_1.createVNode)(MenuItemGroup_1.default, { indent, disabled, title: label, style, class: itemClass }, {
            default: () => (children !== null && children !== void 0 ? children : []).map(item => createMenu(item))
        });
    }
    else if (children) {
        return (0, vue_1.createVNode)(SubMenu_1.default, { key, indent, disabled, unique, title: label, style, class: itemClass }, {
            icon,
            default: () => children.map(item => createMenu(item))
        });
    }
    else {
        return (0, vue_1.createVNode)(MenuItem_1.default, { key, indent, disabled, style, class: itemClass }, {
            icon,
            default: typeof label === 'string' ? () => label : label
        });
    }
}
exports.createMenu = createMenu;
function findPath(tree, key, path = []) {
    for (let i = 0; i < tree.length; i++) {
        const tempPath = [...path];
        tempPath.push(tree[i].key);
        if (tree[i].key === key)
            return tempPath;
        if (tree[i].children) {
            const result = findPath(tree[i].children, key, tempPath);
            if (result)
                return result;
        }
    }
}
exports.findPath = findPath;
function findNode(tree, key) {
    for (let i = 0; i < tree.length; i++) {
        if (tree[i].key === key)
            return tree[i];
        if (tree[i].children) {
            const result = findNode(tree[i].children, key);
            if (result)
                return result;
        }
    }
}
exports.findNode = findNode;
function findParent(tree, key) {
    for (let i = 0; i < tree.length; i++) {
        if (tree[i].children) {
            const index = tree[i].children.findIndex((item) => item.key === key);
            if (index > -1)
                return tree[i];
            const result = findParent(tree[i].children, key);
            if (result)
                return result;
        }
    }
}
exports.findParent = findParent;
