"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const _internal_1 = require("../_internal_");
const interface_1 = require("./interface");
const icon_1 = require("../icon");
const ionicons5_1 = require("@vicons/ionicons5");
exports.default = (0, vue_1.defineComponent)({
    name: 'Message',
    props: interface_1.messageProps,
    emits: ['close'],
    setup(props, { slots, attrs, emit, expose }) {
        const { type, duration, closable, hoverAlive, html, card, icon, action } = (0, vue_1.toRefs)(props);
        const messageElRef = (0, vue_1.ref)();
        const messageCloseTimer = (0, vue_1.ref)();
        const autoClose = (0, vue_1.computed)(() => { var _a; return (_a = duration.value) !== null && _a !== void 0 ? _a : 3000 > 0; });
        const handleCloseMessage = () => {
            emit('close');
        };
        const startCloseTimer = () => {
            messageCloseTimer.value = window.setTimeout(() => {
                handleCloseMessage();
                clearCloseTimer();
            }, duration.value);
        };
        const clearCloseTimer = () => {
            window.clearTimeout(messageCloseTimer.value);
            messageCloseTimer.value = null;
        };
        if (autoClose.value) {
            startCloseTimer();
        }
        const iconVNode = (0, vue_1.computed)(() => {
            return (icon === null || icon === void 0 ? void 0 : icon.value)
                ? icon === null || icon === void 0 ? void 0 : icon.value()
                : type.value === 'loading'
                    ? (0, _utils_1.createComponentVNode)(_internal_1.McBaseLoading, { class: 'mc-message__icon', size: 16, stroke: 22 })
                    : (0, _utils_1.createComponentVNode)(icon_1.McIcon, {
                        size: 18,
                        class: 'mc-message__icon'
                    }, {
                        default: () => {
                            switch (type.value) {
                                case 'text':
                                    return (0, _utils_1.createComponentVNode)(ionicons5_1.AlertCircle);
                                case 'success':
                                    return (0, _utils_1.createComponentVNode)(ionicons5_1.CheckmarkCircle);
                                case 'warning':
                                    return (0, _utils_1.createComponentVNode)(ionicons5_1.Warning);
                                case 'info':
                                    return (0, _utils_1.createComponentVNode)(ionicons5_1.InformationCircle);
                                case 'error':
                                    return (0, _utils_1.createComponentVNode)(ionicons5_1.CloseCircleSharp);
                            }
                        }
                    });
        });
        const closableVNode = (0, vue_1.computed)(() => {
            if (action === null || action === void 0 ? void 0 : action.value)
                return null;
            return closable.value
                ? (0, _utils_1.createComponentVNode)(icon_1.McIcon, {
                    size: 18,
                    class: 'mc-message__close',
                    onClick() {
                        handleCloseMessage();
                    }
                }, { default: () => (0, _utils_1.createComponentVNode)(ionicons5_1.CloseOutline) })
                : null;
        });
        const contentVNode = (0, vue_1.computed)(() => {
            if (html === null || html === void 0 ? void 0 : html.value) {
                return (0, _utils_1.createElementVNode)('div', {
                    class: 'mc-message__content',
                    innerHTML: html.value
                }, null, 8 /* PatchFlags.PROPS */, ['innerHTML']);
            }
            else {
                return (0, _utils_1.createElementVNode)('div', { class: 'mc-message__content' }, [(0, vue_1.renderSlot)(slots, 'default')]);
            }
        });
        expose({
            close: handleCloseMessage,
            el: messageElRef
        });
        return () => {
            var _a;
            const mergedProps = (0, vue_1.mergeProps)({
                ref: messageElRef,
                class: ['mc-message', { 'mc-message--card': card.value }, `mc-message--${type.value}`],
                onMouseenter() {
                    hoverAlive.value && autoClose.value && clearCloseTimer();
                },
                onMouseleave() {
                    hoverAlive.value && autoClose.value && startCloseTimer();
                }
            }, attrs);
            return (0, _utils_1.createElementVNode)('div', mergedProps, [iconVNode.value, contentVNode.value, closableVNode.value, (_a = action === null || action === void 0 ? void 0 : action.value) === null || _a === void 0 ? void 0 : _a.call(action)], 2 /* PatchFlags.CLASS */ | 32 /* PatchFlags.HYDRATE_EVENTS */);
        };
    }
});
