"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const core_1 = require("@vueuse/core");
const vueuc_1 = require("vueuc");
const interface_1 = require("./interface");
const icon_1 = require("../icon");
const button_1 = require("../button");
const ionicons5_1 = require("@vicons/ionicons5");
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Modal',
    inheritAttrs: false,
    props: interface_1.modalProps,
    emits: ['update:show', 'wrapper-click', 'shortcut-stroke', 'after-enter', 'after-leave', 'before-enter', 'cancel', 'confirm'],
    setup(props, { slots, attrs, emit, expose }) {
        // theme register
        (0, _utils_1.useThemeRegister)({
            key: 'Modal',
            main: styles_1.mainCssr,
            light: styles_1.lightCssr,
            dark: styles_1.darkCssr
        });
        const { show, width, height, appearFromCursor, wrapperClosable, shortcutKey, closeOnShortcut, closable, headerStyle, bodyStyle, footerStyle, headerClass, bodyClass, footerClass, title, showHeader, showFooter, cancelText, confirmText, pure, position, animation, onBeforeLeave, appearX, appearY } = (0, vue_1.toRefs)(props);
        const key = (0, _utils_1.createKey)('modal');
        const modalElRef = (0, vue_1.ref)();
        const modalAppearXRef = (0, vue_1.ref)(0);
        const modalAppearYRef = (0, vue_1.ref)(0);
        const modalTransformOrigin = (0, vue_1.computed)(() => (appearFromCursor.value ? `${modalAppearXRef.value}px ${modalAppearYRef.value}px` : 'center center'));
        const modalTransitionName = (0, vue_1.computed)(() => (appearFromCursor.value ? 'mc-modal-scale' : ['scale', 'slide'].includes(animation.value) ? `mc-modal-${animation.value}` : 'mc-modal-scale'));
        const isTopModal = (0, vue_1.computed)(() => topItem.value === key);
        const isMountModal = (0, vue_1.ref)(true);
        const { add, remove, topItem } = (0, _utils_1.useSharedItems)();
        const { x, y } = (0, core_1.useMouse)();
        const { i18n } = (0, _utils_1.useI18n)('modal');
        const magicKeys = (0, core_1.useMagicKeys)({
            passive: false,
            onEventFired(event) {
                isTopModal.value && event.stopImmediatePropagation();
            }
        });
        const keys = (0, vue_1.computed)(() => Array.from(magicKeys.current));
        let closeAction;
        (0, vue_1.watch)(show, () => {
            if (show.value) {
                isMountModal.value = true;
                modalAppearXRef.value = appearX.value || x.value;
                modalAppearYRef.value = appearY.value || y.value;
                add(key);
            }
            else {
                remove(key);
            }
        }, { immediate: true });
        const { pause, resume } = (0, core_1.pausableWatch)(magicKeys[shortcutKey.value], v => {
            if (v && show.value && closeOnShortcut.value && isTopModal.value) {
                callShortcutStroke();
                callUpdateShow(false);
            }
        });
        (0, vue_1.watch)(isTopModal, () => {
            if (!isTopModal.value) {
                pause();
            }
            else {
                resume();
            }
        }, { immediate: true });
        (0, core_1.onClickOutside)(modalElRef, event => {
            if (wrapperClosable.value && isTopModal.value) {
                callWrapperClick(event);
                callUpdateShow(false);
            }
        });
        const callUpdateShow = (val) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!val) {
                const callback = yield ((_a = onBeforeLeave.value) === null || _a === void 0 ? void 0 : _a.call(onBeforeLeave, closeAction));
                if (callback)
                    return;
            }
            emit('update:show', val);
        });
        const callWrapperClick = (e) => {
            closeAction = 'wrapper';
            emit('wrapper-click', e);
        };
        const callShortcutStroke = () => {
            closeAction = 'shortcut';
            emit('shortcut-stroke', keys.value);
        };
        const callAfterEnter = () => {
            emit('after-enter');
        };
        const callAfterLeave = () => {
            isMountModal.value = false;
            emit('after-leave');
        };
        const callBeforeEnter = () => {
            emit('before-enter');
        };
        const callOnCancel = () => {
            closeAction = 'cancel';
            emit('cancel');
        };
        const callOnConfirm = () => {
            closeAction = 'confirm';
            emit('confirm');
        };
        const handleCancel = () => {
            callOnCancel();
            callUpdateShow(false);
        };
        const handleConfirm = () => {
            callOnConfirm();
            callUpdateShow(false);
        };
        const modalPosition = (0, vue_1.computed)(() => {
            const { top, right, bottom, left } = position.value || {};
            return {
                marginTop: top !== undefined ? (typeof top === 'number' ? `${top}px` : top) : 'auto',
                marginRight: right !== undefined ? (typeof right === 'number' ? `${right}px` : right) : 'auto',
                marginBottom: bottom !== undefined ? (typeof bottom === 'number' ? `${bottom}px` : bottom) : 'auto',
                marginLeft: left !== undefined ? (typeof left === 'number' ? `${left}px` : left) : 'auto'
            };
        });
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--modal-width': typeof width.value === 'number' ? `${width.value}px` : width.value,
                '--modal-height': typeof height.value === 'number' ? `${height.value}px` : height.value
            };
        });
        const headerVNode = (0, vue_1.computed)(() => {
            var _a;
            const titleVNode = (0, vue_1.createVNode)('div', { class: 'mc-modal-title' }, [typeof title.value === 'string' ? (0, vue_1.createTextVNode)(title.value) : (_a = title.value) === null || _a === void 0 ? void 0 : _a.call(title)]);
            const closeIconVNode = closable.value
                ? (0, vue_1.createVNode)(button_1.McButton, {
                    render: 'text',
                    class: 'mc-modal-close-button',
                    style: { padding: '0 6px' },
                    onClick: () => {
                        closeAction = 'close';
                        callUpdateShow(false);
                    }
                }, {
                    icon: () => (0, vue_1.createVNode)(icon_1.McIcon, { size: 20 }, { default: () => (0, vue_1.createVNode)(ionicons5_1.Close) })
                })
                : null;
            return !pure.value && showHeader.value ? (0, vue_1.createVNode)('div', { class: ['mc-modal__header', headerClass.value], style: headerStyle.value }, [slots.header ? (0, vue_1.renderSlot)(slots, 'header') : titleVNode, closeIconVNode]) : null;
        });
        const footerVNode = (0, vue_1.computed)(() => {
            return !pure.value && showFooter.value
                ? (0, vue_1.createVNode)('div', { class: ['mc-modal__footer', footerClass.value], style: footerStyle.value }, slots.footer
                    ? [(0, vue_1.renderSlot)(slots, 'footer')]
                    : [
                        cancelText.value !== null ? (0, vue_1.createVNode)(button_1.McButton, { ghost: true, onClick: handleCancel }, { default: () => cancelText.value || i18n('CancelButtonText') }) : null,
                        confirmText.value !== null ? (0, vue_1.createVNode)(button_1.McButton, { style: { marginLeft: '16px' }, type: 'success', onClick: handleConfirm }, { default: () => confirmText.value || i18n('ConfirmButtonText') }) : null
                    ])
                : null;
        });
        const modalVNode = (0, vue_1.computed)(() => {
            const mergedProps = (0, vue_1.mergeProps)({
                ref: modalElRef,
                class: ['mc-modal', pure.value ? 'mc-modal--pure' : ''],
                style: Object.assign(Object.assign({}, cssVars.value), modalPosition.value)
            }, attrs);
            return (0, vue_1.createVNode)('div', { class: 'mc-modal-wrapper', style: `transform-origin: ${modalTransformOrigin.value}` }, [
                (0, vue_1.createVNode)('div', mergedProps, [headerVNode.value, (0, vue_1.createVNode)('div', { class: ['mc-modal__body', bodyClass.value], style: bodyStyle.value }, [(0, vue_1.renderSlot)(slots, 'default')]), footerVNode.value])
            ]);
        });
        const modalContainerVNode = (0, vue_1.computed)(() => {
            return (0, vue_1.createVNode)('div', { class: 'mc-modal-container' }, [
                (0, vue_1.createVNode)(vue_1.Transition, { name: 'mc-modal-mask-fade', appear: true }, {
                    default: () => (show.value ? (0, vue_1.createVNode)('div', { class: 'mc-modal-mask' }) : null)
                }),
                (0, vue_1.createVNode)(vue_1.Transition, { name: modalTransitionName.value, appear: true, onBeforeEnter: callBeforeEnter, onAfterEnter: callAfterEnter, onAfterLeave: callAfterLeave }, {
                    default: () => (show.value ? modalVNode.value : null)
                })
            ]);
        });
        (0, vue_1.provide)(interface_1.modalInjectionKey, modalElRef);
        expose({
            hide: () => callUpdateShow(false),
            el: modalElRef
        });
        return () => (isMountModal.value ? (0, vue_1.createVNode)(vueuc_1.VLazyTeleport, { to: 'body', show: show.value }, { default: () => modalContainerVNode.value }) : null);
    }
});
