"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const lodash_es_1 = require("lodash-es");
const ionicons5_1 = require("@vicons/ionicons5");
const popover_1 = require("../popover");
const interface_1 = require("../popover/interface");
const button_1 = require("../button");
const icon_1 = require("../icon");
const interface_2 = require("./interface");
const styles_1 = require("./styles");
const defaultPropsOverride = {
    trigger: {
        type: String,
        default: 'click'
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'Popconfirm',
    props: Object.assign(Object.assign(Object.assign({}, interface_1.popoverProps), interface_2.popconfirmProps), defaultPropsOverride),
    emits: [...interface_1.popoverEmits, ...interface_2.popconfirmEmits],
    setup(props, { slots }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Popconfirm',
                main: styles_1.mainCssr
            });
        });
        const { cancelText, cancelDisabled, confirmText, confirmDisabled, hideIcon, onCancel, onConfirm } = (0, vue_1.toRefs)(props);
        const popoverRef = (0, vue_1.ref)();
        const handleCancel = () => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            if (onCancel.value) {
                const { value: cancel } = onCancel;
                const callback = yield cancel();
                if (!callback || callback === undefined) {
                    (_a = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
                }
            }
            else {
                (_b = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _b === void 0 ? void 0 : _b.hide();
            }
        });
        const handleConfirm = () => __awaiter(this, void 0, void 0, function* () {
            var _c, _d;
            if (onConfirm.value) {
                const { value: confirm } = onConfirm;
                const callback = yield confirm();
                if (!callback || callback === undefined) {
                    (_c = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _c === void 0 ? void 0 : _c.hide();
                }
            }
            else {
                (_d = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _d === void 0 ? void 0 : _d.hide();
            }
        });
        const getActionVNode = () => {
            const showCancel = cancelText.value !== null;
            const showConfirm = confirmText.value !== null;
            const hasActionSlot = !!slots.action;
            const needActionBlock = showCancel || showConfirm || hasActionSlot;
            return needActionBlock
                ? (0, vue_1.createVNode)('div', {
                    class: 'mc-popconfirm__action'
                }, [
                    hasActionSlot
                        ? (0, vue_1.renderSlot)(slots, 'action')
                        : (0, vue_1.createVNode)(vue_1.Fragment, null, [
                            showCancel ? (0, vue_1.createVNode)(button_1.McButton, { size: 'small', ghost: true, disabled: cancelDisabled.value, onClick: handleCancel }, { default: () => cancelText.value }) : null,
                            showConfirm ? (0, vue_1.createVNode)(button_1.McButton, { size: 'small', type: 'success', disabled: confirmDisabled.value, style: { marginLeft: '8px' }, onClick: handleConfirm }, { default: () => confirmText.value }) : null
                        ])
                ])
                : null;
        };
        return () => {
            const iconVNode = hideIcon.value ? null : (0, _utils_1.getSlotFirstVNode)(slots.icon) || (0, vue_1.createVNode)(icon_1.McIcon, { size: 22, color: '#f59e0b', style: { 'margin-right': '8px' } }, { default: () => (0, vue_1.createVNode)(ionicons5_1.AlertCircle) });
            const mergedProps = (0, vue_1.mergeProps)((0, lodash_es_1.omit)(props, Object.keys(interface_2.popconfirmProps)), {
                ref: popoverRef,
                class: 'mc-popconfirm'
            });
            return (0, vue_1.createVNode)(popover_1.McPopover, mergedProps, {
                default: () => (0, vue_1.renderSlot)(slots, 'default'),
                content: () => (0, vue_1.createVNode)(vue_1.Fragment, null, [
                    (0, vue_1.createVNode)('div', {
                        class: 'mc-popconfirm__content'
                    }, [iconVNode, (0, _utils_1.propsMergeSlots)(props, slots, 'content')]),
                    getActionVNode()
                ])
            });
        };
    }
});
