"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const vueuc_1 = require("vueuc");
const core_1 = require("@vueuse/core");
const interface_1 = require("./interface");
const interface_2 = require("../modal/interface");
const interface_3 = require("../drawer/interface");
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Popover',
    inheritAttrs: false,
    props: interface_1.popoverProps,
    emits: interface_1.popoverEmits,
    setup(props, { slots, attrs, expose, emit }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Popover',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const { trigger, placement, destroyWhenHide, zIndex, show, disabled, withArrow, showDelay, hideDelay, offset, wrapBoundary, matchTrigger, autoSync, title, followMode, x, y, teleport } = (0, vue_1.toRefs)(props);
        const showRef = trigger.value === 'manual' ? show : (0, vue_1.ref)(!!props.show);
        const followerRef = (0, vue_1.ref)(null);
        const followX = (0, vue_1.ref)(0);
        const followY = (0, vue_1.ref)(0);
        const mouseInFollowTrigger = (0, vue_1.ref)(false);
        const contentShowTimer = (0, vue_1.ref)();
        const contentHideTimer = (0, vue_1.ref)();
        const contentElRef = (0, vue_1.ref)(null);
        const mergedX = (0, vue_1.computed)(() => {
            switch (trigger.value) {
                case 'manual':
                    return x.value;
                case 'follow':
                    return followX.value;
                default:
                    return;
            }
        });
        const mergedY = (0, vue_1.computed)(() => {
            switch (trigger.value) {
                case 'manual':
                    return y.value;
                case 'follow':
                    return followY.value;
                default:
                    return;
            }
        });
        const emitThrottled = (0, vue_1.computed)(() => {
            return trigger.value === 'follow' && followMode.value === 'move';
        });
        const teleportDisabled = (0, vue_1.computed)(() => {
            const popover = (0, vue_1.inject)(interface_1.popoverInjectionKey, null);
            const modal = (0, vue_1.inject)(interface_2.modalInjectionKey, null);
            const drawer = (0, vue_1.inject)(interface_3.drawerInjectionKey, null);
            return !teleport.value || !!popover || !!modal || !!drawer;
        });
        (0, core_1.onClickOutside)(contentElRef, (e) => {
            var _b, _c;
            if (trigger.value === 'click' && !((_c = (_b = triggerVNode.value) === null || _b === void 0 ? void 0 : _b.el) === null || _c === void 0 ? void 0 : _c.contains(e.target))) {
                handleContentHide();
            }
        });
        // call emits
        const callShow = () => {
            emit('show', true);
        };
        const callHide = () => {
            emit('hide', false);
        };
        const callUpdateShow = () => {
            emit('update:show', showRef.value);
        };
        const callBorderReached = (flag, dirs) => {
            emit('border-reached', flag, dirs);
        };
        const throttleCallShow = (0, core_1.useThrottleFn)(callShow, 100, false);
        const throttleCallHide = (0, core_1.useThrottleFn)(callHide, 100, false);
        const throttleCallUpdateShow = (0, core_1.useThrottleFn)(callUpdateShow, 100, false);
        // visible control
        const clearShowTimer = () => {
            window.clearTimeout(contentShowTimer.value);
            contentShowTimer.value = null;
        };
        const clearHideTimer = () => {
            window.clearTimeout(contentHideTimer.value);
            contentHideTimer.value = null;
        };
        const handleContentShow = () => {
            clearHideTimer();
            if (showRef.value)
                return;
            contentShowTimer.value = window.setTimeout(() => {
                showRef.value = true;
                emitThrottled.value ? throttleCallShow() : callShow();
                emitThrottled.value ? throttleCallUpdateShow() : callUpdateShow();
            }, showDelay.value);
        };
        const handleContentHide = () => {
            clearShowTimer();
            if (!showRef.value)
                return;
            contentHideTimer.value = window.setTimeout(() => {
                showRef.value = false;
                emitThrottled.value ? throttleCallHide() : callHide();
                emitThrottled.value ? throttleCallUpdateShow() : callUpdateShow();
            }, hideDelay.value);
        };
        const syncPosition = () => {
            var _a;
            // @ts-ignore
            (_a = followerRef.value) === null || _a === void 0 ? void 0 : _a.syncPosition();
        };
        // hover control
        const contentHoverControl = (0, vue_1.computed)(() => {
            if (trigger.value !== 'hover')
                return {};
            return {
                onMouseenter: handleContentShow,
                onMouseleave: handleContentHide
            };
        });
        // list of events
        const triggerEvent = (0, vue_1.computed)(() => {
            if (trigger.value === 'hover') {
                return {
                    onMouseenter: handleContentShow,
                    onMouseleave: handleContentHide
                };
            }
            else if (trigger.value === 'click') {
                return {
                    onClick: () => {
                        if (showRef.value) {
                            handleContentHide();
                        }
                        else {
                            handleContentShow();
                        }
                    }
                };
            }
            else if (trigger.value === 'follow') {
                return {
                    onMouseenter: () => {
                        mouseInFollowTrigger.value = true;
                    },
                    onMouseleave: () => {
                        mouseInFollowTrigger.value = false;
                    }
                };
            }
            return {};
        });
        // render vnodes
        const triggerVNode = (0, vue_1.computed)(() => {
            const firstDefaultVNode = (0, _utils_1.getSlotFirstVNode)(slots.default);
            if (!firstDefaultVNode)
                return null;
            const clonedVNode = (0, vue_1.cloneVNode)(firstDefaultVNode);
            const tempVNode = clonedVNode.type === vue_1.Text ? (0, vue_1.createVNode)('span', null, [clonedVNode]) : clonedVNode;
            if (disabled.value)
                return tempVNode;
            // event bind
            if (!tempVNode.props) {
                tempVNode.props = {};
            }
            else {
                tempVNode.props = Object.assign({}, tempVNode.props);
            }
            for (const [name, handler] of Object.entries(triggerEvent.value)) {
                const originalHandler = tempVNode.props[name];
                tempVNode.props[name] = originalHandler
                    ? (...args) => {
                        originalHandler(...args);
                        handler === null || handler === void 0 ? void 0 : handler(...args);
                    }
                    : handler;
            }
            return tempVNode;
        });
        const triggerEl = (0, vue_1.computed)(() => {
            var _b;
            return ((_b = triggerVNode.value) === null || _b === void 0 ? void 0 : _b.el) || null;
        });
        (0, vue_1.provide)(interface_1.popoverInjectionKey, contentElRef);
        void (0, vue_1.nextTick)(() => {
            // const { top, right, bottom, left, width, height } = useElementBounding(binderElRef);
            if (disabled.value || !triggerEl.value)
                return;
            // auto async popover position
            if (autoSync.value) {
                const { top, right, bottom, left } = (0, core_1.useElementBounding)(triggerEl.value);
                (0, vue_1.watch)([top, right, bottom, left], () => {
                    syncPosition();
                });
            }
            // follow control
            if (trigger.value === 'follow') {
                const { x, y, isOutside, elementHeight, elementWidth, elementX, elementY } = (0, core_1.useMouseInElement)(triggerEl.value);
                const isFirstEnter = (0, vue_1.ref)(true);
                const clickCallEvent = (evt) => {
                    if (showRef.value) {
                        handleContentHide();
                    }
                    else {
                        const { x, y } = evt;
                        followX.value = x;
                        followY.value = y;
                        handleContentShow();
                    }
                };
                const moveCallEvent = () => {
                    if (!isOutside.value && mouseInFollowTrigger.value) {
                        followX.value = x.value;
                        followY.value = y.value;
                        handleContentShow();
                        // No detection for the first entry
                        if (isFirstEnter.value) {
                            isFirstEnter.value = false;
                            return;
                        }
                        void (0, vue_1.nextTick)(() => {
                            if (wrapBoundary.value && contentElRef.value) {
                                let isReachBorder = false;
                                let reachedDir = [];
                                const contentRect = contentElRef.value.getBoundingClientRect();
                                const { x: contentX, y: contentY, width, height } = contentRect;
                                const cursorOffsetX = contentX - x.value;
                                const cursorOffsetY = contentY - y.value;
                                // top boundary detection
                                if (cursorOffsetY < 0 && Math.abs(cursorOffsetY) > elementY.value) {
                                    followY.value += Math.abs(cursorOffsetY);
                                    isReachBorder = true;
                                    reachedDir.push('top');
                                }
                                // right boundary detection
                                if (elementX.value + width + cursorOffsetX >= elementWidth.value) {
                                    followX.value -= width + cursorOffsetX;
                                    isReachBorder = true;
                                    reachedDir.push('right');
                                }
                                // bottom boundary detection
                                if (elementY.value + height + cursorOffsetY >= elementHeight.value) {
                                    followY.value -= height + cursorOffsetY;
                                    isReachBorder = true;
                                    reachedDir.push('bottom');
                                }
                                // left boundary detection
                                if (cursorOffsetX < 0 && Math.abs(cursorOffsetX) > elementX.value) {
                                    followX.value += Math.abs(cursorOffsetX);
                                    isReachBorder = true;
                                    reachedDir.push('left');
                                }
                                callBorderReached(isReachBorder, reachedDir);
                            }
                        });
                    }
                    else {
                        handleContentHide();
                        callBorderReached(false, []);
                        isFirstEnter.value = true;
                    }
                };
                const { pause, resume } = (0, core_1.pausableWatch)([x, y, isOutside], moveCallEvent);
                (0, vue_1.watch)(followMode, () => {
                    (0, vue_1.nextTick)(() => {
                        if (followMode.value === 'move') {
                            resume();
                            triggerEl.value.removeEventListener('click', clickCallEvent);
                        }
                        else {
                            pause();
                            triggerEl.value.addEventListener('click', clickCallEvent);
                        }
                    });
                }, {
                    immediate: true
                });
            }
        });
        expose({
            syncPosition,
            show: handleContentShow,
            hide: handleContentHide,
            el: contentElRef
        });
        return () => (0, _utils_1.createComponentVNode)(vueuc_1.VBinder, null, {
            default: () => [
                (0, _utils_1.createComponentVNode)(vueuc_1.VTarget, null, { default: () => triggerVNode.value }),
                (0, _utils_1.createComponentVNode)(vueuc_1.VFollower, {
                    ref: followerRef,
                    x: mergedX.value,
                    y: mergedY.value,
                    zIndex: zIndex === null || zIndex === void 0 ? void 0 : zIndex.value,
                    show: showRef.value,
                    enabled: showRef.value,
                    placement: placement.value,
                    width: matchTrigger.value ? 'target' : undefined,
                    teleportDisabled: teleportDisabled.value
                }, {
                    default: () => (0, _utils_1.createComponentVNode)(vue_1.Transition, {
                        name: 'mc-popover-fade',
                        appear: true
                    }, {
                        default: () => (0, _utils_1.createDirectives)('v-if', {
                            condition: !disabled.value,
                            node: () => {
                                var _b;
                                const { top = '', right = '', bottom = '', left = '' } = (_b = offset === null || offset === void 0 ? void 0 : offset.value) !== null && _b !== void 0 ? _b : {};
                                const mergedProps = (0, vue_1.mergeProps)(attrs, Object.assign({ ref_key: 'contentElRef', ref: contentElRef, class: ['mc-popover', { 'mc-popover--with-arrow': withArrow.value }], style: {
                                        '--popover-offset-top': top,
                                        '--popover-offset-right': right,
                                        '--popover-offset-bottom': bottom,
                                        '--popover-offset-left': left
                                    } }, contentHoverControl.value));
                                const tempVNode = (0, _utils_1.createElementVNode)('div', mergedProps, [
                                    title.value ? (0, vue_1.createVNode)('div', { class: 'mc-popover__title' }, [title.value]) : null,
                                    (0, _utils_1.propsMergeSlots)(props, slots, 'content'),
                                    withArrow.value ? (0, vue_1.createVNode)('div', { class: 'mc-popover__arrow' }) : null
                                ], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */);
                                return (0, _utils_1.createDirectives)(destroyWhenHide.value ? 'v-if' : 'v-show', {
                                    condition: showRef.value,
                                    node: tempVNode
                                });
                            }
                        }),
                        _: 2 /* SlotFlags.DYNAMIC */
                    })
                }, 8 /* PatchFlags.PROPS */, ['x', 'y', 'zIndex', 'show', 'enabled', 'placement', 'width', 'teleportDisabled'])
            ],
            _: 3 /* SlotFlags.FORWARDED */
        });
    }
});
