import { PropType, VNodeChild, Ref, InjectionKey, CSSProperties } from 'vue';
export declare type PopoverTrigger = 'hover' | 'click' | 'manual' | 'follow';
export declare type PopoverPlacement = 'top' | 'bottom' | 'left' | 'right' | 'top-start' | 'top-end' | 'left-start' | 'left-end' | 'right-start' | 'right-end' | 'bottom-start' | 'bottom-end';
export declare type PopoverTriggerBorder = 'top' | 'right' | 'bottom' | 'left';
export declare type PopoverFollowMode = 'move' | 'click';
export declare type PopoverOffset = {
    top?: CSSProperties['top'];
    right?: CSSProperties['right'];
    bottom?: CSSProperties['bottom'];
    left?: CSSProperties['left'];
};
export declare type PopoverInjection = Ref<HTMLElement | null> | null;
export interface PopoverExposeInstance {
    syncPosition: () => void;
    show: () => void;
    hide: () => void;
    el: HTMLElement;
}
export interface PopoverProps {
    trigger?: PopoverTrigger;
    placement?: PopoverPlacement;
    destroyWhenHide?: boolean;
    zIndex?: number;
    show?: boolean;
    disabled?: boolean;
    withArrow?: boolean;
    showDelay?: number;
    hideDelay?: number;
    offset?: PopoverOffset;
    wrapBoundary?: boolean;
    matchTrigger?: boolean;
    autoSync?: boolean;
    title?: string;
    content?: string | (() => VNodeChild);
    followMode?: PopoverFollowMode;
    x?: number;
    y?: number;
    teleport?: boolean;
}
export declare const popoverProps: {
    trigger: {
        type: PropType<PopoverTrigger | undefined>;
        default: string;
    };
    placement: {
        type: PropType<PopoverPlacement | undefined>;
        default: string;
    };
    destroyWhenHide: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    zIndex: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    show: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    disabled: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    withArrow: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    showDelay: {
        type: PropType<number | undefined>;
        default: number;
    };
    hideDelay: {
        type: PropType<number | undefined>;
        default: number;
    };
    offset: {
        type: PropType<PopoverOffset | undefined>;
        default: undefined;
    };
    wrapBoundary: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    matchTrigger: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    autoSync: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
    title: {
        type: PropType<string | undefined>;
        default: undefined;
    };
    content: {
        type: PropType<string | (() => VNodeChild) | undefined>;
        default: undefined;
    };
    followMode: {
        type: PropType<PopoverFollowMode | undefined>;
        default: string;
    };
    x: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    y: {
        type: PropType<number | undefined>;
        default: undefined;
    };
    teleport: {
        type: PropType<boolean | undefined>;
        default: boolean;
    };
};
export declare const popoverEmits: string[];
export declare const popoverInjectionKey: InjectionKey<PopoverInjection>;
