"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const ionicons5_1 = require("@vicons/ionicons5");
const core_1 = require("@vueuse/core");
const lodash_es_1 = require("lodash-es");
const popover_1 = require("../popover");
const interface_1 = require("../popover/interface");
const icon_1 = require("../icon");
const interface_2 = require("./interface");
const styles_1 = require("./styles");
const defaultPropsOverride = {
    placement: {
        type: String,
        default: 'bottom'
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'Popselect',
    props: Object.assign(Object.assign(Object.assign({}, interface_1.popoverProps), interface_2.popselectProps), defaultPropsOverride),
    emits: [...interface_1.popoverEmits, ...interface_2.popselectEmits],
    setup(props, { slots, attrs, emit }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Popselect',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const { value: valueVM, options, multiple, maxHeight, autoClose, autoScroll, truncate, matchTrigger, itemHeight, itemStyle } = (0, vue_1.toRefs)(props);
        const popoverRef = (0, vue_1.ref)();
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--popselect-inner-max-width': typeof truncate.value === 'number' ? `${truncate.value}px` : '200px'
            };
        });
        let scrollToOption;
        const handleShow = () => {
            (0, vue_1.nextTick)(() => {
                var _a;
                const index = (_a = options.value) === null || _a === void 0 ? void 0 : _a.findIndex(e => e.value === (multiple.value ? valueVM.value[0] : valueVM.value));
                index && scrollToOption(index);
            });
        };
        const handleHide = () => {
            if ((autoClose.value === undefined && !multiple.value) || autoClose.value) {
                requestAnimationFrame(() => {
                    var _a;
                    (_a = popoverRef === null || popoverRef === void 0 ? void 0 : popoverRef.value) === null || _a === void 0 ? void 0 : _a.hide();
                });
            }
        };
        const createOptionVNode = (data) => {
            var _a;
            const { label, value, disabled } = data;
            const isDisabled = !!disabled;
            const isSelected = multiple.value ? valueVM.value.includes(value) : valueVM.value === value;
            const checkIconVNode = multiple.value && isSelected ? (0, _utils_1.createComponentVNode)(icon_1.McIcon, { size: 16, style: 'margin-left: 8px' }, { default: () => (0, _utils_1.createComponentVNode)(ionicons5_1.CheckmarkSharp) }) : null;
            const option = (0, vue_1.toRaw)((_a = options.value) === null || _a === void 0 ? void 0 : _a.find(e => e.value === value));
            const handleUpdateValue = (value) => {
                if (multiple.value) {
                    const index = valueVM.value.indexOf(value);
                    if (index === -1) {
                        valueVM.value.push(value);
                    }
                    else {
                        valueVM.value.splice(index, 1);
                    }
                    emit('update:value', (0, vue_1.toRaw)(valueVM.value), option);
                }
                else {
                    emit('update:value', value, option);
                }
            };
            return (0, _utils_1.createElementVNode)('div', {
                class: ['mc-popselect-option', { 'mc-popselect-option--selected': isSelected, 'mc-popselect-option--disabled': isDisabled }],
                style: itemStyle.value,
                onClick: () => {
                    if (isDisabled)
                        return;
                    valueVM.value !== void 0 && handleUpdateValue(value);
                    emit('select', value, option);
                    handleHide();
                }
            }, [
                (0, _utils_1.createElementVNode)('div', {
                    class: 'mc-popselect-option__inner'
                }, [(0, _utils_1.createElementVNode)('div', { class: { truncate: truncate.value } }, [typeof label === 'string' ? label : label()], 2 /* PatchFlags.CLASS */), checkIconVNode])
            ], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */);
        };
        return () => {
            var _a;
            const subHeight = itemHeight.value;
            const listHeight = options.value ? Math.min(options.value.length * subHeight, (_a = maxHeight.value) !== null && _a !== void 0 ? _a : 0) : 0;
            const { list, containerProps, wrapperProps, scrollTo } = (0, core_1.useVirtualList)((0, vue_1.isReactive)(options.value) ? options.value : (0, vue_1.reactive)(options.value), {
                // Keep `itemHeight` in sync with the item's row.
                itemHeight: subHeight
            });
            scrollToOption = scrollTo;
            const mergedProps = (0, vue_1.mergeProps)((0, lodash_es_1.omit)(props, Object.keys(interface_2.popselectProps)), {
                ref_key: 'popoverRef',
                ref: popoverRef,
                class: 'mc-popselect',
                style: Object.assign({ padding: '0px', minWidth: matchTrigger.value ? 'none' : '110px' }, cssVars.value)
            });
            return (0, _utils_1.createComponentVNode)(popover_1.McPopover, Object.assign(Object.assign({}, mergedProps), { onShow: (...args) => {
                    autoScroll.value && handleShow();
                    attrs.show && attrs.onShow(...args);
                } }), {
                default: () => (0, vue_1.renderSlot)(slots, 'default'),
                content: () => (0, _utils_1.createElementVNode)('div', Object.assign(Object.assign({}, containerProps), { class: 'mc-popselect__content', style: { height: listHeight + 'px', overflow: 'auto', padding: '4px 4px 0px 4px' } }), [
                    (0, _utils_1.createElementVNode)('div', wrapperProps.value, (0, _utils_1.createDirectives)('v-for', list.value, item => createOptionVNode(item.data)), 4 /* PatchFlags.STYLE */)
                ], 4 /* PatchFlags.STYLE */)
            }, 16 /* PatchFlags.FULL_PROPS */);
        };
    }
});
