"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PopupProvider = void 0;
const vue_1 = require("vue");
const modal_1 = require("../modal");
const drawer_1 = require("../drawer");
class PopupProvider {
    constructor(rootComponent) {
        Object.defineProperty(this, "app", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "rootComponent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "hostElement", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "plugins", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        this.rootComponent = rootComponent;
        this.app = (0, vue_1.createApp)(this.rootComponent);
        this.hostElement = document.createElement('div');
    }
    registerPlugins(plugins = []) {
        this.plugins = [...this.plugins, ...plugins];
    }
    mount() {
        if (!this.app || !this.hostElement)
            return;
        // register plugins
        [...this.plugins, ...PopupProvider._plugins].forEach(plugin => this.app.use(plugin));
        // mount app
        this.app.mount(this.hostElement);
    }
    unmount() {
        var _a;
        (_a = this.app) === null || _a === void 0 ? void 0 : _a.unmount();
        this.app = null;
        this.hostElement = null;
        this.rootComponent = null;
    }
    static use(plugin) {
        PopupProvider._plugins.push(plugin);
        return PopupProvider;
    }
}
exports.PopupProvider = PopupProvider;
Object.defineProperty(PopupProvider, "_plugins", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: []
});
function McPopup(source, options = {}) {
    let PopupApp = null;
    const instance = (0, vue_1.ref)();
    const visible = (0, vue_1.ref)(false);
    const destroy = () => {
        instance.value = null;
        PopupApp === null || PopupApp === void 0 ? void 0 : PopupApp.unmount();
        PopupApp = null;
    };
    const createDefaultVNode = () => {
        const { props: sourceProps = {}, on: sourceEmits = {} } = options;
        const props = {};
        const events = {};
        for (const [name, handler] of Object.entries(sourceEmits)) {
            events[`on${name.charAt(0).toUpperCase()}${name.slice(1)}`] = handler;
        }
        for (const [name, value] of Object.entries(sourceProps)) {
            props[name] = (0, vue_1.isRef)(value) ? value.value : value;
        }
        return (0, vue_1.createVNode)(typeof source === 'string' ? (0, vue_1.defineComponent)({ template: source }) : source, Object.assign(Object.assign({}, props), events));
    };
    const modalVNode = (props = {}, ctx) => {
        var _a;
        const { footer, header } = (_a = props.slots) !== null && _a !== void 0 ? _a : {};
        return (0, vue_1.createVNode)(modal_1.McModal, Object.assign(Object.assign({ ref: instance }, props), { show: visible.value, 'onUpdate:show': (value) => {
                var _a;
                visible.value = value;
                (_a = props['onUpdate:show']) === null || _a === void 0 ? void 0 : _a.call(props, value);
            }, onAfterLeave: () => {
                var _a, _b;
                ((_a = options.autoDestroy) !== null && _a !== void 0 ? _a : true) && destroy();
                (_b = props['onAfterLeave']) === null || _b === void 0 ? void 0 : _b.call(props);
            }, appearFromCursor: props.appearFromCursor || false }), {
            header,
            default: createDefaultVNode,
            footer
        });
    };
    const drawerVNode = props => {
        var _a;
        const { header } = (_a = props.slots) !== null && _a !== void 0 ? _a : {};
        return (0, vue_1.createVNode)(drawer_1.McDrawer, Object.assign(Object.assign({ ref: instance }, props), { show: visible.value, 'onUpdate:show': (value) => {
                var _a;
                visible.value = value;
                (_a = props['onUpdate:show']) === null || _a === void 0 ? void 0 : _a.call(props, value);
            }, onAfterLeave: () => {
                var _a, _b;
                ((_a = options.autoDestroy) !== null && _a !== void 0 ? _a : true) && destroy();
                (_b = props['onAfterLeave']) === null || _b === void 0 ? void 0 : _b.call(props);
            } }), {
            header,
            default: createDefaultVNode
        });
    };
    return {
        instance,
        show(maybePopupConfig, config = {}) {
            if (instance.value === null) {
                throw new Error('[McPopup]: Current popup instance has been destroyed.');
            }
            visible.value = true;
            if (!PopupApp) {
                const { plugins = [] } = options;
                PopupApp = new PopupProvider(typeof maybePopupConfig === 'string' ? (0, vue_1.createVNode)(maybePopupConfig === 'modal' ? modalVNode : drawerVNode, Object.assign({}, config)) : (0, vue_1.createVNode)(modalVNode, Object.assign({}, (maybePopupConfig !== null && maybePopupConfig !== void 0 ? maybePopupConfig : {}))));
                PopupApp.registerPlugins(plugins);
                PopupApp.mount();
            }
        },
        hide() {
            var _a;
            (_a = instance.value) === null || _a === void 0 ? void 0 : _a.hide();
        },
        destroy
    };
}
exports.default = McPopup;
