"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const _utils_1 = require("../_utils_");
const Splitter_1 = __importDefault(require("./Splitter"));
const interface_1 = require("./interface");
exports.default = (0, vue_1.defineComponent)({
    name: 'Split',
    iKey: interface_1.SplitIKey,
    props: {
        horizontal: {
            type: Boolean,
            default: false
        }
    },
    setup(props, { slots }) {
        const { horizontal } = (0, vue_1.toRefs)(props);
        const splitElRef = (0, vue_1.ref)();
        const { width } = (0, core_1.useElementBounding)(splitElRef);
        const splitPaneInitialWidth = (0, vue_1.ref)('0');
        const cssVars = (0, vue_1.computed)(() => {
            return {
                '--split-pane-initial-width': `${splitPaneInitialWidth.value}%`
            };
        });
        const splitsVNode = (0, vue_1.computed)(() => {
            const splitPanes = slots.default ? (0, _utils_1.flatten)(slots.default(), interface_1.SplitPaneIKey) : null;
            if (!splitPanes)
                return null;
            splitPaneInitialWidth.value = (100 / splitPanes.length).toFixed(1);
            const splitters = new Array(splitPanes.length - 1).fill(null).map(() => (0, vue_1.createVNode)(Splitter_1.default));
            return new Array(2 * splitPanes.length - 1).fill(null).map((e, index) => {
                if (index % 2 === 0)
                    return splitPanes[index >> 1];
                return splitters[(index - 1) >> 1];
            });
        });
        const ResizeEventBusKey = Symbol();
        const BusResize = (0, core_1.useEventBus)(ResizeEventBusKey);
        (0, vue_1.provide)(interface_1.splitInjectionKey, {
            parentWidth: width,
            splitPaneInitialWidth,
            BusResize
        });
        (0, vue_1.onUnmounted)(() => {
            BusResize.reset();
        });
        return () => (0, vue_1.createVNode)('div', {
            ref: splitElRef,
            class: ['mc-split', horizontal.value ? 'mc-split--horizontal' : 'mc-split--vertical'],
            style: cssVars.value
        }, [splitsVNode.value]);
    }
});
