"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const _utils_1 = require("../_utils_");
const _internal_1 = require("../_internal_");
const _transition_1 = require("../_transition_");
const styles_1 = require("./styles");
const interface_1 = require("./interface");
const SIZE_MAP = {
    small: {
        labelHeight: '18px',
        labelMinWidth: '36px',
        handlerSize: '14px',
        fontSize: '12px',
        textCheckedPadding: '0px 22px 0px 6px',
        textUncheckedPadding: '0px 6px 0px 22px'
    },
    medium: {
        labelHeight: '22px',
        labelMinWidth: '44px',
        handlerSize: '18px',
        fontSize: '14px',
        textCheckedPadding: '0px 26px 0px 10px',
        textUncheckedPadding: '0px 10px 0px 26px'
    },
    large: {
        labelHeight: '26px',
        labelMinWidth: '52px',
        handlerSize: '22px',
        fontSize: '16px',
        textCheckedPadding: '0px 30px 0px 14px',
        textUncheckedPadding: '0px 14px 0px 30px'
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'Switch',
    props: interface_1.switchProps,
    emits: ['update:value', 'switch'],
    setup(props, { slots, emit, expose }) {
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Switch',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const key = (0, _utils_1.createKey)('switch');
        const { value: valueVM, disabled, size, checkedValue, uncheckedValue, checkedText, uncheckedText, checkedColor, uncheckedColor, handlerColor, textPlacement, square, checked, onBeforeSwitch, loading, inelastic, width } = (0, vue_1.toRefs)(props);
        const switchElRef = (0, vue_1.ref)();
        const internalValue = (0, vue_1.ref)(checked.value);
        const mergedValue = (0, core_1.isDefined)(valueVM) ? valueVM : internalValue;
        const isChecked = (0, vue_1.computed)(() => mergedValue.value === checkedValue.value);
        const showInnerText = (0, vue_1.computed)(() => textPlacement.value === 'both' || textPlacement.value === 'in');
        const showOuterText = (0, vue_1.computed)(() => textPlacement.value === 'both' || textPlacement.value === 'out');
        const showCheckedText = (0, vue_1.computed)(() => !!(checkedText.value || slots['checked-text'] || slots['checkedText']));
        const showUncheckedText = (0, vue_1.computed)(() => !!(uncheckedText.value || slots['unchecked-text'] || slots['uncheckedText']));
        const cssVars = (0, vue_1.computed)(() => {
            var _a;
            const { labelHeight, labelMinWidth, handlerSize, fontSize, textUncheckedPadding, textCheckedPadding } = (_a = SIZE_MAP[size.value]) !== null && _a !== void 0 ? _a : SIZE_MAP.medium;
            return {
                '--switch-checked-color': checkedColor.value,
                '--switch-unchecked-color': uncheckedColor.value,
                '--switch-handler-color': handlerColor.value,
                '--switch-ripple-color': (0, _utils_1.setColorAlpha)(checkedColor.value, 0),
                '--switch-font-size': fontSize,
                '--switch-label-height': labelHeight,
                '--switch-label-width': (0, _utils_1.cssUnitTransform)(width.value),
                '--switch-label-min-width': width.value ? (0, _utils_1.cssUnitTransform)(width.value) : labelMinWidth,
                '--switch-label-border-radius': square.value ? '3px' : labelHeight,
                '--switch-text-checked-padding': textCheckedPadding,
                '--switch-text-unchecked-padding': textUncheckedPadding,
                '--switch-handler-size': handlerSize,
                '--switch-handler-border-radius': square.value ? '3px' : handlerSize
            };
        });
        const callUpdateValue = () => {
            const value = isChecked.value ? uncheckedValue.value : checkedValue.value;
            if ((0, core_1.isDefined)(valueVM)) {
                emit('update:value', value);
            }
            else {
                mergedValue.value = value;
            }
            emit('switch', value);
        };
        const handleChange = () => __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (disabled.value)
                return;
            try {
                const callback = yield ((_a = onBeforeSwitch.value) === null || _a === void 0 ? void 0 : _a.call(onBeforeSwitch));
                if (callback || callback === undefined) {
                    callUpdateValue();
                }
            }
            catch (error) { }
        });
        const innerCheckedText = () => {
            return (0, core_1.and)(isChecked, showCheckedText).value ? (0, _utils_1.createElementVNode)('span', { class: 'mc-switch-label__content' }, (0, _utils_1.propsMergeSlots)(props, slots, 'checkedText')) : null;
        };
        const innerUncheckedText = () => {
            return (0, core_1.and)((0, core_1.not)(isChecked), showUncheckedText).value ? (0, _utils_1.createElementVNode)('span', { class: 'mc-switch-label__content' }, (0, _utils_1.propsMergeSlots)(props, slots, 'uncheckedText')) : null;
        };
        expose({
            el: switchElRef.value,
            switch: callUpdateValue
        });
        // main logic...
        return () => {
            var _a;
            return (0, _utils_1.createElementVNode)('div', {
                ref_key: 'switchElRef',
                ref: switchElRef,
                class: [
                    'mc-switch',
                    `mc-switch--${size.value}`,
                    {
                        'mc-switch--checked': isChecked.value,
                        'mc-switch--disabled': disabled.value,
                        'mc-switch--square': square.value,
                        'mc-switch--loading': loading.value,
                        'mc-switch--inelastic': inelastic.value
                    }
                ],
                style: cssVars.value
            }, [
                (0, _utils_1.createElementVNode)('input', { class: 'mc-switch-input', value: mergedValue.value, id: key, type: 'checkbox', checked: isChecked.value, disabled: disabled.value, onChange: handleChange }, null, 8 /* PatchFlags.PROPS */, [
                    'value',
                    'checked',
                    'disabled'
                ]),
                (0, core_1.and)(showUncheckedText, showOuterText).value ? (0, _utils_1.createElementVNode)('span', { class: 'mc-switch-label-text--left', onClick: handleChange }, [(0, _utils_1.propsMergeSlots)(props, slots, 'uncheckedText')]) : null,
                (0, _utils_1.createElementVNode)('label', { class: 'mc-switch-label', for: key }, [
                    showInnerText.value ? (_a = innerCheckedText()) !== null && _a !== void 0 ? _a : innerUncheckedText() : null,
                    (0, _utils_1.createElementVNode)('div', { class: 'mc-switch-label__handler' }, [
                        (0, _utils_1.createComponentVNode)(_transition_1.McIconSwitchTransition, null, {
                            default: () => {
                                if (loading.value) {
                                    return (0, _utils_1.createComponentVNode)(_internal_1.McBaseLoading, { size: 14 });
                                }
                                else if (slots.icon) {
                                    return (0, _utils_1.renderSlot)(slots, 'icon');
                                }
                                else {
                                    return isChecked.value ? (0, _utils_1.renderSlot)(slots, 'checked-icon') : (0, _utils_1.renderSlot)(slots, 'unchecked-icon');
                                }
                            }
                        })
                    ])
                ]),
                (0, core_1.and)(showCheckedText, showOuterText).value ? (0, _utils_1.createElementVNode)('span', { class: 'mc-switch-label-text--right', onClick: handleChange }, [(0, _utils_1.propsMergeSlots)(props, slots, 'checkedText')]) : null
            ], 2 /* PatchFlags.CLASS */ | 4 /* PatchFlags.STYLE */);
        };
    }
});
