"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const core_1 = require("@vueuse/core");
const interface_1 = require("./interface");
const Tab_1 = __importDefault(require("./src/Tab"));
const styles_1 = require("./styles");
exports.default = (0, vue_1.defineComponent)({
    name: 'Tabs',
    props: interface_1.tabsProps,
    emits: ['update:value', 'tab-switch', 'tab-click'],
    setup(props, { slots, emit, expose }) {
        var _a;
        // theme register
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'Tabs',
                main: styles_1.mainCssr,
                light: styles_1.lightCssr,
                dark: styles_1.darkCssr
            });
        });
        const { value: valueVM, defaultTab, type, showLine, center, stretch, tabGap, animation, activeColor, barPosition, headerStyle, headerClass, contentStyle, contentClass, onBeforeTabSwitch } = (0, vue_1.toRefs)(props);
        const activeTabName = (0, vue_1.ref)((valueVM === null || valueVM === void 0 ? void 0 : valueVM.value) || ((_a = defaultTab === null || defaultTab === void 0 ? void 0 : defaultTab.value) !== null && _a !== void 0 ? _a : ''));
        const activeTabVNode = (0, vue_1.ref)();
        const tabsElRef = (0, vue_1.ref)();
        const headerElRef = (0, vue_1.ref)();
        const barElRef = (0, vue_1.ref)();
        const barUpdatedTimer = (0, vue_1.ref)();
        const cssVars = (0, vue_1.computed)(() => {
            var _a;
            return {
                // '--tab-default-color': defaultColor.value,
                '--tab-active-color': activeColor.value,
                '--tab-gap': stretch.value ? 0 : ((_a = tabGap.value) !== null && _a !== void 0 ? _a : 0) / 2 + 'px'
            };
        });
        if (valueVM) {
            (0, vue_1.watch)(valueVM, name => {
                name && handleBeforeTabSwitch(name);
            });
        }
        if (type.value === 'bar' || type.value === 'segment') {
            if (animation.value === 'slide') {
                (0, vue_1.watch)(activeTabVNode, () => {
                    (0, vue_1.nextTick)(() => {
                        updateBarStyle();
                    });
                });
                const { top, right, width, height } = (0, core_1.useElementBounding)(headerElRef);
                (0, core_1.throttledWatch)([top, right, width, height], () => {
                    clearBarUpdatedTimer();
                    const { value: barEl } = barElRef;
                    barEl.style.transition = '0s';
                    updateBarStyle();
                }, {
                    throttle: 32
                });
            }
        }
        const callUpdateTab = (name) => {
            activeTabName.value = name;
            valueVM && emit('update:value', name);
            emit('tab-switch', name);
        };
        const handleTabClick = (name) => {
            emit('tab-click', name);
            handleBeforeTabSwitch(name);
        };
        const handleBeforeTabSwitch = (name) => __awaiter(this, void 0, void 0, function* () {
            // callback only when the value is changed
            if (activeTabName.value !== name) {
                if (onBeforeTabSwitch === null || onBeforeTabSwitch === void 0 ? void 0 : onBeforeTabSwitch.value) {
                    const { value: beforeTabSwitch } = onBeforeTabSwitch;
                    const callback = yield beforeTabSwitch(activeTabName.value, name);
                    if (callback || callback === undefined) {
                        callUpdateTab(name);
                    }
                }
                else {
                    callUpdateTab(name);
                }
            }
        });
        const clearBarUpdatedTimer = () => {
            window.clearTimeout(barUpdatedTimer.value);
            barUpdatedTimer.value = null;
        };
        const updateBarStyle = () => {
            if (!activeTabVNode.value)
                return;
            const { value: { el: activeTabEl } } = activeTabVNode;
            const { value: barEl } = barElRef;
            const { offsetWidth, offsetLeft } = activeTabEl;
            barEl.style.left = `${offsetLeft}px`;
            barEl.style.maxWidth = `${offsetWidth}px`;
            barUpdatedTimer.value = window.setTimeout(() => {
                barEl.style.transition = '0.2s';
            }, 64);
        };
        (0, vue_1.provide)(interface_1.tabsInjectionKey, activeTabName);
        const getTabVNode = (maybeTabPane) => {
            var _a, _b;
            const { children, props, type } = maybeTabPane;
            const { name, tabLabel = '', tabStyle = '', tabClass = '', disabled = false } = (_a = (0, _utils_1.kebabCaseEscape)(props)) !== null && _a !== void 0 ? _a : {};
            const isTab = type.iKey === interface_1.tabIKey;
            const isActive = activeTabName.value === name;
            const isDisabled = typeof disabled === 'boolean' ? disabled : disabled === '';
            const tabVNode = (0, vue_1.createVNode)(Tab_1.default, (0, vue_1.mergeProps)(isTab ? (_b = maybeTabPane.props) !== null && _b !== void 0 ? _b : {} : { style: tabStyle }, {
                class: [{ 'mc-tabs-tab--active': isActive, 'mc-tabs-tab--disabled': isDisabled }, tabClass],
                onClick: () => {
                    if (isDisabled)
                        return;
                    name !== undefined && handleTabClick(name);
                }
            }), {
                default: () => {
                    var _a, _b, _c, _d;
                    if (isTab) {
                        return (_b = (_a = children === null || children === void 0 ? void 0 : children.default) === null || _a === void 0 ? void 0 : _a.call(children)) !== null && _b !== void 0 ? _b : null;
                    }
                    else {
                        return (_d = (_c = children === null || children === void 0 ? void 0 : children.tab) === null || _c === void 0 ? void 0 : _c.call(children)) !== null && _d !== void 0 ? _d : [(0, vue_1.createVNode)('span', { class: 'mc-tabs-tab__label' }, [typeof tabLabel === 'string' ? (0, vue_1.createTextVNode)(tabLabel) : tabLabel()])];
                    }
                }
            });
            if (isActive) {
                activeTabVNode.value = tabVNode;
            }
            return tabVNode;
        };
        const lineBarVNode = (0, vue_1.computed)(() => {
            if (type.value === 'bar') {
                if (animation.value === 'slide') {
                    return (0, vue_1.createVNode)('div', {
                        ref: barElRef,
                        class: ['mc-tabs__header-bar', `mc-tabs__header-bar--${barPosition.value}`]
                    });
                }
            }
            else if (type.value === 'segment') {
                if (animation.value === 'slide') {
                    return (0, vue_1.createVNode)('div', {
                        ref: barElRef,
                        class: ['mc-tabs__header-bar']
                    });
                }
            }
            return null;
        });
        const tabsHeaderVNode = (0, vue_1.computed)(() => {
            var _a, _b;
            // use IKey, ensure non-SpecificVNode element won't be rendered in header
            const [firstTabPane, maybeTabPanes] = (0, _utils_1.getSlotFirstVNode)(slots.default, [interface_1.tabPaneIKey, interface_1.tabIKey], true);
            if (!maybeTabPanes || maybeTabPanes.length === 0)
                return null;
            activeTabName.value = activeTabName.value || ((_b = (_a = firstTabPane === null || firstTabPane === void 0 ? void 0 : firstTabPane.props) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : '');
            return (0, vue_1.createVNode)('div', {
                class: ['mc-tabs__header-scroll-content', headerClass === null || headerClass === void 0 ? void 0 : headerClass.value],
                style: headerStyle === null || headerStyle === void 0 ? void 0 : headerStyle.value
            }, maybeTabPanes.map(maybeTabPane => getTabVNode(maybeTabPane)));
        });
        expose({
            switchTo: handleBeforeTabSwitch,
            el: tabsElRef
        });
        return () => (0, vue_1.createVNode)('div', {
            ref: tabsElRef,
            class: ['mc-tabs', `mc-tabs--${type.value}`],
            style: cssVars.value
        }, [
            (0, vue_1.createVNode)('div', {
                ref: headerElRef,
                class: [
                    'mc-tabs__header',
                    {
                        'mc-tabs__header--center': center.value,
                        'mc-tabs__header--stretch': stretch.value,
                        'mc-tabs__header--with-line': type.value === 'segment' || type.value === 'empty' ? false : showLine.value
                    },
                    type.value === 'bar' || type.value === 'segment' ? `mc-tabs__header--bar-${animation.value}` : ''
                ]
            }, [lineBarVNode.value, tabsHeaderVNode.value]),
            slots.default && (0, _utils_1.flatten)(slots.default(), interface_1.tabPaneIKey).length > 0
                ? (0, vue_1.createVNode)('div', {
                    class: ['mc-tabs__content', contentClass === null || contentClass === void 0 ? void 0 : contentClass.value],
                    style: contentStyle.value
                }, (0, _utils_1.flatten)(slots.default(), interface_1.tabIKey, true))
                : null
        ]);
    }
});
