"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const interface_1 = require("../interface");
exports.default = (0, vue_1.defineComponent)({
    name: 'TabPane',
    iKey: interface_1.tabPaneIKey,
    props: interface_1.tabPaneProps,
    setup(props, { slots }) {
        const { name, preload, lazy } = (0, vue_1.toRefs)(props);
        const valueRef = (0, vue_1.inject)(interface_1.tabsInjectionKey, null);
        if (!valueRef) {
            throw new Error('[McTabPane]: McTabPane must be placed inside McTabs.');
        }
        const isActive = (0, vue_1.computed)(() => {
            return (name === null || name === void 0 ? void 0 : name.value) === (valueRef === null || valueRef === void 0 ? void 0 : valueRef.value);
        });
        const hasRendered = (0, vue_1.ref)(isActive.value);
        if (lazy.value && !hasRendered.value) {
            (0, core_1.watchOnce)(isActive, () => {
                hasRendered.value = true;
            });
        }
        return () => {
            const tabPaneVNode = (0, vue_1.createVNode)('div', { class: 'mc-tab-pane' }, [(0, vue_1.renderSlot)(slots, 'default')]);
            if (preload.value) {
                return (0, vue_1.withDirectives)(tabPaneVNode, [[vue_1.vShow, isActive.value]]);
            }
            else {
                if (lazy.value) {
                    return hasRendered.value ? (0, vue_1.withDirectives)(tabPaneVNode, [[vue_1.vShow, isActive.value]]) : (0, vue_1.createCommentVNode)('v-if', true);
                }
                else {
                    return isActive.value ? tabPaneVNode : (0, vue_1.createCommentVNode)('v-if', true);
                }
            }
        };
    }
});
