"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
const _utils_1 = require("../_utils_");
const interface_1 = require("./interface");
const index_1 = require("./styles/index");
const BASE_COLOR_MAP = {
    primary: {
        color: '#3b82f6'
    },
    success: {
        color: '#16a34a'
    },
    warning: {
        color: '#fb923c'
    },
    danger: {
        color: '#dc2626'
    },
    info: {
        color: '#6B7280'
    }
};
exports.default = (0, vue_1.defineComponent)({
    name: 'TextLink',
    props: interface_1.textLinkProps,
    setup(props, { slots }) {
        (0, vue_1.onMounted)(() => {
            (0, _utils_1.useThemeRegister)({
                key: 'TextLink',
                main: index_1.mainCssr
            });
        });
        const { type, to, underline, trigger, color, hoverColor, block, raw } = (0, vue_1.toRefs)(props);
        const showUnderline = (0, vue_1.computed)(() => {
            if (!underline.value)
                return '';
            switch (trigger.value) {
                case 'always':
                    return 'underline';
                case 'hover':
                    return 'underline-hover';
                default:
                    return '';
            }
        });
        const cssVars = (0, vue_1.computed)(() => {
            const compositeInputColor = {
                color: color.value || BASE_COLOR_MAP[type.value].color
            };
            const { default: defaultColorSet, hover: hoverColorSet, active: activeColorSet, disabled: disabledColorSet } = (0, _utils_1.useColorFactory)(compositeInputColor, {
                hover: [0, 0, 0, 0.12]
            });
            return {
                '--text-link-default-color': color.value || defaultColorSet.color,
                '--text-link-hover-color': hoverColor.value || hoverColorSet.color,
                '--text-link-active-color': activeColorSet.color,
                '--text-link-disabled-color': disabledColorSet.color
            };
        });
        return () => {
            var _a, _b;
            const textContent = (_b = (_a = (0, _utils_1.getSlotFirstVNode)(slots.default)) === null || _a === void 0 ? void 0 : _a.children) !== null && _b !== void 0 ? _b : '';
            const isEmail = typeof textContent === 'string' && /^([A-Za-z0-9_\-\.])+\@([A-Za-z0-9_\-\.])+\.([A-Za-z]{2,4})$/.test(textContent);
            return (0, vue_1.createVNode)('a', {
                class: ['mc-text-link', { 'mc-text-link--block': block.value }, showUnderline.value],
                style: cssVars.value,
                href: isEmail ? (!raw.value ? `mailto:${textContent}` : '#') : to.value
            }, [(0, vue_1.renderSlot)(slots, 'default')]);
        };
    }
});
