"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useThemeController = exports.isDark = exports.isLight = exports.globalTheme = void 0;
const vue_1 = require("vue");
const core_1 = require("@vueuse/core");
const _utils_1 = require("./_utils_");
const globalThemeState = (0, core_1.createGlobalState)(() => (0, core_1.useStorage)('meetcode-ui-theme-local-storage', 'light'));
exports.globalTheme = globalThemeState();
exports.isLight = (0, vue_1.computed)(() => exports.globalTheme.value === 'light');
exports.isDark = (0, vue_1.computed)(() => exports.globalTheme.value === 'dark');
/**
 * User's integrate theme controller
 */
function useThemeController(options = {}) {
    const { initialTheme, useOsTheme } = options;
    const themeChangeEventHook = (0, core_1.createEventHook)();
    const setTheme = (theme = 'light') => {
        if (exports.globalTheme.value !== theme) {
            exports.globalTheme.value = theme;
            themeChangeEventHook.trigger(theme);
        }
    };
    if (initialTheme) {
        setTheme(initialTheme);
    }
    else if (useOsTheme) {
        const isDark = (0, core_1.usePreferredDark)();
        (0, vue_1.watch)(isDark, colorScheme => {
            if (colorScheme) {
                setTheme('dark');
            }
            else {
                setTheme('light');
            }
        }, { immediate: true });
    }
    return {
        current: exports.globalTheme,
        isDark: exports.isDark,
        isLight: exports.isLight,
        setTheme,
        switchTheme() {
            if (exports.isLight.value) {
                setTheme('dark');
            }
            else {
                setTheme('light');
            }
        },
        onThemeChange: themeChangeEventHook.on,
        defineTheme: _utils_1.defineTheme
    };
}
exports.useThemeController = useThemeController;
